@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/align.c,v 4.21 2002/01/05 02:47:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/align.c,v 4.23 2003/02/25 01:17:42 william Exp $
  */
 
 #define _INCLUDE_FROM_ALIGN_C_
@@ -333,12 +333,11 @@ void DistrSelObjs()
          Msg(gszMsgBox);
       }
    } else {
-      struct SelRec *left_sel, *right_sel;
-      struct SelRec *top_sel, *bottom_sel;
-      struct SelRec *vert_sel=NULL, *hori_sel=NULL, *new_sel;
-      struct MoveSubCmdRec *move_cmd;
-      struct SubCmdRec *sub_cmd;
-      struct SelRec *tmp_sel_ptr;
+      struct SelRec *left_sel=NULL, *right_sel=NULL;
+      struct SelRec *top_sel=NULL, *bottom_sel=NULL;
+      struct SelRec *vert_sel=NULL, *hori_sel=NULL, *new_sel=NULL;
+      struct SubCmdRec *sub_cmd=NULL;
+      struct SelRec *tmp_sel_ptr=NULL;
 
       if (topSel==botSel || topSel->next==botSel) return;
 
@@ -346,13 +345,9 @@ void DistrSelObjs()
       if (tmp_sel_ptr == NULL) FailAllocMessage();
       tmp_sel_ptr->next = tmp_sel_ptr->prev = NULL;
 
-      move_cmd = (struct MoveSubCmdRec *)malloc(sizeof(struct MoveSubCmdRec));
-      if (move_cmd == NULL) FailAllocMessage();
-      memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
       sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
       if (sub_cmd == NULL) FailAllocMessage();
       memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-      sub_cmd->detail.mv = move_cmd;
 
       StartCompositeCmd();
       HighLightReverse();
@@ -540,8 +535,8 @@ void DistrSelObjs()
             break;
          }
          if (dx != 0) {
-            move_cmd->dx = dx;
-            move_cmd->dy = 0;
+            sub_cmd->detail.move.dx = dx;
+            sub_cmd->detail.move.dy = 0;
             tmp_sel_ptr->obj = sel_ptr->next->obj;
             PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
             RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
@@ -577,8 +572,8 @@ void DistrSelObjs()
             break;
          }
          if (dy != 0) {
-            move_cmd->dx = 0;
-            move_cmd->dy = dy;
+            sub_cmd->detail.move.dx = 0;
+            sub_cmd->detail.move.dy = dy;
             tmp_sel_ptr->obj = sel_ptr->next->obj;
             PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
             RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
@@ -597,7 +592,6 @@ void DistrSelObjs()
       free(bottom_sel);
 
       EndCompositeCmd();
-      free(move_cmd);
       free(sub_cmd);
       free(tmp_sel_ptr);
 
@@ -902,7 +896,6 @@ void AlignSelObjs()
       }
       EndCompositeCmd();
    } else {
-      struct MoveSubCmdRec *move_cmd=NULL;
       struct SubCmdRec *sub_cmd=NULL;
       struct SelRec *tmp_sel_ptr=NULL;
       struct ObjRec *locked_obj=NULL;
@@ -924,13 +917,9 @@ void AlignSelObjs()
       if (tmp_sel_ptr == NULL) FailAllocMessage();
       tmp_sel_ptr->next = tmp_sel_ptr->prev = NULL;
 
-      move_cmd = (struct MoveSubCmdRec *)malloc(sizeof(struct MoveSubCmdRec));
-      if (move_cmd == NULL) FailAllocMessage();
-      memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
       sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
       if (sub_cmd == NULL) FailAllocMessage();
       memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-      sub_cmd->detail.mv = move_cmd;
 
       StartCompositeCmd();
       HighLightReverse();
@@ -987,8 +976,8 @@ void AlignSelObjs()
                MoveObj(obj_ptr, dx, dy);
                RecordReplaceAnObj(obj_ptr);
             } else {
-               move_cmd->dx = dx;
-               move_cmd->dy = dy;
+               sub_cmd->detail.move.dx = dx;
+               sub_cmd->detail.move.dy = dy;
                tmp_sel_ptr->obj = obj_ptr;
                PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
                RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
@@ -1001,7 +990,6 @@ void AlignSelObjs()
       }
       EndCompositeCmd();
 
-      free(move_cmd);
       free(sub_cmd);
       free(tmp_sel_ptr);
 
@@ -1123,19 +1111,14 @@ void AlignSelToPage()
       case ALIGN_B: dy = paperHeight - selRbY; break;
       }
       if (dx != 0 || dy != 0) {
-         struct MoveSubCmdRec *move_cmd;
-         struct SubCmdRec *sub_cmd;
+         struct SubCmdRec *sub_cmd=NULL;
 
-         move_cmd = (struct MoveSubCmdRec*)malloc(sizeof(struct MoveSubCmdRec));
-         if (move_cmd == NULL) FailAllocMessage();
-         memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
          sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
          if (sub_cmd == NULL) FailAllocMessage();
          memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-         sub_cmd->detail.mv = move_cmd;
 
-         move_cmd->dx = dx;
-         move_cmd->dy = dy;
+         sub_cmd->detail.move.dx = dx;
+         sub_cmd->detail.move.dy = dy;
 
          PrepareToRecord(CMD_MOVE, topSel, botSel, numObjSelected);
          RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
@@ -1144,7 +1127,6 @@ void AlignSelToPage()
                MoveObj(sel_ptr->obj, dx, dy);
             }
          }
-         free(move_cmd);
          free(sub_cmd);
       }
    }
@@ -1253,21 +1235,16 @@ void AlignSelToGrid()
          }
       }
    } else {
-      struct MoveSubCmdRec *move_cmd;
-      struct SubCmdRec *sub_cmd;
-      struct SelRec *tmp_sel_ptr;
+      struct SubCmdRec *sub_cmd=NULL;
+      struct SelRec *tmp_sel_ptr=NULL;
 
       tmp_sel_ptr = (struct SelRec *)malloc(sizeof(struct SelRec));
       if (tmp_sel_ptr == NULL) FailAllocMessage();
       tmp_sel_ptr->next = tmp_sel_ptr->prev = NULL;
 
-      move_cmd = (struct MoveSubCmdRec *)malloc(sizeof(struct MoveSubCmdRec));
-      if (move_cmd == NULL) FailAllocMessage();
-      memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
       sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
       if (sub_cmd == NULL) FailAllocMessage();
       memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-      sub_cmd->detail.mv = move_cmd;
 
       for (sel_ptr = topSel; sel_ptr != NULL; sel_ptr = sel_ptr->next) {
          obj_ptr = sel_ptr->obj;
@@ -1315,8 +1292,8 @@ void AlignSelToGrid()
                MoveObj(obj_ptr, dx, dy);
                RecordReplaceAnObj(obj_ptr);
             } else {
-               move_cmd->dx = dx;
-               move_cmd->dy = dy;
+               sub_cmd->detail.move.dx = dx;
+               sub_cmd->detail.move.dy = dy;
                tmp_sel_ptr->obj = obj_ptr;
                PrepareToRecord(CMD_MOVE, tmp_sel_ptr, tmp_sel_ptr, 1);
                RecordCmd(CMD_MOVE, sub_cmd, NULL, NULL, 0);
@@ -1327,7 +1304,6 @@ void AlignSelToGrid()
             RecordReplaceAnObj(obj_ptr);
          }
       }
-      free(move_cmd);
       free(sub_cmd);
       free(tmp_sel_ptr);
    }
