@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.30 2002/01/05 02:47:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/arc.c,v 4.32 2003/07/22 23:28:16 william Exp $
  */
 
 #define _INCLUDE_FROM_ARC_C_
@@ -2304,7 +2304,7 @@ void MakePreciseArc()
    obj_ptr = CreateArcObj(0, 0, x1, y1, x2, y2, dir, -r, -r, r*2, r*2,
          theta1*64, angle2*64, FALSE);
 
-   PlaceTopObj(obj_ptr);
+   PlaceTopObj(obj_ptr, NULL, NULL);
    SelectTopObj();
    RecordNewObjCmd();
    SetFileModified(TRUE);
