@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.37 2002/01/05 02:47:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/attr.c,v 4.39 2003/02/25 01:17:42 william Exp $
  */
 
 #define _INCLUDE_FROM_ATTR_C_
@@ -1865,8 +1865,12 @@ void AddFileAttrs()
    int ltx, lty, rbx, rby;
 
    if (gstWBInfo.do_whiteboard) {
-      MsgBox(TgLoadString(STID_ATTACH_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
-      return;
+      /*
+       * Now it's allowed.
+       *
+       * MsgBox(TgLoadString(STID_ATTACH_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
+       * return;
+       */
    }
    if (topSel == NULL) {
       MsgBox(TgLoadString(STID_SELECT_AT_LEAST_ONE_TEXT), TOOL_NAME, INFO_MB);
@@ -1911,8 +1915,12 @@ void DetachFileAttrs()
    struct SelRec *sel_ptr;
 
    if (gstWBInfo.do_whiteboard) {
-      MsgBox(TgLoadString(STID_DETACH_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
-      return;
+      /*
+       * Now it's allowed.
+       *
+       * MsgBox(TgLoadString(STID_DETACH_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
+       * return;
+       */
    }
    if (tgifObj->fattr == NULL) {
       MsgBox(TgLoadString(STID_FILE_CONTAINS_NO_ATTRS), TOOL_NAME, INFO_MB);
@@ -2101,8 +2109,12 @@ void EditFileAttrs()
    char **entries=NULL;
 
    if (gstWBInfo.do_whiteboard) {
-      MsgBox(TgLoadString(STID_EDIT_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
-      return;
+      /*
+       * Now it's allowed.
+       *
+       * MsgBox(TgLoadString(STID_EDIT_FILE_ATTRS_IN_WB), TOOL_NAME, INFO_MB);
+       * return;
+       */
    }
    if (tgifObj->fattr == NULL) {
       MsgBox(TgLoadString(STID_FILE_CONTAINS_NO_ATTRS), TOOL_NAME, INFO_MB);
