@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.34 2002/01/05 02:47:58 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/auxtext.c,v 4.41 2003/08/08 05:07:23 william Exp $
  */
 
 #define _INCLUDE_FROM_AUXTEXT_C_
@@ -93,6 +93,25 @@ int GetNumberOfMiniLines(minilines)
    return count;
 }
 
+int IsHex(ch, pn_val)
+   char ch;
+   int *pn_val;
+{
+   if (ch >= '0' && ch <= '9') {
+      *pn_val = ((int)(ch - '0'));
+      return TRUE;
+   }
+   if (ch >= 'a' && ch <= 'f') {
+      *pn_val = 10+((int)(ch - 'a'));
+      return TRUE;
+   }
+   if (ch >= 'A' && ch <= 'F') {
+      *pn_val = 10+((int)(ch - 'F'));
+      return TRUE;
+   }
+   return FALSE;
+}
+
 int InputOctalString(pszBuf, pnBufSize)
    char *pszBuf;
    int *pnBufSize;
@@ -113,61 +132,85 @@ int InputOctalString(pszBuf, pnBufSize)
    UtilTrimBlanks(szSpec);
    if (*szSpec == '\0') return FALSE;
 
-   for (i=0, psz=szSpec; !nError && i < nBufSizeIn-1 && *psz != '\0';
-         i++, psz++) {
-      int nThisIsOctal=FALSE;
+   if (szSpec[0] == '0' && szSpec[1] == 'x') {
+      for (i=0, psz=(&szSpec[2]); !nError && i < nBufSizeIn-1 && *psz != '\0';
+            i++, psz=(&psz[2])) {
+         int hi_nibble=0, lo_nibble=0;
 
-      if (*psz == '\\') {
-         if (psz[1] == '\0') {
-            pszBuf[i] = '\\';
-         } else if (psz[1] == '\\') {
-            pszBuf[i] = '\\';
-            psz++;
-         } else if (psz[1] == 'r') {
-            pszBuf[i] = '\r';
-            psz++;
-         } else if (psz[1] == 'n') {
-            pszBuf[i] = '\n';
-            psz++;
-         } else if (psz[1] >= '0' && psz[1] <= '7') {
-            char *psz1=NULL, saved_ch='\0';
-            int oval=0;
+         if (IsHex(psz[0], &hi_nibble) && IsHex(psz[1], &lo_nibble)) {
+            int hval=(((hi_nibble&0x0ff)<<4)+(lo_nibble&0x0ff));
 
-            *psz = '0';
-            for (psz1=(&psz[2]); *psz1 >= '0' && *psz1 <= '7'; psz1++) ;
-            saved_ch = (*psz1);
-            *psz1 = '\0';
-            if (sscanf(psz, "%o", &oval) != 1) {
-               nError = TRUE;
-            } else {
-               pszBuf[i] = (char)(oval & 0xff);
+            pszBuf[i] = hval;
 
-               nThisIsOctal = TRUE;
-               if (canvasFontDoubleByte && canvasFontDoubleByteModBytes) {
-                  if (nPrevIsOctal) {
-                     nByte1OfOctal = !nByte1OfOctal;
-                  } else {
-                     nByte1OfOctal = TRUE;
-                  }
-                  if ((pszBuf[i] & 0x80) == 0) {
-                     pszBuf[i] |= 0x80;
-                  }
+            if (canvasFontDoubleByte && canvasFontDoubleByteModBytes) {
+               if ((pszBuf[i] & 0x80) == 0) {
+                  pszBuf[i] |= 0x80;
                }
             }
-            *psz1 = saved_ch;
-            *psz = '\\';
-            if (!nError) {
-               psz = psz1;
-               psz--;
+         } else {
+            nError = TRUE;
+         }
+      }
+   } else {
+      for (i=0, psz=szSpec; !nError && i < nBufSizeIn-1 && *psz != '\0';
+            i++, psz++) {
+         int nThisIsOctal=FALSE;
+
+         if (*psz == '\\') {
+            if (psz[1] == '\0') {
+               pszBuf[i] = '\\';
+            } else if (psz[1] == '\\') {
+               pszBuf[i] = '\\';
+               psz++;
+            } else if (psz[1] == 'r') {
+               pszBuf[i] = '\r';
+               psz++;
+            } else if (psz[1] == 'n') {
+               pszBuf[i] = '\n';
+               psz++;
+            } else if (psz[1] >= '0' && psz[1] <= '7') {
+               char *psz1=NULL, saved_ch='\0';
+               int oval=0;
+
+               *psz = '0';
+               for (psz1=(&psz[2]); *psz1 >= '0' && *psz1 <= '7'; psz1++) ;
+               saved_ch = (*psz1);
+               *psz1 = '\0';
+               if (sscanf(psz, "%o", &oval) != 1) {
+                  nError = TRUE;
+               } else {
+                  pszBuf[i] = (char)(oval & 0xff);
+
+                  nThisIsOctal = TRUE;
+                  if (canvasFontDoubleByte && canvasFontDoubleByteModBytes) {
+                     if (nPrevIsOctal) {
+                        nByte1OfOctal = !nByte1OfOctal;
+                     } else {
+                        nByte1OfOctal = TRUE;
+                     }
+                     if ((pszBuf[i] & 0x80) == 0) {
+                        pszBuf[i] |= 0x80;
+                     }
+                  }
+               }
+               *psz1 = saved_ch;
+               *psz = '\\';
+               if (!nError) {
+                  psz = psz1;
+                  psz--;
+               }
+            } else {
+               i--;
+               psz++;
             }
          } else {
-            i--;
-            psz++;
+            pszBuf[i] = *psz;
          }
-      } else {
-         pszBuf[i] = *psz;
+         nPrevIsOctal = nThisIsOctal;
       }
-      nPrevIsOctal = nThisIsOctal;
+   }
+   if (canvasFontDoubleByte && ((i & 0x1) != 0)) {
+      nError = TRUE;
    }
    if (nError) {
       *pszBuf = '\0';
@@ -223,7 +266,7 @@ int CharIsCntrlShiftINS(key_ev, key_sym)
  *
  * if (CharIsESC(key_ev, buf, key_sym, &has_ch)) {
  * } else if (CharIsCRorLF(key_ev, buf, key_sym, &has_ch)) {
- * } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch)) {
+ * } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch, FALSE)) {
  * } else if (CharIsTAB(key_ev, buf, key_sym, &has_ch)) {
  * }
  */
@@ -278,14 +321,33 @@ int CharIsDEL(key_ev, psz_buf, key_sym, pn_has_ch)
    return (*psz_buf == '\177');
 }
 
-int CharIsBSorDEL(key_ev, psz_buf, key_sym, pn_has_ch)
+int CharIsBS(key_ev, psz_buf, key_sym, pn_has_ch, left_is_BS)
    XKeyEvent *key_ev;
    char *psz_buf;
    KeySym key_sym;
-   int *pn_has_ch;
+   int *pn_has_ch, left_is_BS;
 {
-   if (key_sym==XK_Left || key_sym==XK_KP_Left || key_sym==XK_BackSpace ||
-         key_sym==XK_Delete || key_sym==XK_KP_Delete) {
+   if ((left_is_BS && (key_sym==XK_Left || key_sym==XK_KP_Left)) ||
+         key_sym==XK_BackSpace) {
+      return TRUE;
+   }
+   if (key_ev != NULL && (key_ev->state & ControlMask) && (key_sym == XK_h)) {
+      return TRUE;
+   }
+   if (pn_has_ch != NULL) {
+      return ((*pn_has_ch) && (*psz_buf == '\b'));
+   }
+   return (*psz_buf == '\b');
+}
+
+int CharIsBSorDEL(key_ev, psz_buf, key_sym, pn_has_ch, left_is_BS)
+   XKeyEvent *key_ev;
+   char *psz_buf;
+   KeySym key_sym;
+   int *pn_has_ch, left_is_BS;
+{
+   if ((left_is_BS && (key_sym==XK_Left || key_sym==XK_KP_Left)) ||
+         key_sym==XK_BackSpace || key_sym==XK_Delete || key_sym==XK_KP_Delete) {
       return TRUE;
    }
    if (key_ev != NULL && (key_ev->state & ControlMask) && (key_sym == XK_h)) {
@@ -333,6 +395,25 @@ int CharIsEnd(key_ev, psz_buf, key_sym, pn_has_ch)
    return (key_sym==XK_End || key_sym==XK_KP_End);
 }
 
+int CharIsCntrlSpace(key_ev, psz_buf, key_sym, pn_has_ch)
+   XKeyEvent *key_ev;
+   char *psz_buf;
+   KeySym key_sym;
+   int *pn_has_ch;
+{
+   int state=key_ev->state;
+
+   if (((state & ControlMask) == ControlMask) && ((state & METAMASK) == 0)) {
+      if (key_sym == XK_space || (psz_buf[0] == ' ' && psz_buf[1] == '\0')) {
+         return TRUE;
+      } else if (key_sym == XK_backslash || (psz_buf[0] == '\\' &&
+            psz_buf[1] == '\0')) {
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
 void TranslateKeys(s, key_sym)
    char *s;
    KeySym *key_sym;
@@ -718,6 +799,7 @@ int PaintBackingPixmap(text_ptr, bbox_w, bbox_h, min_lbearing, max_rextra,
    return TRUE;
 }
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG /* debug, do not translate */
 static
 void DebugCachedTextBitmapImage(image, before, w, h, min_lbearing, max_rextra)
@@ -744,6 +826,7 @@ void DebugCachedTextBitmapImage(image, before, w, h, min_lbearing, max_rextra)
    }
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 void MakeCachedTextBitmap(ObjPtr)
    struct ObjRec *ObjPtr;
@@ -1808,6 +1891,28 @@ void SaveTextObj(FP, ObjPtr)
    if (fprintf(FP, "])") == EOF) writeFileFailed = TRUE;
 }
 
+void SaveHexString(FP, buf)
+   FILE *FP;
+   char *buf;
+{
+   for ( ; *buf != '\0'; buf++) {
+      int ival=0;
+
+      ival = ((((unsigned int)(*buf)) >> 4) & 0x0f);
+      if (ival >= 10) {
+         fprintf(FP, "%c", (char)(((int)'a')+(ival-10)));
+      } else {
+         fprintf(FP, "%c", (char)(((int)'0')+ival));
+      }
+      ival = (((unsigned int)(*buf)) & 0x0f);
+      if (ival >= 10) {
+         fprintf(FP, "%c", (char)(((int)'a')+(ival-10)));
+      } else {
+         fprintf(FP, "%c", (char)(((int)'0')+ival));
+      }
+   }
+}
+
 char *ReadString(Str)
    char *Str;
 {
