@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.44 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choice.c,v 4.47 2003/07/23 06:06:35 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOICE_C_
@@ -160,6 +160,8 @@
 int	curChoice=NOTHING;
 int	cycleThroughChoice=FALSE;
 
+int	setCurChoiceDontUpdateUI=FALSE;
+
 static int	curRaisedChoice=INVALID;
 static int	curRaisedMode=INVALID;
 
@@ -495,6 +497,11 @@ void ShowCurFont()
    recs[0].height = choiceWindowH;
    XSetClipRectangles(mainDisplay, choiceGC, 0, 0, recs, 1, YXBanded);
 
+   /*
+    * The following code was added so that when you move the text cursor
+    *         from a double-byte font to a single-byte font, we can close
+    *         the double-byte input method.
+    */
    if (gnInputMethod != TGIM_NONE) {
       if (snPrevShowDoubleByteFont != INVALID &&
             snPrevShowDoubleByteFont && !canvasFontDoubleByte &&
@@ -1110,13 +1117,16 @@ void SetCurChoice(NewChoice)
    case VERTEXMODE: break;
    case ROTATEMODE: break;
    }
-   ShowCursor();
+   if (!setCurChoiceDontUpdateUI) {
+      ShowCursor();
+
+      RedrawModeWindow();
+      UpdatePinnedMenu(MENU_MODE);
+
+      ShowCurChoiceMouseStatus(curChoice, 0, FALSE);
+   }
    textCursorShown = FALSE;
 
-   RedrawModeWindow();
-   UpdatePinnedMenu(MENU_MODE);
-
-   ShowCurChoiceMouseStatus(curChoice, 0, FALSE);
    if (inHyperSpace && curChoice != NOTHING) {
       ToggleHyperSpace(FALSE);
    } else if (!inHyperSpace && selected_single_text_obj != NULL) {
