@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.37 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/choose.c,v 4.44 2003/08/05 04:36:22 william Exp $
  */
 
 #define _INCLUDE_FROM_CHOOSE_C_
@@ -44,6 +44,8 @@
 #include "strtbl.e"
 #include "util.e"
 
+GC nameGC=(GC)0, revNameGC=(GC)0;
+
 typedef struct NamesRec {
    Window base_win, dsp_base_win, dsp_win, scroll_win;
    int base_win_w, base_win_h;
@@ -90,8 +92,6 @@ typedef struct NamesRec {
 
 static struct NamesRec namesInfo;
 
-static GC nameGC=(GC)0, revNameGC=(GC)0;
-
 static XComposeStatus c_stat;
 
 int GetWorkingDirectory(buf, buf_sz)
@@ -284,7 +284,7 @@ void ResetNamesInfo()
    pni->first_index = INVALID;
    pni->marked_index = INVALID;
    /* do not translate -- the string is used to measure things */
-   pni->a_btn_w = ButtonWidth("OK", 8);
+   pni->a_btn_w = ButtonWidth("OK", 8, NULL);
    pni->btn_gap = defaultFontWidth;
    pni->screen_w = DisplayWidth(mainDisplay, mainScreen);
    pni->screen_h = DisplayHeight(mainDisplay, mainScreen);
@@ -351,7 +351,7 @@ void NamesAddButton(btn_str, btn_id)
 
    pni->btn_id[pni->num_btns] = btn_id;
    strcpy(pni->btn_str[pni->num_btns], btn_str);
-   pni->btn_w += ButtonWidth(btn_str, 8);
+   pni->btn_w += ButtonWidth(btn_str, 8, NULL);
    if (pni->num_btns != 0) {
       pni->btn_w += pni->btn_gap;
    }
@@ -395,7 +395,6 @@ void RedrawNameScrollWin()
 
 #include "xbm/check.xbm"
 
-static
 void DrawCheckbox(dpy, win, gc, x, y, w, h, checked)
    Display *dpy;
    Window win;
@@ -661,7 +660,7 @@ void RedrawNameBaseWindow()
    left = pni->btn_start;
    for (i=0; i < pni->num_btns; i++) {
       int len=strlen(pni->btn_str[i]);
-      int button_w=ButtonWidth(pni->btn_str[i], 8), button_h=0;
+      int button_w=ButtonWidth(pni->btn_str[i], 8, NULL), button_h=0;
 
       if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
          button_h = defaultFontHeight + 4;
@@ -674,7 +673,7 @@ void RedrawNameBaseWindow()
       pni->btn_bbox[i].rbx = pni->btn_bbox[i].ltx+button_w;
       pni->btn_bbox[i].rby = pni->btn_bbox[i].lty+button_h;
       DisplayButtonInBBox(pni->base_win, pni->btn_str[i],
-            len, &(pni->btn_bbox[i]), BUTTON_NORMAL, FALSE, 0);
+            len, &(pni->btn_bbox[i]), BUTTON_NORMAL, FALSE, 0, NULL);
       left = pni->btn_bbox[i].rbx + 1 + defaultFontWidth;
    }
 }
@@ -1536,6 +1535,10 @@ int ControlChar(key_ev, key_sym)
       case XK_KP_Right: return (BAD);
       case XK_Down: key_sym = ((unsigned long)'f')&0xff; break;
       case XK_KP_Down: key_sym = ((unsigned long)'f')&0xff; break;
+      case XK_Page_Up: key_sym = ((unsigned long)'b')&0xff; break;
+      case XK_KP_Page_Up: key_sym = ((unsigned long)'b')&0xff; break;
+      case XK_Page_Down: key_sym = ((unsigned long)'f')&0xff; break;
+      case XK_KP_Page_Down: key_sym = ((unsigned long)'f')&0xff; break;
       }
    } else {
       switch (key_sym) {
@@ -1547,6 +1550,10 @@ int ControlChar(key_ev, key_sym)
       case XK_KP_Right: return BAD;
       case XK_Down: key_sym = ((unsigned long)'j')&0xff; break;
       case XK_KP_Down: key_sym = ((unsigned long)'j')&0xff; break;
+      case XK_Page_Up: key_sym = ((unsigned long)'b')&0xff; break;
+      case XK_KP_Page_Up: key_sym = ((unsigned long)'b')&0xff; break;
+      case XK_Page_Down: key_sym = ((unsigned long)'f')&0xff; break;
+      case XK_KP_Page_Down: key_sym = ((unsigned long)'f')&0xff; break;
       }
    }
    switch (key_sym&0xff) {
@@ -1736,7 +1743,7 @@ void KeyPressInNames(key_ev, pn_changing, pn_selected_btn_index)
    } else if (CharIsESC(key_ev, buf, key_sym, &has_ch)) {
       *pn_changing = FALSE;
       *pn_selected_btn_index = GetBtnIndexFromBtnId(BUTTON_CANCEL);
-   } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch)) {
+   } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch, FALSE)) {
       if (pni->edit_style == NAMES_SELECT_FILE &&
             (pni->faking_dot_dot || pni->pop_from_root)) {
          *pn_changing = FALSE;
@@ -1753,7 +1760,9 @@ void KeyPressInNames(key_ev, pn_changing, pn_selected_btn_index)
    } else if ((!pni->tabbed_from_root || pni->num_entries != 0) &&
          ((key_sym>'\040' && key_sym<='\177' &&
          (key_ev->state & ControlMask)) || key_sym==XK_Up ||
-         key_sym==XK_Down || key_sym==XK_KP_Up || key_sym==XK_KP_Down)) {
+         key_sym==XK_Down || key_sym==XK_KP_Up || key_sym==XK_KP_Down ||
+         key_sym==XK_Page_Up || key_sym==XK_KP_Page_Up ||
+         key_sym==XK_Page_Down || key_sym==XK_KP_Page_Down)) {
       SpecialKeyInNames(key_ev, key_sym, pn_changing, pn_selected_btn_index);
    } else if (key_sym>='\040' && key_sym<='\177' &&
          (!pni->tabbed_from_root || pni->num_entries != 0)) {
@@ -1840,7 +1849,7 @@ void ButtonPressInNames(button_ev, pn_changing, pn_selected_btn_index)
                      PointerMotionMask | ButtonReleaseMask, GrabModeAsync,
                      GrabModeAsync, None, handCursor, CurrentTime);
                DisplayButtonInBBox(pni->base_win, btn_str, len,
-                     bbox, BUTTON_INVERT, FALSE, 0);
+                     bbox, BUTTON_INVERT, FALSE, 0, NULL);
                while (looping) {
                   XNextEvent(mainDisplay, &ev);
 
@@ -1854,13 +1863,13 @@ void ButtonPressInNames(button_ev, pn_changing, pn_selected_btn_index)
                         if (!selected) {
                            selected = TRUE;
                            DisplayButtonInBBox(pni->base_win, btn_str, len,
-                                 bbox, BUTTON_INVERT, FALSE, 0);
+                                 bbox, BUTTON_INVERT, FALSE, 0, NULL);
                         }
                      } else {
                         if (selected) {
                            selected = FALSE;
                            DisplayButtonInBBox(pni->base_win, btn_str, len,
-                                 bbox, BUTTON_NORMAL, FALSE, 0);
+                                 bbox, BUTTON_NORMAL, FALSE, 0, NULL);
                         }
                      }
                   }
@@ -1869,7 +1878,7 @@ void ButtonPressInNames(button_ev, pn_changing, pn_selected_btn_index)
                   *pn_changing = FALSE;
                   *pn_selected_btn_index = i;
                   DisplayButtonInBBox(pni->base_win, btn_str, len, bbox,
-                        BUTTON_NORMAL, FALSE, 0);
+                        BUTTON_NORMAL, FALSE, 0, NULL);
                }
                break;
             }
@@ -2115,7 +2124,7 @@ int Names(win_name, pn_selected_index, selected_str, str_sz, p_void)
    }
    left = pni->btn_start;
    for (i=0; i < pni->num_btns; i++) {
-      int button_w=ButtonWidth(pni->btn_str[i], 8), button_h=0;
+      int button_w=ButtonWidth(pni->btn_str[i], 8, NULL), button_h=0;
 
       if (boldMsgFontSet == NULL && boldMsgFontPtr == NULL) {
          pni->btn_bbox[i].lty = (6+ITEM_DSPED)*ROW_HEIGHT;
@@ -2250,7 +2259,8 @@ int Names(win_name, pn_selected_index, selected_str, str_sz, p_void)
       if (pni->exposed && selected_btn_index != INVALID) {
          DisplayButtonInBBox(pni->base_win, pni->btn_str[selected_btn_index],
                strlen(pni->btn_str[selected_btn_index]),
-               &(pni->btn_bbox[selected_btn_index]), BUTTON_INVERT, FALSE, 0);
+               &(pni->btn_bbox[selected_btn_index]), BUTTON_INVERT, FALSE, 0,
+               NULL);
          XSync(mainDisplay, False);
       }
       if (pni->edit_style == NAMES_SELECT_FILE) {
@@ -2291,7 +2301,8 @@ int Names(win_name, pn_selected_index, selected_str, str_sz, p_void)
       if (pni->exposed && selected_btn_index != INVALID) {
          DisplayButtonInBBox(pni->base_win, pni->btn_str[selected_btn_index],
                strlen(pni->btn_str[selected_btn_index]),
-               &(pni->btn_bbox[selected_btn_index]), BUTTON_NORMAL, FALSE, 0);
+               &(pni->btn_bbox[selected_btn_index]), BUTTON_NORMAL, FALSE, 0,
+               NULL);
          XSync(mainDisplay, False);
       }
    }
