@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.44 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/cmd.c,v 4.57 2003/02/25 01:17:43 william Exp $
  */
 
 #define _INCLUDE_FROM_CMD_C_
@@ -43,6 +43,7 @@
 #include "drawing.e"
 #include "dup.e"
 #include "file.e"
+#include "grid.e"
 #include "imgproc.e"
 #include "mainmenu.e"
 #include "mark.e"
@@ -171,6 +172,9 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
          next_sel = sel_ptr->next;
          free(sel_ptr);
       }
+      if (CmdPtr->subcmd != NULL) {
+         free(CmdPtr->subcmd);
+      }
       break;
    case CMD_STRETCH:
       for (sel_ptr=CmdPtr->top_before; sel_ptr!=NULL; sel_ptr=next_sel) {
@@ -243,13 +247,16 @@ void DeleteARedoRecord(CmdPtr, percent_start, percent_end)
       }
       break;
    case CMD_CHAT_A_LINE:
-#ifdef _TGIF_WB2
-      for (sel_ptr=CmdPtr->top_after; sel_ptr!=NULL; sel_ptr=next_sel) {
-         next_sel = sel_ptr->next;
-         FreeObj(sel_ptr->obj);
-         free(sel_ptr);
+      if (CmdPtr->subcmd != NULL) {
+         UtilFree(CmdPtr->subcmd->detail.chat.buf);
+         free(CmdPtr->subcmd);
       }
-#endif /* _TGIF_WB2 */
+      break;
+   case CMD_WB_SLIDESHOW:
+      if (CmdPtr->subcmd != NULL) free(CmdPtr->subcmd);
+      break;
+   case CMD_WB_CLEARALL:
+      if (CmdPtr->subcmd != NULL) free(CmdPtr->subcmd);
       break;
    }
 #ifdef _TGIF_WB2
@@ -567,7 +574,7 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
    for (index=0; index < CmdPtr->count_before; index++) {
       char *psz_full_id=(((char**)CmdPtr->pos_before)[index]), *psz=NULL;
       char pid_and_host[MAXSTRING];
-      int id=(-1), found=FALSE, pos_to_match=INVALID;
+      int id=(-1), found=FALSE, pos_to_match=INVALID, obj_is_tgifobj=FALSE;
 
       if (*psz_full_id == '#') {
          psz = strchr(++psz_full_id, ':');
@@ -602,6 +609,9 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          *psz = '\0';
          if (sscanf(psz_full_id, "%d", &id) == 1) {
             strcpy(pid_and_host, &psz[1]);
+            if (id == INVALID) {
+               obj_is_tgifobj = TRUE;
+            }
          } else {
 #ifdef _TGIF_WB2 /* debug, do not translate */
             sprintf(gszMsgBox, "%s (%s), psz_full_id = %s, pid = %ld.",
@@ -615,12 +625,16 @@ int SetupBeforePointers(CmdPtr, modify_before_pointers, pp_actual_pos_before)
          *psz = '/';
       }
       if (ok) {
-         for ( ; ok && obj_ptr != NULL; obj_ptr=obj_ptr->next, cur_pos++) {
+         for (obj_ptr=topObj, cur_pos=0 ; ok && obj_ptr != NULL;
+               obj_ptr=obj_ptr->next, cur_pos++) {
             if (pos_to_match == INVALID) {
                if (obj_ptr->creator_full_id == NULL) {
                   if (obj_ptr->id == id &&
                         strcmp(pid_and_host, gszLocalPID) == 0) {
                      found = TRUE;
+                  } else if (obj_ptr->id == INVALID &&
+                        CmdPtr->include_tgif_obj && obj_is_tgifobj) {
+                     found = TRUE;
                   }
                } else {
                   if (strcmp(obj_ptr->creator_full_id, psz_full_id) == 0) {
@@ -690,9 +704,18 @@ int RedoDeleteCmd(CmdPtr)
    struct SelRec *sel_ptr;
 
    if (CmdPtr->serialized) {
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+         if (CmdPtr->include_tgif_obj) {
+            UnlinkObj(topObj);
+         }
          return FALSE;
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    topSel = CmdPtr->top_before;
    botSel = CmdPtr->bot_before;
@@ -753,12 +776,23 @@ int UndoOrRedoMoveCmd(CmdPtr)
    int dx, dy;
 
    if (CmdPtr->serialized) {
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+         if (CmdPtr->include_tgif_obj) {
+            UnlinkObj(topObj);
+         }
          return FALSE;
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
-   dx = (CmdPtr->undone) ? CmdPtr->dx : -(CmdPtr->dx);
-   dy = (CmdPtr->undone) ? CmdPtr->dy : -(CmdPtr->dy);
+   dx = (CmdPtr->undone) ? CmdPtr->subcmd->detail.move.dx :
+           -(CmdPtr->subcmd->detail.move.dx);
+   dy = (CmdPtr->undone) ? CmdPtr->subcmd->detail.move.dy :
+           -(CmdPtr->subcmd->detail.move.dy);
 
    CopySel(CmdPtr->top_before, CmdPtr->count_before, &topSel, &botSel);
 
@@ -1023,8 +1057,14 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
       }
 #endif /* _TGIF_DBG_WB2 */
 #endif /* _TGIF_WB2 */
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
       if (shuffle) {
          if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+            if (CmdPtr->include_tgif_obj) {
+               UnlinkObj(topObj);
+            }
             return FALSE;
          }
          topSel = CmdPtr->top_before;
@@ -1049,6 +1089,9 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
             saved_bot_sel = CmdPtr->bot_before;
             CmdPtr->top_before = CmdPtr->bot_before = NULL;
             if (!SetupBeforePointers(CmdPtr, TRUE, NULL)) {
+               if (CmdPtr->include_tgif_obj) {
+                  UnlinkObj(topObj);
+               }
                return FALSE;
             }
             CmdPtr->top_after = CmdPtr->top_before;
@@ -1057,10 +1100,16 @@ int UndoOrRedoReplaceCmd(CmdPtr, HighLightSingleObj)
             CmdPtr->bot_before = saved_bot_sel;
          } else {
             if (!SetupBeforePointers(CmdPtr, FALSE, &actual_pos_before)) {
+               if (CmdPtr->include_tgif_obj) {
+                  UnlinkObj(topObj);
+               }
                return FALSE;
             }
          }
       }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    LinkJustTheObjects(CmdPtr->top_before, CmdPtr->bot_before);
 
@@ -1196,11 +1245,20 @@ int UndoOrRedoOneToManyCmd(CmdPtr)
    int *actual_pos_before=NULL;
 
    if (CmdPtr->serialized) {
-     if (CmdPtr->undone) {
-        if (!SetupBeforePointers(CmdPtr, TRUE, &actual_pos_before)) {
-           return FALSE;
-        }
-     }
+      if (CmdPtr->include_tgif_obj) {
+         AddObj(NULL, topObj, tgifObj);
+      }
+      if (CmdPtr->undone) {
+         if (!SetupBeforePointers(CmdPtr, TRUE, &actual_pos_before)) {
+            if (CmdPtr->include_tgif_obj) {
+               UnlinkObj(topObj);
+            }
+            return FALSE;
+         }
+      }
+      if (CmdPtr->include_tgif_obj) {
+         UnlinkObj(topObj);
+      }
    }
    if (CmdPtr->include_tgif_obj) {
       AddObj(NULL, topObj, tgifObj);
@@ -1328,6 +1386,22 @@ void UndoOrRedoGotoPageCmd(CmdPtr)
    ShowPage();
 }
 
+static
+void UndoOrRedoWBSlideShow(CmdPtr)
+   struct CmdRec *CmdPtr;
+{
+   int into_slideshow=CmdPtr->subcmd->detail.slideshow.into_slideshow;
+
+   if (CmdPtr->undone) {
+      /* executing the command */
+      if (!inSlideShow && into_slideshow) {
+         EnterSlideShow();
+      } else if (inSlideShow && !into_slideshow) {
+         LeaveSlideShow();
+      }
+   }
+}
+
 typedef struct tagUndoRedoRestoreDrawingModeRec {
    int cur_choice;
    int text_cursor_shown;
@@ -1396,9 +1470,10 @@ void UndoRedoRestoreDrawingMode(p_urrdmi)
    default: SetCurChoice(p_urrdmi->cur_choice); break;
    }
 }
-void UndoACmd(CmdPtr, HighLight)
+
+void UndoACmd(CmdPtr, HighLight, nPerformAction)
    struct CmdRec *CmdPtr;
-   int HighLight;
+   int HighLight, nPerformAction;
 {
    struct CmdRec *cmd_ptr;
 
@@ -1408,11 +1483,11 @@ void UndoACmd(CmdPtr, HighLight)
    case CMD_COMPOSITE:
       if (CmdPtr->first->type==CMD_MOVE || CmdPtr->first->type==CMD_STRETCH) {
          for (cmd_ptr=CmdPtr->last; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->prev) {
-            UndoACmd(cmd_ptr, FALSE);
+            UndoACmd(cmd_ptr, FALSE, nPerformAction);
          }
       } else {
          for (cmd_ptr=CmdPtr->last; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->prev) {
-            UndoACmd(cmd_ptr, TRUE);
+            UndoACmd(cmd_ptr, TRUE, nPerformAction);
          }
       }
       break;
@@ -1449,16 +1524,16 @@ void UndoCmd()
       return;
    }
    undoingOrRedoing = TRUE;
-   UndoACmd(curCmd, TRUE);
+   UndoACmd(curCmd, TRUE, TRUE);
    curCmd = curCmd->prev;
    undoingOrRedoing = FALSE;
 
    UndoRedoRestoreDrawingMode(&urrdmi);
 }
 
-int RedoACmd(CmdPtr, HighLight)
+int RedoACmd(CmdPtr, HighLight, nPerformAction)
    struct CmdRec *CmdPtr;
-   int HighLight;
+   int HighLight, nPerformAction;
 {
    int ok=TRUE;
    struct CmdRec *cmd_ptr;
@@ -1470,19 +1545,19 @@ int RedoACmd(CmdPtr, HighLight)
       if (CmdPtr->first->type==CMD_MOVE || CmdPtr->first->type==CMD_STRETCH) {
          for (cmd_ptr=CmdPtr->first->next; cmd_ptr!=NULL;
                cmd_ptr=cmd_ptr->next) {
-            if (!RedoACmd(cmd_ptr, FALSE)) {
+            if (!RedoACmd(cmd_ptr, FALSE, nPerformAction)) {
                ok = FALSE;
                break;
             }
          }
          if (ok) {
-            if (!RedoACmd(CmdPtr->first, TRUE)) {
+            if (!RedoACmd(CmdPtr->first, TRUE, nPerformAction)) {
                ok = FALSE;
             }
          }
       } else {
          for (cmd_ptr=CmdPtr->first; cmd_ptr!=NULL; cmd_ptr=cmd_ptr->next) {
-            if (!RedoACmd(cmd_ptr, TRUE)) {
+            if (!RedoACmd(cmd_ptr, TRUE, nPerformAction)) {
                ok = FALSE;
                break;
             }
@@ -1497,6 +1572,7 @@ int RedoACmd(CmdPtr, HighLight)
    case CMD_ONE_TO_MANY: ok = UndoOrRedoOneToManyCmd(CmdPtr); break;
    case CMD_MANY_TO_ONE: ok = UndoOrRedoOneToManyCmd(CmdPtr); break;
    case CMD_GOTO_PAGE: UndoOrRedoGotoPageCmd(CmdPtr); break;
+   case CMD_WB_SLIDESHOW: UndoOrRedoWBSlideShow(CmdPtr); break;
    }
    CmdPtr->undone = FALSE;
    return ok;
@@ -1528,7 +1604,7 @@ void RedoCmd()
       curCmd = curCmd->next;
    }
    undoingOrRedoing = TRUE;
-   RedoACmd(curCmd, TRUE);
+   RedoACmd(curCmd, TRUE, TRUE);
    undoingOrRedoing = FALSE;
 
    UndoRedoRestoreDrawingMode(&urrdmi);
@@ -1789,10 +1865,37 @@ void FreeAfterSel(CmdPtr)
    CmdPtr->top_after = CmdPtr->bot_after = NULL;
 }
 
+#ifdef _TGIF_WB2
+static
+void PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
+      NumObjs, psi)
+   struct SubCmdRec *SubCmdPtr;
+   struct SelRec *TopSel, *BotSel;
+   int NumObjs;
+   SerializationInfo *psi;
+{
+   psi->subcmd = SubCmdPtr;
+   psi->top_sel = TopSel;
+   psi->bot_sel = BotSel;
+   psi->num_objs = NumObjs;
+
+   psi->include_tgif_obj = recordCmdIncludeTgifObj;
+   psi->new_colormap = recordCmdUsesNewColormap;
+   psi->logical_clock = gstWBInfo.logical_clock;
+   psi->sender_process_id = gszLocalPID;
+
+   psi->top_before = topSelBeforeInCmd;
+   psi->bot_before = botSelBeforeInCmd;
+   psi->pos_before = stackingPosition;
+   psi->count_before = stackingCount;
+   psi->pos_before_has_ids = stackingPositionHasIds;
+}
+#endif /* _TGIF_WB2 */
+
 void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    int CmdType, NumObjs;
-   struct SelRec *TopSel, *BotSel;
    struct SubCmdRec *SubCmdPtr;
+   struct SelRec *TopSel, *BotSel;
 {
    int logical_clock=0;
 #ifdef _TGIF_WB2
@@ -1803,16 +1906,33 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
 #ifdef _TGIF_WB2
       char *psz_wb_data=NULL;
 
-      serializingFile = TRUE;
+      if (gstWBInfo.dont_serialize || CmdType == CMD_GOTO_PAGE) {
+         /*
+          * One of the important thing that happens in SerializeCmd()
+          *         is that the local logical clock got incremented.
+          *         If we don't increment the logical clock, we may
+          *         create a new command that have the same logical clock
+          *         as the one created last time we got here and that's
+          *         illegal. Therefore, we must increment the logical clock.
+          */
+         logical_clock = gstWBInfo.logical_clock++;
+      } else {
+         SerializationInfo si;
 
-      if ((psz_wb_data=SerializeCmd(CmdType, SubCmdPtr, TopSel, BotSel,
-            NumObjs, &logical_clock)) != NULL) {
-         if (!SendWBData(psz_wb_data,logical_clock)) {
-            wb_send_data_failed = TRUE;
+         memset(&si, 0, sizeof(SerializationInfo));
+         PrepareExtendedSerializationInfoForRecordCmd(SubCmdPtr, TopSel, BotSel,
+               NumObjs, &si);
+         
+         serializingFile = TRUE;
+
+         if ((psz_wb_data=SerializeCmd(CmdType, &si, &logical_clock)) != NULL) {
+            if (!SendWBData(psz_wb_data,logical_clock)) {
+               wb_send_data_failed = TRUE;
+            }
+            SerializeFreeBuf(psz_wb_data);
          }
-         SerializeFreeBuf(psz_wb_data);
+         serializingFile = FALSE;
       }
-      serializingFile = FALSE;
 #endif /* _TGIF_WB2 */
    } else {
       if (historyDepth == 0) return;
@@ -1890,19 +2010,40 @@ void RecordCmd(CmdType, SubCmdPtr, TopSel, BotSel, NumObjs)
    case CMD_NEW: break;
    case CMD_DELETE: break;
    case CMD_MOVE:
-      curCmd->dx = SubCmdPtr->detail.mv->dx;
-      curCmd->dy = SubCmdPtr->detail.mv->dy;
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.move.dx = SubCmdPtr->detail.move.dx;
+      curCmd->subcmd->detail.move.dy = SubCmdPtr->detail.move.dy;
       break;
    case CMD_STRETCH: FreeAfterSel(curCmd); break;
    case CMD_REPLACE: FreeAfterSel(curCmd); break;
    case CMD_ONE_TO_MANY: break;
    case CMD_MANY_TO_ONE: break;
    case CMD_GOTO_PAGE: curCmd->count_after = NumObjs; break;
-   case CMD_WB_CLEARALL: break;
+   case CMD_WB_CLEARALL:
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.clearall.page_style =
+            SubCmdPtr->detail.clearall.page_style;
+      curCmd->subcmd->detail.clearall.print_mag =
+            SubCmdPtr->detail.clearall.print_mag;
+      break;
+   case CMD_CHAT_A_LINE:
+      /* intentionally forget what has been chatted */
+      break;
+   case CMD_WB_SLIDESHOW:
+      curCmd->subcmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
+      if (curCmd->subcmd == NULL) FailAllocMessage();
+      memset(curCmd->subcmd, 0, sizeof(struct SubCmdRec));
+      curCmd->subcmd->detail.slideshow.into_slideshow =
+            SubCmdPtr->detail.slideshow.into_slideshow;
+      break;
    }
 #ifdef _TGIF_WB2
    if (wb_send_data_failed) {
-      UndoACmd(curCmd, FALSE);
+      UndoACmd(curCmd, FALSE, TRUE);
       UnlinkCmd(curCmd, &gstWBInfo.first_cmd, &gstWBInfo.last_cmd);
       DeleteARedoRecord(curCmd, (-1.0), (-1.0));
    }
@@ -2002,8 +2143,39 @@ void ChangeReplaceOneCmdToDeleteCmd()
 void RecordWBClearAll()
 {
    if (gstWBInfo.do_whiteboard) {
+      struct SubCmdRec subcmd;
+
+      memset(&subcmd, 0, sizeof(struct SubCmdRec));
+      subcmd.detail.clearall.page_style = pageStyle;
+      subcmd.detail.clearall.print_mag = printMag;
+
       PrepareToRecord(CMD_WB_CLEARALL, NULL, NULL, 0);
-      RecordCmd(CMD_WB_CLEARALL, NULL, NULL, NULL, 0);
+      RecordCmd(CMD_WB_CLEARALL, &subcmd, NULL, NULL, 0);
+   }
+}
+
+void RecordWBSlideShow(into_slideshow)
+   int into_slideshow;
+{
+   if (gstWBInfo.do_whiteboard) {
+      if (pageLayoutMode == PAGE_STACK && firstPage != lastPage) {
+         struct SubCmdRec subcmd;
+
+         memset(&subcmd, 0, sizeof(struct SubCmdRec));
+         subcmd.detail.slideshow.into_slideshow = into_slideshow;
+
+         PrepareToRecord(CMD_WB_SLIDESHOW, NULL, NULL, 0);
+         RecordCmd(CMD_WB_SLIDESHOW, &subcmd, NULL, NULL, 0);
+      }
+   }
+}
+
+void RecordWBChatALine(pSubCmdPtr)
+   struct SubCmdRec *pSubCmdPtr;
+{
+   if (gstWBInfo.do_whiteboard) {
+      PrepareToRecord(CMD_CHAT_A_LINE, NULL, NULL, 0);
+      RecordCmd(CMD_CHAT_A_LINE, pSubCmdPtr, NULL, NULL, 0);
    }
 }
 
