@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/color.c,v 4.43 2002/01/05 12:05:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/color.c,v 4.45 2003/02/25 01:17:43 william Exp $
  */
 
 #define _INCLUDE_FROM_COLOR_C_
@@ -1271,8 +1271,9 @@ int ChangeObjColor(ObjPtr, ColorIndex)
    if ((canChangeAttrColor || changePropertiesOfAttrs) &&
          ObjPtr->type != OBJ_TEXT) {
       for (attr_ptr=ObjPtr->fattr; attr_ptr != NULL; attr_ptr=attr_ptr->next) {
-         if (attr_ptr->obj->color != ColorIndex) {
-            attr_ptr->obj->color = ColorIndex;
+         if (SetMiniLinesProperty(PROP_MASK_COLOR, ColorIndex,
+               &attr_ptr->obj->detail.t->minilines)) {
+            InvalidateTextCache(attr_ptr->obj->detail.t);
             changed = TRUE;
          }
       }
