@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.43 2002/01/05 02:47:59 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/const.h,v 4.50 2003/07/11 05:13:42 william Exp $
  */
 
 #ifndef _TGIF_CONST_H_
@@ -200,6 +200,7 @@
 #define GETFLT(category,val,name) ScanValue("%f", &(val), name, category)
 #define GETDBL(category,val,name) ScanValue("%lg", &(val), name, category)
 #define GETSTR(category,val,name) ScanValue("%s", (val), name, category)
+#define GETDYN(category,val,name) ScanDynStrValue(&(val), name, category)
 
 #define GetPinObj(ObjPtr) ((ObjPtr)->detail.r->pin_connected ? \
       (ObjPtr)->detail.r->last : (ObjPtr)->detail.r->first)
@@ -464,9 +465,10 @@
 #define TIFFEPSI_FILE 9
 #define PNG_FILE 10
 #define JPEG_FILE 11
-#define NETLIST_FILE 12
+#define PPM_FILE 12
+#define NETLIST_FILE 13
 
-#define MAXDEFWHERETOPRINT 13
+#define MAXDEFWHERETOPRINT 14
 
 /* measurement */
 
@@ -705,17 +707,18 @@
 
 /* cmd stuff */
 
-#define CMD_COMPOSITE	 0
-#define CMD_NEW		 1
-#define CMD_DELETE	 2
-#define CMD_MOVE	 3
-#define CMD_STRETCH	 4
-#define CMD_ONE_TO_MANY	 5
-#define CMD_MANY_TO_ONE	 6
-#define CMD_REPLACE	 7
-#define CMD_GOTO_PAGE	 8
-#define CMD_WB_CLEARALL	 9
-#define CMD_CHAT_A_LINE	10
+#define CMD_COMPOSITE     0
+#define CMD_NEW           1
+#define CMD_DELETE        2
+#define CMD_MOVE          3
+#define CMD_STRETCH       4
+#define CMD_ONE_TO_MANY   5
+#define CMD_MANY_TO_ONE   6
+#define CMD_REPLACE       7
+#define CMD_GOTO_PAGE     8
+#define CMD_WB_CLEARALL   9
+#define CMD_CHAT_A_LINE  10
+#define CMD_WB_SLIDESHOW 11
 
 /* rcb radius stuff */
 
@@ -726,7 +729,8 @@
 
 #define ITEM_DSPED 10
 #define ITEM_LEN 30
-#define ROW_HEIGHT ((msgFontPtr==NULL)?(defaultFontHeight+1):(msgFontHeight+1))
+#define ROW_HEIGHT \
+  ((msgFontSet==NULL&&msgFontPtr==NULL)?(defaultFontHeight+1):(msgFontHeight+1))
 #define BUTTON_OK 0
 #define BUTTON_SETDIR 1
 #define BUTTON_CANCEL 2
@@ -839,6 +843,10 @@
 #define TGIM_CHINPUT    1
 #define TGIM_KINPUT     2
 #define TGIM_XIM        3
+#define TGIM_TGTWB5     4
+
+#define TGIM_DBIM       0x1
+#define TGIM_SBIM       0x2
 
 /* postscript stuff */
 
