@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.24 2002/01/05 02:48:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/dialog.c,v 4.27 2003/08/05 04:36:22 william Exp $
  */
 
 #define _INCLUDE_FROM_DIALOG_C_
@@ -808,7 +808,7 @@ int HandleMsgBoxKeyEvent(MBInfoPtr, input)
             return INVALID;
          }
          return TRUE;
-      } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch)) {
+      } else if (CharIsBSorDEL(key_ev, buf, key_sym, &has_ch, FALSE)) {
          if (MBInfoPtr->return_str == NULL) {
             return INVALID;
          }
@@ -851,7 +851,7 @@ int HandleMsgBoxKeyEvent(MBInfoPtr, input)
                         MBInfoPtr->btn_info[i].str,
                         strlen(MBInfoPtr->btn_info[i].str),
                         &MBInfoPtr->btn_info[i].bbox, BUTTON_INVERT,
-                        MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                        MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA, NULL);
                   XSync(mainDisplay, False);
                   return MBInfoPtr->btn_info[i].id;
                }
@@ -940,7 +940,7 @@ int HandleMsgBoxBtnEvent(MBInfoPtr, input)
             DisplayButtonInBBox(MBInfoPtr->btn_win, MBInfoPtr->btn_info[i].str,
                   strlen(MBInfoPtr->btn_info[i].str),
                   &MBInfoPtr->btn_info[i].bbox, BUTTON_INVERT,
-                  MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                  MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA, NULL);
             if (!debugNoPointerGrab) {
                XGrabPointer(mainDisplay, MBInfoPtr->btn_win, FALSE,
                      PointerMotionMask | ButtonReleaseMask, GrabModeAsync,
@@ -965,7 +965,8 @@ int HandleMsgBoxBtnEvent(MBInfoPtr, input)
                            strlen(MBInfoPtr->btn_info[i].str),
                            &MBInfoPtr->btn_info[i].bbox,
                            BUTTON_NORMAL,
-                           MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                           MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA,
+                           NULL);
                      return MBInfoPtr->btn_info[i].id;
                   }
                   break;
@@ -986,7 +987,8 @@ int HandleMsgBoxBtnEvent(MBInfoPtr, input)
                               MBInfoPtr->btn_info[i].str,
                               strlen(MBInfoPtr->btn_info[i].str),
                               &MBInfoPtr->btn_info[i].bbox, BUTTON_NORMAL,
-                              MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                              MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA,
+                              NULL);
                         inside = FALSE;
                      }
                   } else {
@@ -998,7 +1000,8 @@ int HandleMsgBoxBtnEvent(MBInfoPtr, input)
                               MBInfoPtr->btn_info[i].str,
                               strlen(MBInfoPtr->btn_info[i].str),
                               &MBInfoPtr->btn_info[i].bbox, BUTTON_INVERT,
-                              MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                              MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA,
+                              NULL);
                         inside = TRUE;
                      }
                   }
@@ -1073,7 +1076,8 @@ void RefreshMsgBox(MBInfoPtr)
             DisplayButtonInBBox(MBInfoPtr->btn_win, MBInfoPtr->btn_info[i].str,
                   strlen(MBInfoPtr->btn_info[i].str),
                   &MBInfoPtr->btn_info[i].bbox, BUTTON_NORMAL,
-                  MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA);
+                  MBInfoPtr->btn_info[i].highlight, BTN_XY_EXTRA,
+                  NULL);
          }
       }
    }
