@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.32 2002/01/05 02:48:00 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/eps.c,v 4.38 2003/07/22 23:28:16 william Exp $
  */
 
 #define _INCLUDE_FROM_EPS_C_
@@ -87,6 +87,8 @@ static char defTiffToXbmCmd[]="tifftopnm %s | pgmtopbm | pbmtoxbm";
 static char defPsToXbmCmd[] =
       "gs -q -dNOPAUSE -sDEVICE=pbm -sOutputFile=- -- \"%s\" | pbmtoxbm";
 
+static float bitmapThresholdFor8bpsPreviewBitmap=(float)0.5;
+
 void InitEPS()
 {
    char *c_ptr=NULL;
@@ -151,6 +153,17 @@ void InitEPS()
          NULL && UtilStrICmp(c_ptr, "true") == 0) {
       autoEPSPreviewBitmap = TRUE;
    }
+   bitmapThresholdFor8bpsPreviewBitmap = (float)0.5;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"BitmapThreshold"))!=NULL) {
+      bitmapThresholdFor8bpsPreviewBitmap = (float) atof(c_ptr);
+      if (bitmapThresholdFor8bpsPreviewBitmap < ((float)0) ||
+            bitmapThresholdFor8bpsPreviewBitmap > (((float)1)+INT_TOL)) {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "BitmapThreshold", c_ptr, "0.5");
+         fprintf(stderr, "\n");
+         bitmapThresholdFor8bpsPreviewBitmap = (float)0.5;
+      }
+   }
 }
 
 static
@@ -198,6 +211,31 @@ void DumpXBmByte(fp, num_nibbles, image_h, nibble_index, row_index,
    }
 }
 
+static
+int UpdateEndDetectState(c, p_end_detect_state, p_line_count)
+   char c;
+   int *p_end_detect_state;
+   int *p_line_count;
+{
+   int end_detect_state=(*p_end_detect_state);
+
+   if (c == '\n') {
+      (*p_line_count)++;
+      *p_end_detect_state = 0;
+
+      return FALSE;
+   }
+   switch (end_detect_state) {
+   case 0: *p_end_detect_state = ((c == '%') ? 1 : 0); break;
+   case 1: *p_end_detect_state = ((c == '%') ? 2 : 0); break;
+   case 2: *p_end_detect_state = ((c == 'E' || c == 'e') ? 3 : 0); break;
+   case 3: *p_end_detect_state = ((c == 'n') ? 4 : 0); break;
+   case 4: *p_end_detect_state = ((c == 'd') ? 5 : 0); break;
+   default: *p_end_detect_state = 0; break;
+   }
+   return (*p_end_detect_state == 5);
+}
+
 static
 int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
    FILE *fp;
@@ -206,9 +244,9 @@ int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
    Pixmap *bitmap;
    XImage **image;
 {
-   int k, j;
-   int i, num_nibbles, bit_count, out_byte_count=0;
-   char *line, *c_ptr;
+   int k=0, j=0, i=0, num_nibbles=0, bit_count=0, out_byte_count=0, line_sz=0;
+   int line_count=0, end_detect_state=0; /* detecting "%%End..." */
+   char *line=NULL, *c_ptr=NULL;
 
    switch (bps) {
    case 1:
@@ -216,12 +254,16 @@ int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
             (int)(image_w>>2)+1;
       if (num_nibbles & 0x1) num_nibbles++;
       break;
+   case 8:
+      num_nibbles = (image_w<<1);
+      break;
    default:
       sprintf(gszMsgBox, TgLoadString(STID_GIVEN_BPS_PREVIEW_NOT_SUP), bps);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
       return FALSE;
    }
-   line = (char*)malloc((num_nibbles+10)*sizeof(char));
+   line_sz = (num_nibbles+10)*sizeof(char);
+   line = (char*)malloc(line_sz);
    if (line == NULL) FailAllocMessage();
    if (cmdLineDosEpsFilter && cmdLinePreviewOnly) {
       *bitmap = None;
@@ -240,9 +282,28 @@ int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
    for (i=0; i < image_h; i++) {
       int byte_data=0;
 
-      for (j=0, c_ptr = line; j < num_nibbles; j++, c_ptr++) {
+      for (j=0, c_ptr=line; j < num_nibbles; j++, c_ptr++) {
          while (TRUE) {
-            if ((*c_ptr=(char)getc(fp)) == (char)EOF) {
+            int c=getc(fp);
+
+            if (c == EOF) {
+               sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
+                     file_name);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+               free(line);
+               if (*bitmap != None) {
+                  XFreePixmap(mainDisplay, *bitmap);
+                  *bitmap = None;
+               }
+               if (*image != NULL) {
+                  XDestroyImage(*image);
+                  *image = NULL;
+               }
+               return FALSE;
+            }
+            *c_ptr = (char)c;
+            if (UpdateEndDetectState(*c_ptr, &end_detect_state, &line_count)) {
+               /* %%End... detected */
                sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
                      file_name);
                MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -288,12 +349,30 @@ int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
                byte_data = data;
             }
          } else {
-            for (k = 0; k < 4; k++) {
-               if (bit_count++ == image_w) break;
+            switch (bps) {
+            case 1:
+               for (k = 0; k < 4; k++) {
+                  if (bit_count++ == image_w) break;
 
-               if (data & (1<<(3-k))) {
-                  XPutPixel(*image, j*4+k, i, 1);
+                  if (data & (1<<(3-k))) {
+                     XPutPixel(*image, j*4+k, i, 1);
+                  }
                }
+               break;
+            case 8:
+               if (j & 0x1) {
+                  double dval=(double)0;
+
+                  byte_data |= (data<<4);
+                  dval = (((double)byte_data) / ((double)255));
+                  if (dval > ((double)bitmapThresholdFor8bpsPreviewBitmap)) {
+                     XPutPixel(*image, (j>>1), i, 1);
+                  }
+               } else {
+                  byte_data = data;
+               }
+               break;
+            default: /* should not get here */ break;
             }
          }
       }
@@ -304,7 +383,7 @@ int ReadPreviewBitmap(fp, file_name, image_w, image_h, bps, bitmap, image)
          }
       }
    }
-   if (fgets(line, MAXSTRING, fp) == NULL) {
+   if (fgets(line, line_sz, fp) == NULL) {
       sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BMP_IN_EPS),
             file_name);
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -930,8 +1009,14 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
       } else if (!(cmdLineDosEpsFilter && !cmdLinePreviewOnly) &&
             !preview_found && strncmp(line, "%%BeginPreview:", 15) == 0) {
          int bps=1;
-         char *preview_line;
+         char *preview_line=NULL;
 
+         /*
+          * The 4th argument to %%BeginPreview is the number of lines
+          *         in the preview image.  It's probably a good idea
+          *         to handle it or have a way not to read beyond
+          *         %%EndImage or %%EndPreview.
+          */
          if (sscanf(&(line[15]), "%d %d %d", &image_w, &image_h, &bps) != 3) {
             sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BOX_IN_EPS),
                   real_fname);
@@ -941,15 +1026,20 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
             if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
             CleanUpLines();
             return BitmapFileInvalid;
-         } else if (bps != 1) {
-            sprintf(gszMsgBox, TgLoadString(STID_GIVEN_BPS_PREVIEW_NOT_SUP),
-                  bps);
-            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
-            if (need_to_free_line) free(line);
-            fclose(fp);
-            if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
-            CleanUpLines();
-            return BitmapFileInvalid;
+         } else {
+            switch (bps) {
+            case 1: break;
+            case 8: break;
+            default:
+               sprintf(gszMsgBox, TgLoadString(STID_GIVEN_BPS_PREVIEW_NOT_SUP),
+                     bps);
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+               if (need_to_free_line) free(line);
+               fclose(fp);
+               if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
+               CleanUpLines();
+               return BitmapFileInvalid;
+            }
          }
          if (boundingbox_found) {
             /*
@@ -980,9 +1070,7 @@ int DoMyReadEPSFile(file_name, pn_image_w, pn_image_h, p_bitmap, p_image,
          preview_found = TRUE;
          if (!ReadPreviewBitmap(fp, real_fname, image_w, image_h, bps, &bitmap,
                &image)) {
-            sprintf(gszMsgBox, TgLoadString(STID_INVALID_PREVIEW_BOX_IN_EPS),
-                  real_fname);
-            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            /* error message already displayed in ReadPreviewBitmap() */
             if (need_to_free_line) free(line);
             fclose(fp);
             if (*tiff_eps_fname != '\0') unlink(tiff_eps_fname);
@@ -1270,7 +1358,7 @@ int JustReadEPSLines(xbm_ptr)
          }
          propagated_eps_status = INVALID;
       } else if (!preview_found && strncmp(line, "%%BeginPreview:", 15) == 0) {
-         char *preview_line;
+         char *preview_line=NULL;
 
          preview_found = TRUE;
          found = FALSE;
@@ -1486,7 +1574,7 @@ void DumpEPSObj(FP, ObjPtr)
    fprintf(FP, "/letter {} def\n");
    fprintf(FP, "/legal {} def\n");
    fprintf(FP, "/a4 {} def\n");
-   fprintf(FP, "/setpagedevice {} def\n");
+   fprintf(FP, "/setpagedevice {pop} def\n");
    fprintf(FP, "/erasepage {} def\n");
    fprintf(FP, "/bop-hook {} def\n");
    fprintf(FP, "/eop-hook {} def\n");
