@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/expr.c,v 4.10 2002/01/05 02:48:01 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/expr.c,v 4.13 2003/02/25 01:17:44 william Exp $
  */
 
 #define _INCLUDE_FROM_EXPR_C_
@@ -86,9 +86,11 @@ static char *pszExpr=NULL;
 static struct OpRec *topOpStk=NULL;
 static struct ValRec *topValStk=NULL;
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG
 static int gnExprDebug=FALSE;
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 int BadOperandType(operator_str)
@@ -895,6 +897,7 @@ int ParseForIDVal(ppsz_token, v)
    return TRUE;
 }
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG /* debug, do not translate */
 static char *opName[] = {
    "BEGIN", "?", ":", "||", "&&", "|", "^", "&", "==", "!=", ">", "<", ">=",
@@ -948,6 +951,7 @@ void DebugValStk()
    fprintf(stderr, "                   |________...\n");
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 int ProcessOp(prec)
