@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.45 2002/03/21 07:44:50 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/grid.c,v 4.49 2003/08/16 19:38:52 william Exp $
  */
 
 #define _INCLUDE_FROM_GRID_C_
@@ -62,6 +62,7 @@
 #include "strtbl.e"
 #include "text.e"
 #include "util.e"
+#include "wb.e"
 #include "xprtfltr.e"
 
 #define ENGLISH_GRID_STEP 8
@@ -560,6 +561,10 @@ void WhereToPrintMsg()
          break;
       case PNG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_PNG_FILE)); break;
       case JPEG_FILE: Msg(TgLoadString(STID_WILL_EXPORT_JPEG_FILE)); break;
+      case PPM_FILE:
+         Msg(TgLoadString(colorDump ? STID_WILL_EXPORT_PPM_FILE :
+               STID_WILL_EXPORT_PBM_FILE));
+         break;
       case NETLIST_FILE:
          Msg(TgLoadString(STID_WILL_EXPORT_NETLIST_FILE));
          break;
@@ -1382,6 +1387,10 @@ int RefreshShowHideMenu(menu)
    /* ShowMode */
    ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWMODE, !noModeWindow);
 
+   if (gstWBInfo.do_whiteboard) {
+      /* ShowChat */
+      ok &= TgSetMenuItemCheckById(menu, CMDID_TOGGLESHOWCHAT, !noChatWindow);
+   }
    return ok;
 }
 
