@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/help.c,v 4.16 2002/01/05 02:48:01 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/help.c,v 4.18 2003/02/25 01:29:29 william Exp $
  */
 
 #define _INCLUDE_FROM_HELP_C_
@@ -75,7 +75,7 @@ void About()
    sprintf(c_ptr, TgLoadString(STID_HYPER_GRAPHICS_INFO), hyperGraphicsURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    sprintf(c_ptr, TgLoadString(STID_MAILING_LIST_INFO),
-         mailingList, joinMailingList, oldMailingList);
+         mailingList, joinMailingList, oldMailingList, mailingURL);
    c_ptr = (&c_ptr[strlen(c_ptr)]);
    sprintf(c_ptr, TgLoadString(STID_SEND_BUG_REPORT_TO), "bill.cheng@acm.org");
    c_ptr = (&c_ptr[strlen(c_ptr)]);
