@@ -5,7 +5,7 @@
  *
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -19,7 +19,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.25 2002/02/22 22:58:35 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/http.c,v 4.32 2003/08/08 05:07:24 william Exp $
  */
 
 #define _INCLUDE_FROM_HTTP_C_
@@ -133,10 +133,13 @@ static int gnDecode[] = {
 };
 
 static
-void DoBase64Encode(buf, buf_len, return_buf)
-   char *buf, *return_buf;
+void DoBase64Encode(buf_in, buf_len, return_buf_in)
+   char *buf_in, *return_buf_in;
    int buf_len;
 {
+   unsigned char *buf=(unsigned char *)buf_in;
+   unsigned char *return_buf=(unsigned char *)return_buf_in;
+
    while (buf_len > 0) {
       switch (buf_len) {
       case 1:
@@ -186,6 +189,83 @@ char *Base64Encode(buf)
    return return_buf;
 }
 
+static
+int Convert(buf4, count, buf3)
+   int *buf4, count;
+   char *buf3;
+{
+   switch (count) {
+   case 2:
+      buf3[0] = ((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03);
+      buf3[1] = '\0';
+      return 1;
+   case 3:
+      buf3[0] = ((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03);
+      buf3[1] = ((buf4[1]<<4) & 0xf0) | ((buf4[2]>>2) & 0x0f);
+      buf3[2] = '\0';
+      return 2;
+   case 4:
+      buf3[0] = (char)(((buf4[0]<<2) & 0xfc) | ((buf4[1]>>4) & 0x03));
+      buf3[1] = (char)(((buf4[1]<<4) & 0xf0) | ((buf4[2]>>2) & 0x0f));
+      buf3[2] = (char)(((buf4[2]<<6) & 0xc0) | (buf4[3] & 0x3f));
+      buf3[3] = '\0';
+      return 3;
+   default:
+#ifdef _TGIF_DBG /* debug, do not translate */
+      Error("Convert()", "Bad count passed to Convert().");
+#endif /* _TGIF_DBG */
+      break;
+   }
+   return 0;
+}
+
+static
+void DoBase64Decode(buf, buf_sz, return_buf)
+   char *buf, *return_buf;
+   int buf_sz;
+{
+   int i=0, j=0;
+   int intbuf[5];
+
+   intbuf[4] = (-1);
+   for (i=0, j=0; i < buf_sz; i++) {
+      intbuf[j++] = gnDecode[(int)buf[i]];
+      if (j == 4) {
+         char out_buf[4];
+         int k=0, bytes=0;
+
+         if (intbuf[3] == (-1)) {
+            if (intbuf[2] == (-1)) {
+               bytes = Convert(intbuf, 2, out_buf);
+            } else {
+               bytes = Convert(intbuf, 3, out_buf);
+            }
+         } else {
+            bytes = Convert(intbuf, 4, out_buf);
+         }
+         for (k=0; k < bytes; k++) {
+            *return_buf++ = (char)(out_buf[k] & 0x0ff);
+         }
+         j = 0;
+      }
+   }
+   *return_buf = '\0';
+}
+
+char *Base64Decode(buf)
+   char *buf;
+{
+   int len=strlen(buf);
+   char *return_buf=(char*)malloc(len*sizeof(char));
+
+   if (return_buf == NULL) {
+      FailAllocMessage();
+      return NULL;
+   }
+   DoBase64Decode(buf, len, return_buf);
+   return return_buf;
+}
+
 static
 void BubbleAuthInfoToTop(pai)
    struct AuthInfo *pai;
@@ -767,6 +847,7 @@ int HttpDoWrite(n_socket, psz_path, psz_host, us_port)
    return status;
 }
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG /* debug, do not translate */
 static char SZ_DEF_GIF_NAME[]="/tmp/htclient.gif";
 
@@ -851,6 +932,7 @@ void DebugHttpDumpResponse(buf)
    }
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 int ParseBadFormat(buf)
@@ -1233,7 +1315,7 @@ int ReadFromSocket(n_socket)
          return TRUE;
       } else {
          sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
-               "WaitForEvent()", status);
+               "ReadFromSocket()", status);
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
          break;
       }
