@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.96 2002/03/08 00:28:35 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/mainloop.c,v 4.117 2003/08/08 05:07:24 william Exp $
  */
 
 #define _INCLUDE_FROM_MAINLOOP_C_
@@ -44,6 +44,7 @@
 
 #include "animate.e"
 #include "auxtext.e"
+#include "chat.e"
 #include "choice.e"
 #include "cmd.e"
 #include "color.e"
@@ -91,6 +92,7 @@
 #include "strtbl.e"
 #include "tangram2.e"
 #include "text.e"
+#include "tidget.e"
 #include "util.e"
 #include "version.e"
 #include "wb.e"
@@ -125,6 +127,8 @@ char	*displayName=NULL;
 int	initializingMain=FALSE;
 int	cmdLineUseXlib=FALSE;
 int	cmdLineVersion=FALSE;
+int	cmdLineNoModeWindow=FALSE;
+int	cmdLineNoChatWindow=FALSE;
 
 int			numExtraWins=0;
 struct WinInfoRec	*extraWinInfo=NULL;
@@ -317,6 +321,7 @@ void CleanUpMergeArgs()
 void CleanUp()
 {
    tgCleanUpProfile();
+   CleanUpChat();
    CleanUpWhiteBoard();
 
    TieLooseEnds();
@@ -374,6 +379,7 @@ void CleanUp()
    CleanUpMeasureTooltip();
    CleanUpSpecial();
    CleanUpTalkToSelfFiledes();
+   CleanUpTidget();
 
    if (iconWindowCreated) {
       XDestroyWindow(mainDisplay, iconBaseWindow);
@@ -396,8 +402,9 @@ void ExposeEventHandler(input, recursive)
    XEvent *input;
    int recursive;
 {
-   register int i;
+   int i=0;
    XEvent ev;
+   TidgetInfo *handling_pti=NULL;
 
    if (input->xany.window == choiceWindow) {
       while (XCheckWindowEvent(mainDisplay, choiceWindow, ExposureMask, &ev)) ;
@@ -441,6 +448,8 @@ void ExposeEventHandler(input, recursive)
    } else if (input->xany.window == statusWindow) {
       while (XCheckWindowEvent(mainDisplay, statusWindow, ExposureMask, &ev)) ;
       RedrawStatusWindow();
+   } else if (IsChatWindowEvent(input, &handling_pti)) {
+      ChatEventHandler(input, handling_pti);
    } else if (input->xany.window == modeWindow) {
       while (XCheckWindowEvent(mainDisplay, modeWindow, ExposureMask, &ev)) ;
       RedrawModeWindow();
@@ -472,6 +481,8 @@ void ExposeEventHandler(input, recursive)
             }
          }
       }
+   } else if (TidgetManagerHandleEvent(input)) {
+      /* input has been handled by the TidgetManager */
    } else {
       for (i = 0; i < numExtraWins; i++) {
          if (input->xany.window == extraWinInfo[i].window &&
@@ -854,14 +865,15 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
       } else {
          sprintf(full_name, "%s%c%s", curDir, DIR_SEP, file_name);
       }
-      if (obj_file) {
-         SetCurDir(full_name);
-         *curSymDir = '\0';
-      } else {
-         SetCurSymDir(full_name);
+      if (!cmdLineMerge) {
+         if (obj_file) {
+            SetCurDir(full_name);
+            *curSymDir = '\0';
+         } else {
+            SetCurSymDir(full_name);
+         }
+         curFileDefined = TRUE;
       }
-      curFileDefined = TRUE;
-
       SetFileModified(FALSE);
       sprintf(gszMsgBox, TgLoadString(STID_CUR_EMPTY_FILE_IS),
             (short_name ? rest : file_name));
@@ -950,7 +962,9 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
 
          curFileDefined = TRUE;
          if (file_is_remote) {
-            SetCurDir(remote_fname);
+            if (!cmdLineMerge) {
+               SetCurDir(remote_fname);
+            }
             if (!foundGoodStateObject) PasteString(remote_buf, TRUE, TRUE);
          } else {
             if (*gzipped_fname == '\0') {
@@ -967,11 +981,13 @@ int JustLoadAFile(pszFile, pnNeedToCheckAutoExec, nAttemptToMerge, pnAbortMerge)
                         gzipped_fname);
                }
             }
-            if (obj_file) {
-               SetCurDir(full_name);
-               *curSymDir = '\0';
-            } else {
-               SetCurSymDir(full_name);
+            if (!cmdLineMerge) {
+               if (obj_file) {
+                  SetCurDir(full_name);
+                  *curSymDir = '\0';
+               } else {
+                  SetCurSymDir(full_name);
+               }
             }
          }
          if (file_is_remote) {
@@ -1048,7 +1064,7 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    int *pnNeedToCheckAutoExec;
 {
    char *c_ptr=NULL;
-   int cannot_opendisplay=FALSE;
+   int cannot_opendisplay=FALSE, allow_open=TRUE;
 #ifdef USE_XAPPLRESDIR
    FILE *xenv_fp=NULL;
    char tmp_buf[MAXPATHLENGTH+1];
@@ -1146,6 +1162,7 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    signal(SIGBUS, EmergencySave);
 #endif /* ~linux */
    signal(SIGSEGV, EmergencySave);
+   signal(SIGCHLD, SIG_IGN);
 
 #ifndef _NO_LOCALE_SUPPORT
    MySetLocale(LC_ALL, "C");
@@ -1160,13 +1177,48 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    prTgifFoundColorInfo = FALSE;
 
    if (pszFile == NULL) {
+      /* Only gets here if PRTGIF of UrlDump() */
       DelAllPages();
       lastPageNum = 1;
       InitPage();
       return TRUE;
    }
+   InitWhiteBoard();
    initializingMain = TRUE;
 
+   if (gstWBInfo.do_whiteboard) {
+      static int initialized=FALSE;
+
+#ifdef _TGIF_DBG /* debug, do not translate */
+      TgAssert(!initialized,
+            "Whiteboard initialized too many times in JustInit()", NULL);
+#endif /* _TGIF_DBG */
+      if (gstWBInfo.join_session_in_progress) {
+         WBCompleteJoin();
+         allow_open = FALSE;
+      }
+      initialized = TRUE;
+   }
+   if (!allow_open) {
+      char *fname=NULL;
+
+      *gszMsgBox = '\0';
+      if (cmdLineMerge && cmdLineMergeArgc > 0 && cmdLineMergeArgv != NULL) {
+         fname = cmdLineMergeArgv[0];
+         sprintf(gszMsgBox, TgLoadString(STID_JOIN_WB_IN_PROGRESS_IGNORED),
+               fname);
+         cmdLineMerge = FALSE;
+      } else if (*pszFile != '\0') {
+         fname = pszFile;
+         sprintf(gszMsgBox, TgLoadString(STID_JOIN_WB_IN_PROGRESS_IGNORED),
+               fname);
+         *pszFile = '\0';
+      }
+      if (fname != NULL) {
+         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         UtilFree(fname);
+      }
+   }
    if (cmdLineMerge && cmdLineMergeArgc > 0 && cmdLineMergeArgv != NULL) {
       int arg_index=0, abort_merge=FALSE;
 
@@ -1227,9 +1279,13 @@ int JustInit(pszFile, pnNeedToCheckAutoExec)
    } else if (*pszFile != '\0') {
       JustLoadAFile(pszFile, pnNeedToCheckAutoExec, FALSE, NULL);
    } else {
-      DelAllPages();
-      lastPageNum = 1;
-      InitPage();
+      if (allow_open) {
+         DelAllPages();
+         lastPageNum = 1;
+         InitPage();
+      } else {
+         SetFileModified(TRUE);
+      }
    }
    SetDefaultDrawWinClipRecs();
    UpdateDirInfo();
@@ -1260,76 +1316,82 @@ int GetAnXEvent(pXEvent)
    fprintf(stderr,"BlockRemoteCmdDepth (GetAnXEvent): %d\n",
          gstWBInfo.BlockRemoteCmdDepth);
 #endif /* DEBUG0 */
-   if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
-         drawWindow)) {
-      if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+   if (XPending(mainDisplay)) {
+      if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+            drawWindow)) {
+         if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+         }
+      } else {
+         XNextEvent(mainDisplay, pXEvent);
       }
    } else {
-      if (XPending(mainDisplay)) {
-         XNextEvent(mainDisplay, pXEvent);
-      } else {
-         struct timeval timeout;
-         fd_set fdset;
-         int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
+      struct timeval timeout;
+      fd_set fdset;
+      int select_width=XConnectionNumber(mainDisplay)+1, status=0, nfds=0;
 
-         nfds = max(select_width, (int)(talkToSelfFiledes[0]+1));
+      nfds = max(select_width, (int)(talkToSelfFiledes[0]+1));
 
-         timeout.tv_sec = 15;
-         timeout.tv_usec = 0;
+      timeout.tv_sec = 15;
+      timeout.tv_usec = 0;
 
-         FD_ZERO(&fdset);
-         FD_SET(select_width-1, &fdset); /* X file descriptor */
-         FD_SET(talkToSelfFiledes[0], &fdset); /* pipe */
+      FD_ZERO(&fdset);
+      FD_SET(select_width-1, &fdset); /* X file descriptor */
+      FD_SET(talkToSelfFiledes[0], &fdset); /* pipe */
 #ifdef __hpux
-         status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
+      status = select(nfds, (int*)&fdset, NULL, NULL, &timeout);
 #else /* !__hpux */
-         status = select(nfds, &fdset, NULL, NULL, &timeout);
+      status = select(nfds, &fdset, NULL, NULL, &timeout);
 #endif /* __hpux */
-         if (status < 0) {
-            sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
-                  "WaitForEvent()"),
-            fprintf(stderr, "%s\n", gszMsgBox);
-            rc = FALSE;
-         } else if (status == 0) {
-            /* timeout */
-            rc = FALSE;
-         } else if (status == 1) {
-            if (FD_ISSET(talkToSelfFiledes[0], &fdset)) {
-               char buf[2];
+      if (status < 0) {
+         sprintf(gszMsgBox, TgLoadString(STID_FUNC_SELECT_SYS_CALL_FAILED),
+               "GetAnXEvent()"),
+         fprintf(stderr, "%s\n", gszMsgBox);
+         rc = FALSE;
+      } else if (status == 0) {
+         /* timeout */
+         rc = FALSE;
+      } else if (status == 1) {
+         if (FD_ISSET(talkToSelfFiledes[0], &fdset)) {
+            char buf[2];
 
-               if (read(talkToSelfFiledes[0], buf, 1) != 1) {
+            if (read(talkToSelfFiledes[0], buf, 1) != 1) {
+               sprintf(gszMsgBox,
+                     TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
+                     TOOL_NAME),
+               fprintf(stderr, "%s\n", gszMsgBox);
+            } else {
+               if (*buf == 'a') {
+                  SendCommandToSelf(CMDID_DATA_IN_MBUFF, 0);
+               } else if (*buf == 'c') {
+                  /*
+                   * Get current tgwb state and establish
+                   *         a TCP conection to send the data.
+                   */
+                  HandleNewUserRequest();
+               } else {
                   sprintf(gszMsgBox,
-                        TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
+                        TgLoadString(STID_UNRECOG_BYTE_FROM_SELF_PIPE), *buf,
                         TOOL_NAME),
                   fprintf(stderr, "%s\n", gszMsgBox);
-               } else {
-                  if (*buf == 'a') {
-                     SendCommandToSelf(CMDID_DATA_IN_MBUFF, 0);
-                  } else if (*buf == 'c') {
-                     /*
-                      * Get current tgwb state and establish
-                      *         a TCP conection to send the data.
-                      */
-                     HandleNewUserRequest();
-                  } else {
-                     sprintf(gszMsgBox,
-                           TgLoadString(STID_READ_FROM_SELF_PIPE_FAIL), 1,
-                           TOOL_NAME),
-                     fprintf(stderr, "%s\n", gszMsgBox);
-                  }
                }
-               rc = FALSE;
-            } else if (FD_ISSET(select_width-1, &fdset)) {
-               XNextEvent(mainDisplay, pXEvent);
+            }
+            rc = FALSE;
+         } else if (FD_ISSET(select_width-1, &fdset)) {
+            if (gnInputMethod != TGIM_NONE && tgIMExpectNextEvent(mainDisplay,
+                  drawWindow)) {
+               if (!tgIMHandleNextEvent(mainDisplay, drawWindow, pXEvent)) {
+               }
             } else {
-               rc = FALSE;
+               XNextEvent(mainDisplay, pXEvent);
             }
          } else {
-            sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
-                  "GetAnXEvent()", status);
-            fprintf(stderr, "%s\n", gszMsgBox);
             rc = FALSE;
          }
+      } else {
+         sprintf(gszMsgBox, TgLoadString(STID_FUNC_INVALID_RC_FOR_SELECT),
+               "GetAnXEvent()", status);
+         fprintf(stderr, "%s\n", gszMsgBox);
+         rc = FALSE;
       }
    }
    gstWBInfo.BlockRemoteCmdDepth++;
@@ -1358,6 +1420,7 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
 {
    XEvent input;
    int rc=0;
+   TidgetInfo *handling_pti=NULL;
 
    exitNormally = FALSE;
    if (pnNeedToCheckAutoExec != NULL && (*pnNeedToCheckAutoExec) &&
@@ -1374,16 +1437,20 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
    }
 
    if (input.type == FocusIn) {
-      if (canvasFontDoubleByte && curChoice == DRAWTEXT && textCursorShown) {
-         tgIMFocusIn(mainDisplay, mainWindow);
+      if (curChoice == DRAWTEXT && textCursorShown) {
+         tgIMFocusIn(mainDisplay, input.xany.window);
       }
    } else if (input.type == FocusOut) {
-      if (canvasFontDoubleByte && curChoice == DRAWTEXT && textCursorShown) {
-         tgIMFocusOut(mainDisplay, mainWindow);
+      if (curChoice == DRAWTEXT && textCursorShown) {
+         tgIMFocusOut(mainDisplay, input.xany.window);
       }
    } else if (input.type == KeyRelease) {
       return 0;
    } else if (input.type == KeyPress) {
+      if (TidgetManagerWantAllKeyPressEvents()) {
+         TidgetManagerHandleAllKeyPressEvent(&input);
+         return 0;
+      }
       rc = ShortHand(&input);
       switch (rc) {
       case BAD: /* <CONTROL> or <META> */ return 0;
@@ -1449,6 +1516,8 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       UserRedrawEventHandler(&input);
    } else if (statusWindow != None && input.xany.window == statusWindow) {
       StatusEventHandler(&input);
+   } else if (chatWindow != None && IsChatWindowEvent(&input, &handling_pti)) {
+      ChatEventHandler(&input, handling_pti);
    } else if (modeWindow != None && input.xany.window == modeWindow) {
       if ((rc=ModeEventHandler(&input)) != INVALID) {
          return rc;
@@ -1466,6 +1535,8 @@ int TryProcessAnXEvent(pnNeedToCheckAutoExec)
       ColorEventHandler(&input);
    } else if (input.xany.window == colorDummyWindow) {
       ColorDummyEventHandler(&input);
+   } else if (TidgetManagerHandleEvent(&input)) {
+      /* input has been handled by the TidgetManager */
    } else {
       int i=0;
 
@@ -1513,7 +1584,7 @@ int TryProcessAnAbortXEvent(esc_to_abort, pf_abort_callback, p_void)
       } else if (!esc_to_abort && has_ch) {
          unsigned int state=key_ev->state;
 
-         if ((state & ControlMask) && (!(state & Mod1Mask)) && s[0] == '\003') {
+         if ((state & ControlMask) && (!(state & METAMASK)) && s[0] == '\003') {
             if (pf_abort_callback == NULL || (pf_abort_callback)(p_void)) {
                return TRUE;
             }
