@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.111 2002/03/21 07:44:55 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/menuinfo.c,v 4.119 2003/08/16 19:38:52 william Exp $
  */
 
 #define _INCLUDE_FROM_MENUINFO_C_
@@ -281,6 +281,12 @@ static TgMenuItemInfo stackedPageMenuItemInfo[] = {
      NULL, CMDID_ADDPAGEBEFORE },
    { N_("Add Page After..."), NULL,
      N_("Add a page after the current page"), NULL, CMDID_ADDPAGEAFTER },
+   { N_("Add Page Before (Copy All)..."), NULL,
+     N_("Add a page before the current page and copy everything on the current page to the new page"),
+     NULL, CMDID_ADDPAGEBEFORECOPYALL },
+   { N_("Add Page After (Copy All)..."), NULL,
+     N_("Add a page after the current page and copy everything on the current page to the new page"),
+     NULL, CMDID_ADDPAGEAFTERCOPYALL },
    { N_("Delete Current Page"), NULL, N_("Delete the current page"), NULL,
      CMDID_DELETECURPAGE },
    { N_("Delete Pages..."), NULL, N_("Delete user-specified pages"), NULL,
@@ -532,6 +538,12 @@ static TgMenuItemInfo importMenuItemInfo[] = {
      CMDID_IMPORTPNGFILE },
    { N_("Import JPEG File..."), NULL, N_("Embed/Import a JPEG file"), NULL,
      CMDID_IMPORTJPEGFILE },
+   { N_("Import PBM File..."), NULL, N_("Embed/Import a PBM file"), NULL,
+     CMDID_IMPORTPBMFILE },
+   { N_("Import PGM File..."), NULL, N_("Embed/Import a PGM file"), NULL,
+     CMDID_IMPORTPGMFILE },
+   { N_("Import PPM File..."), NULL, N_("Embed/Import a PPM file"), NULL,
+     CMDID_IMPORTPPMFILE },
    { N_("Import Other File..."), NULL, N_("Embed/Import another type of file"),
      NULL, CMDID_IMPORTOTHERFILE },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
@@ -627,6 +639,9 @@ static TgMenuItemInfo fileMenuItemInfo[] = {
    { N_("Save Selected As..."), "Alt+~",
      N_("Save selected objects in a different file"), NULL,
      CMDID_SAVESELECTEDAS },
+   { N_("Save Pages As..."), NULL,
+     N_("Save user-specified pages in a different file"), NULL,
+     CMDID_SAVEPAGESAS },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Print"), "Cntrl+p", N_("Print/export current drawing"), NULL,
      CMDID_PRINT },
@@ -828,6 +843,16 @@ static TgMenuItemInfo textMenuItemInfo[] = {
    { N_("Insert Center Subscript"), NULL,
      N_("Insert subscript below the text cursor"), NULL,
      CMDID_INSERTCENTERSUBSCRIPT },
+#ifdef _TGIF_DBG
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Insert Hex/Octal Character..."), NULL,
+     N_("Insert a character using hex or octal code"), NULL,
+     CMDID_INSERTHEXOCTALCHAR },
+#endif /* _TGIF_DBG */
+   { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
+   { N_("Reset input method..."), NULL,
+     N_("Reset connection to the input method (in case it is stuck)"), NULL,
+     CMDID_RESETINMETHOD },
    { TGMUITEM_SEPARATOR, NULL, NULL, NULL, INVALID },
    { N_("Insert Thin Space..."), NULL,
      N_("Insert a think space at the text cursor"), NULL,
@@ -1127,6 +1152,10 @@ static TgMenuItemInfo showHideMenuItemInfo[] = {
      CMDID_TOGGLESHOWSTATUS },
    { N_("Show Mode"), NULL, N_("Show mode window"), NULL,
      CMDID_TOGGLESHOWMODE },
+#ifdef _TGIF_WB2
+   { N_("Show Chat"), NULL, N_("Show chat window"), NULL,
+     CMDID_TOGGLESHOWCHAT },
+#endif /* _TGIF_WB2 */
    { NULL, NULL, NULL, NULL, INVALID }
 };
 static TgMenuInfo showHideMenuInfo={ TGMUTYPE_TEXT, showHideMenuItemInfo,
