@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/move.c,v 4.20 2002/03/07 18:11:33 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/move.c,v 4.23 2003/02/25 01:17:46 william Exp $
  */
 
 #define _INCLUDE_FROM_MOVE_C_
@@ -511,18 +511,14 @@ int ConstrainedMoveAllSel(Dx, Dy, ltx, lty, rbx, rby)
 void MoveAllSel(Dx, Dy)
    register int Dx, Dy;
 {
-   int ltx, lty, rbx, rby;
-   struct MoveSubCmdRec *move_cmd;
-   struct SubCmdRec *sub_cmd;
+   int ltx=0, lty=0, rbx=0, rby=0;
+   struct SubCmdRec *sub_cmd=NULL;
 
-   move_cmd = (struct MoveSubCmdRec *)malloc(sizeof(struct MoveSubCmdRec));
    sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
-   if (move_cmd == NULL || sub_cmd == NULL) FailAllocMessage();
-   memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
+   if (sub_cmd == NULL) FailAllocMessage();
    memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-   sub_cmd->detail.mv = move_cmd;
-   move_cmd->dx = Dx;
-   move_cmd->dy = Dy;
+   sub_cmd->detail.move.dx = Dx;
+   sub_cmd->detail.move.dy = Dy;
 
    if (moveMode==CONST_MOVE && !justDupped) {
       MarkObjectsForMove();
@@ -555,7 +551,6 @@ void MoveAllSel(Dx, Dy)
             selLtX-GRID_ABS_SIZE(1)+Dx, selLtY-GRID_ABS_SIZE(1)+Dy,
             selRbX+GRID_ABS_SIZE(1)+Dx, selRbY+GRID_ABS_SIZE(1)+Dy);
    }
-   free(move_cmd);
    free(sub_cmd);
 }
 
@@ -575,8 +570,7 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
       struct SelRec *saved_top_sel=topSel, *saved_bot_sel=botSel;
       int ltx=ObjPtr->bbox.ltx, lty=ObjPtr->bbox.lty;
       int rbx=ObjPtr->bbox.rbx, rby=ObjPtr->bbox.rby;
-      struct MoveSubCmdRec *move_cmd;
-      struct SubCmdRec *sub_cmd;
+      struct SubCmdRec *sub_cmd=NULL;
 
       topSel = botSel = (struct SelRec *)malloc(sizeof(struct SelRec));
       if (topSel == NULL) FailAllocMessage();
@@ -585,15 +579,11 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
       topSel->obj = ObjPtr;
       UpdSelBBox();
 
-      move_cmd = (struct MoveSubCmdRec *)malloc(
-            sizeof(struct MoveSubCmdRec));
       sub_cmd = (struct SubCmdRec *)malloc(sizeof(struct SubCmdRec));
-      if (move_cmd == NULL || sub_cmd == NULL) FailAllocMessage();
-      memset(move_cmd, 0, sizeof(struct MoveSubCmdRec));
+      if (sub_cmd == NULL) FailAllocMessage();
       memset(sub_cmd, 0, sizeof(struct SubCmdRec));
-      sub_cmd->detail.mv = move_cmd;
-      move_cmd->dx = Dx;
-      move_cmd->dy = Dy;
+      sub_cmd->detail.move.dx = Dx;
+      sub_cmd->detail.move.dy = Dy;
 
       MoveAllSelObjects(Dx, Dy);
       PrepareToRecord(CMD_MOVE, topSel, botSel, numObjSelected);
@@ -603,7 +593,6 @@ void MoveAnObj(ObjPtr, TopOwner, Dx, Dy)
             ltx-GRID_ABS_SIZE(1)+Dx, lty-GRID_ABS_SIZE(1)+Dy,
             rbx+GRID_ABS_SIZE(1)+Dx, rby+GRID_ABS_SIZE(1)+Dy);
 
-      free(move_cmd);
       free(sub_cmd);
 
       free(topSel);
@@ -2009,7 +1998,7 @@ void MoveSelVs(OrigX, OrigY)
                            drawOrigX, drawOrigY, num_pts, pv);
                      break;
                   case LT_INTSPLINE:
-                     free(poly_ptr->intvlist);
+                     if (poly_ptr->intvlist != NULL) free(poly_ptr->intvlist);
                      poly_ptr->svlist = MakeIntSplinePolyVertex(
                            &(poly_ptr->sn), &(poly_ptr->intn),
                            &(poly_ptr->intvlist), drawOrigX, drawOrigY,
@@ -2043,7 +2032,9 @@ void MoveSelVs(OrigX, OrigY)
                            drawOrigX, drawOrigY, num_pts, pv);
                      break;
                   case LT_INTSPLINE:
-                     free(polygon_ptr->intvlist);
+                     if (polygon_ptr->intvlist != NULL) {
+                        free(polygon_ptr->intvlist);
+                     }
                      polygon_ptr->svlist =
                            MakeIntSplinePolygonVertex(&(polygon_ptr->sn),
                            &(polygon_ptr->intn), &(polygon_ptr->intvlist),
