@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.62 2002/03/10 23:28:53 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/msg.c,v 4.80 2003/08/16 19:38:52 william Exp $
  */
 
 #define _INCLUDE_FROM_MSG_C_
@@ -36,7 +36,9 @@
 #include "tgifdefs.h"
 #include "patchlvl.h"
 
+#include "auxtext.e"
 #include "button.e"
+#include "chat.e"
 #include "cutpaste.e"
 #include "cursor.e"
 #include "dialog.e"
@@ -57,6 +59,7 @@
 #include "remote.e"
 #include "setup.e"
 #include "strtbl.e"
+#include "text.e"
 #include "util.e"
 #include "version.e"
 #include "wb.e"
@@ -212,19 +215,6 @@ int TgAssert(must_be_true, msg1, msg2)
    return FALSE;
 }
 
-int BoldMsgDoubleByte(pszStr)
-   char *pszStr;
-{
-#ifdef ENABLE_NLS
-   if (boldMsgFontDoubleByte) {
-      if ((*pszStr) & 0x80) {
-         return TRUE;
-      }
-   }
-#endif /* ENABLE_NLS */
-   return FALSE;
-}
-
 int BoldMsgCharBytes(pszStr)
    char *pszStr;
 {
@@ -364,6 +354,32 @@ int BoldMsgTextWidth(font_ptr, psz_str, len)
    return XTextWidth(font_ptr, psz_str, len);
 }
 
+int ItalicMsgTextWidth(font_ptr, psz_str, len)
+   XFontStruct *font_ptr;
+   char *psz_str;
+   int len;
+{
+#ifdef ENABLE_NLS
+   if (italicMsgFontSet != NULL) {
+      return XmbTextEscapement(italicMsgFontSet, psz_str, len);
+   }
+#endif /* ENABLE_NLS */
+   return XTextWidth(font_ptr, psz_str, len);
+}
+
+int BoldItalicMsgTextWidth(font_ptr, psz_str, len)
+   XFontStruct *font_ptr;
+   char *psz_str;
+   int len;
+{
+#ifdef ENABLE_NLS
+   if (boldItalicMsgFontSet != NULL) {
+      return XmbTextEscapement(boldItalicMsgFontSet, psz_str, len);
+   }
+#endif /* ENABLE_NLS */
+   return XTextWidth(font_ptr, psz_str, len);
+}
+
 void DrawMenuString(dpy, win, gc, x, baseline_y, menu_str, len)
    Display *dpy;
    Window win;
@@ -412,6 +428,40 @@ void DrawBoldMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
    XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
 }
 
+void DrawItalicMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
+   Display *dpy;
+   Window win;
+   GC gc;
+   int x, baseline_y, len;
+   char *msg_str;
+{
+#ifdef ENABLE_NLS
+   if (italicMsgFontSet != NULL) {
+      XmbDrawString(dpy, win, italicMsgFontSet, gc, x, baseline_y, msg_str,
+            len);
+      return;
+   }
+#endif /* ENABLE_NLS */
+   XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
+}
+
+void DrawBoldItalicMsgString(dpy, win, gc, x, baseline_y, msg_str, len)
+   Display *dpy;
+   Window win;
+   GC gc;
+   int x, baseline_y, len;
+   char *msg_str;
+{
+#ifdef ENABLE_NLS
+   if (boldItalicMsgFontSet != NULL) {
+      XmbDrawString(dpy, win, boldItalicMsgFontSet, gc, x, baseline_y, msg_str,
+            len);
+      return;
+   }
+#endif /* ENABLE_NLS */
+   XDrawString(dpy, win, gc, x, baseline_y, msg_str, len);
+}
+
 static
 void AddMsg(Msg)
    char *Msg;
@@ -839,37 +889,30 @@ void SetStringStatus(StatusStr)
    }
 }
 
-typedef struct StatusInfoRec {
-   char btn_str[MAX_STATUS_BTNS][MAXSTRING+1];
-   char one_line_str[MAXSTRING+1];
-   int one_line_status;
-   struct StatusInfoRec *next;
-} *StatusInfoPtr;
-
-static struct StatusInfoRec *topStatusInfo=NULL;
+static MouseOverStatusInfo *topStatusInfo=NULL;
 
 void SaveStatusStrings()
 {
-   struct StatusInfoRec *psi;
-   int i;
+   MouseOverStatusInfo *pmosi=NULL;
+   int i=0;
 
    if (PRTGIF || noStatusWindow) return;
 
-   psi = (struct StatusInfoRec *)malloc(sizeof(struct StatusInfoRec));
-   if (psi == NULL) FailAllocMessage();
-   psi->next = topStatusInfo;
+   pmosi = (MouseOverStatusInfo *)malloc(sizeof(MouseOverStatusInfo));
+   if (pmosi == NULL) FailAllocMessage();
+   pmosi->next = topStatusInfo;
    for (i=0; i < MAX_STATUS_BTNS; i++) {
-      UtilStrCpyN(psi->btn_str[i], MAXSTRING+1, btnStatusStr[i]);
+      UtilStrCpyN(pmosi->btn_str[i], MAXSTRING+1, btnStatusStr[i]);
    }
-   UtilStrCpyN(psi->one_line_str, MAXSTRING+1, oneLineStatusStr);
-   psi->one_line_status = oneLineStatus;
-   topStatusInfo = psi;
+   UtilStrCpyN(pmosi->one_line_str, MAXSTRING+1, oneLineStatusStr);
+   pmosi->one_line_status = oneLineStatus;
+   topStatusInfo = pmosi;
 }
 
 void RestoreStatusStrings()
 {
-   struct StatusInfoRec *psi;
-   int i;
+   MouseOverStatusInfo *pmosi=NULL;
+   int i=0;
 
    if (PRTGIF || noStatusWindow || topStatusInfo == NULL) return;
 
@@ -886,9 +929,9 @@ void RestoreStatusStrings()
    } else {
       RedrawStatusWindow();
    }
-   psi = topStatusInfo->next;
+   pmosi = topStatusInfo->next;
    free(topStatusInfo);
-   topStatusInfo = psi;
+   topStatusInfo = pmosi;
 }
 
 void SaveStatusStringsIntoBuf(ppsz_buf, pn_one_line)
@@ -1121,10 +1164,10 @@ void InitStatus()
 void CleanUpStatus()
 {
    while (topStatusInfo != NULL) {
-      struct StatusInfoRec *next_si=topStatusInfo->next;
+      MouseOverStatusInfo *next_mosi=topStatusInfo->next;
 
       free(topStatusInfo);
-      topStatusInfo = next_si;
+      topStatusInfo = next_mosi;
    }
    if (redraw_bits) { }
    if (btn1_bits) { }
@@ -1201,6 +1244,60 @@ int ScanValue(fmt, v, item, stype)
    return 0;
 }
 
+int ScanDynStrValue(v, item, stype)
+   struct DynStrRec *v;
+   char *item, *stype;
+   /*
+    * 
+    */
+{
+   char *c_ptr=NULL, *psz=NULL, msg[MAXPATHLENGTH];
+
+   if (scanVal == NULL) return INVALID;
+#ifdef DEBUG /* debug, do not translate */
+   printf("get %s for %s from %s:  ", item, stype, scanVal);
+#endif
+
+   if ((c_ptr=GetString()) == NULL) {
+#ifdef DEBUG /* debug, do not translate */
+      printf("nothing\n");
+#endif
+      (void)sprintf(msg, TgLoadString(STID_FILE_LINE_MISS_FIELD_IN_OBJ),
+            scanFileName, scanLineNum, item, stype);
+      if (PRTGIF) {
+         fprintf(stderr, "%s\n", msg);
+      } else {
+         Msg(msg);
+      }
+      return INVALID;
+   }
+   while (*c_ptr != '"' && *c_ptr != '\0') c_ptr++;
+   if (c_ptr != NULL) {
+      psz = ReadString(++c_ptr);
+   }
+   if (psz == NULL) {
+#ifdef DEBUG /* debug, do not translate */
+      printf("error in %s\n", c_ptr);
+#endif
+      (void)sprintf(msg, TgLoadString(STID_FILE_LINE_BAD_FIELD_IN_OBJ),
+            scanFileName, scanLineNum, item, stype, c_ptr);
+      if (PRTGIF) {
+         fprintf(stderr, "%s\n", msg);
+      } else {
+         Msg(msg);
+      }
+      return INVALID;
+   }
+   *(--psz) = '\0';
+   DynStrSet(v, c_ptr);
+
+#ifdef DEBUG /* debug, do not translate */
+   printf("got \"%s\"\n", ((struct DynStrRec *)v)->s);
+#endif
+
+   return 0;
+}
+
 #define MAXEMERGENCYCOUNT 5
 
 static int	emergencyCount = 0;
@@ -1378,62 +1475,64 @@ void Usage(tool_name)
    char buf[MAXSTRING];
 
    GetTgifVersionAndPatchLevel(buf, sizeof(buf));
-   fprintf(stderr, "%s\n", buf);
+   fprintf(stdout, "%s\n", buf);
 
-   fprintf(stderr, "%s\n\n", copyrightString);
-   fprintf(stderr, "Usage:\t%s \\\n", tool_name);
-   fprintf(stderr, "\t[-display displayname] \\\n");
-   fprintf(stderr, "\t[-fg <color>] \\\n");
-   fprintf(stderr, "\t[-bg <color>] \\\n");
-   fprintf(stderr, "\t[-bd <color>] \\\n");
-   fprintf(stderr, "\t[-rv] [-nv] \\\n");
-   fprintf(stderr, "\t[-bw] \\\n");
-   fprintf(stderr, "\t[-reqcolor] \\\n");
-   fprintf(stderr, "\t[-cwo[+sbwarp]] \\\n");
-   fprintf(stderr, "\t[-hyper] \\\n");
-   fprintf(stderr, "\t[-a4] \\\n");
-   fprintf(stderr, "\t[-geometry <geom>] [=<geom>] \\\n");
-   fprintf(stderr, "\t[-exec <file>] \\\n");
-   fprintf(stderr, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
-   fprintf(stderr, "\t[-usexlib] \\\n");
-   fprintf(stderr, "\t[-listdontreencode] \\\n");
-   fprintf(stderr, "\t[-version] \\\n");
-   fprintf(stderr, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
+   fprintf(stdout, "%s\n\n", copyrightString);
+   fprintf(stdout, "Usage:\t%s \\\n", tool_name);
+   fprintf(stdout, "\t[-display displayname] \\\n");
+   fprintf(stdout, "\t[-fg <color>] \\\n");
+   fprintf(stdout, "\t[-bg <color>] \\\n");
+   fprintf(stdout, "\t[-bd <color>] \\\n");
+   fprintf(stdout, "\t[-rv] [-nv] \\\n");
+   fprintf(stdout, "\t[-bw] \\\n");
+   fprintf(stdout, "\t[-reqcolor] \\\n");
+   fprintf(stdout, "\t[-cwo[+sbwarp]] \\\n");
+   fprintf(stdout, "\t[-hyper] \\\n");
+   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-geometry <geom>] [=<geom>] \\\n");
+   fprintf(stdout, "\t[-exec <file>] \\\n");
+   fprintf(stdout, "\t[-dbim {xcin|chinput|xim|kinput2}] \\\n");
+   fprintf(stdout, "\t[-sbim xim] \\\n");
+   fprintf(stdout, "\t[-usexlib] \\\n");
+   fprintf(stdout, "\t[-listdontreencode] \\\n");
+   fprintf(stdout, "\t[-version | --version] \\\n");
+   fprintf(stdout, "\t[-nomode] \\\n");
+   fprintf(stdout, "\t[{<file>[.%s] | -merge <file1>[.%s] <file2>[.%s] ...}]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT, OBJ_FILE_EXT);
-   fprintf(stderr, "or:\n");
-   fprintf(stderr, "\t%s -print \\\n", tool_name);
-   fprintf(stderr, "\t[-display displayname] \\\n");
-   fprintf(stderr, "\t[-stdout] \\\n");
-   fprintf(stderr, "\t[-eps | -p | -epsi | -tiffepsi] \\\n");
-   fprintf(stderr, "\t[-ps | -f] \\\n");
-   fprintf(stderr, "\t[-text] \\\n");
-   fprintf(stderr, "\t[-gif | -xpm | -xbm | -html] \\\n");
-   fprintf(stderr, "\t[-raw[+h[eaderonly]]] \\\n");
-   fprintf(stderr, "\t[-dosepsfilter [-previewonly]] \\\n");
-   fprintf(stderr, "\t[-status] \\\n");
-   fprintf(stderr, "\t[-gray] \\\n");
-   fprintf(stderr, "\t[-color | -reqcolor] \\\n");
-   fprintf(stderr, "\t[-adobe | -adobe=<number>/<number>] \\\n");
-   fprintf(stderr,
+   fprintf(stdout, "or:\n");
+   fprintf(stdout, "\t%s -print \\\n", tool_name);
+   fprintf(stdout, "\t[-display displayname] \\\n");
+   fprintf(stdout, "\t[-stdout] \\\n");
+   fprintf(stdout, "\t[-eps | -p | -epsi | -tiffepsi] \\\n");
+   fprintf(stdout, "\t[-ps | -f] \\\n");
+   fprintf(stdout, "\t[-text] \\\n");
+   fprintf(stdout, "\t[-gif | -xpm | -xbm | -html] \\\n");
+   fprintf(stdout, "\t[-raw[+h[eaderonly]]] \\\n");
+   fprintf(stdout, "\t[-dosepsfilter [-previewonly]] \\\n");
+   fprintf(stdout, "\t[-status] \\\n");
+   fprintf(stdout, "\t[-gray] \\\n");
+   fprintf(stdout, "\t[-color | -reqcolor] \\\n");
+   fprintf(stdout, "\t[-adobe | -adobe=<number>/<number>] \\\n");
+   fprintf(stdout,
          "\t[-dontreencode=\"<%s>:<%s>:...\" | -listdontreencode] \\\n",
          "fontfamily", "fontfamily");
-   fprintf(stderr, "\t[-version] \\\n");
-   fprintf(stderr, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
-   fprintf(stderr, "\t[-producedby=\"<string>\"] \\\n");
-   fprintf(stderr, "\t[-page <number>] \\\n");
-   fprintf(stderr, "\t[-print_cmd \"<command>\"] \\\n");
-   fprintf(stderr, "\t[-one_file_per_page] \\\n");
-   fprintf(stderr, "\t[-pepsc] \\\n");
-   fprintf(stderr, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
-   fprintf(stderr, "\t[-dontcondense | -condensed] \\\n");
-   fprintf(stderr, "\t[-a4] \\\n");
-   fprintf(stderr, "\t[-quiet] \\\n");
-   fprintf(stderr, "\t[-bop_hook \"<string>\"] \\\n");
-   fprintf(stderr, "\t[-eop_hook \"<string>\"] \\\n");
-   fprintf(stderr, "\t[-tmp_file_mode \"<octal number>\"] \\\n");
-   fprintf(stderr, "\t[-odir] \\\n");
-   fprintf(stderr, "\t[-exec <file>] \\\n");
-   fprintf(stderr, "\t[<file1>[.%s] <file2>[.%s] ...]\n",
+   fprintf(stdout, "\t[-version | --version] \\\n");
+   fprintf(stdout, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
+   fprintf(stdout, "\t[-producedby=\"<string>\"] \\\n");
+   fprintf(stdout, "\t[-page <number>] \\\n");
+   fprintf(stdout, "\t[-print_cmd \"<command>\"] \\\n");
+   fprintf(stdout, "\t[-one_file_per_page] \\\n");
+   fprintf(stdout, "\t[-pepsc] \\\n");
+   fprintf(stdout, "\t[-pdfspd | -pdfspd=true | -pdfspd=false] \\\n");
+   fprintf(stdout, "\t[-dontcondense | -condensed] \\\n");
+   fprintf(stdout, "\t[-a4] \\\n");
+   fprintf(stdout, "\t[-quiet] \\\n");
+   fprintf(stdout, "\t[-bop_hook \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-eop_hook \"<string>\"] \\\n");
+   fprintf(stdout, "\t[-tmp_file_mode \"<octal number>\"] \\\n");
+   fprintf(stdout, "\t[-odir] \\\n");
+   fprintf(stdout, "\t[-exec <file>] \\\n");
+   fprintf(stdout, "\t[<file1>[.%s] <file2>[.%s] ...]\n",
          OBJ_FILE_EXT, OBJ_FILE_EXT);
    exit(-1);
 }
@@ -1451,7 +1550,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
    *outputDir = '\0';
    for (argc--, argv++; argc > 0; argc--, argv++) {
       if (**argv == '-') {
-         if (strcmp(*argv, "-version") == 0) {
+         if (strcmp(*argv, "-version") == 0 ||
+               strcmp(*argv, "--version") == 0) {
             show_copyright = TRUE;
          }
       }
@@ -1552,6 +1652,14 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
             cmdLineWhereToPrint = JPEG_FILE;
             cmdLineColor = TRUE;
             cmdLineOpenDisplay = TRUE;
+         } else if (strcmp(*argv,"-ppm") == 0) {
+            cmdLineWhereToPrint = PPM_FILE;
+            cmdLineColor = TRUE;
+            cmdLineOpenDisplay = TRUE;
+         } else if (strcmp(*argv,"-pbm") == 0) {
+            cmdLineWhereToPrint = PPM_FILE;
+            cmdLineColor = FALSE;
+            cmdLineOpenDisplay = TRUE;
          } else if (strcmp(*argv,"-xpm") == 0) {
             cmdLineWhereToPrint = XBM_FILE;
             cmdLineColor = TRUE;
@@ -1708,7 +1816,8 @@ int ProcessPrTgifOptions(argc, argv, from_prtgif)
 #ifdef _DONT_REENCODE
             fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
 #endif /* _DONT_REENCODE */
-         } else if (strcmp(*argv, "-version") == 0) {
+         } else if (strcmp(*argv, "-version") == 0 ||
+               strcmp(*argv, "--version") == 0) {
             cmdLineVersion = TRUE;
          } else if (strncmp(*argv, "-dontreencode=", 14) == 0) {
             strcpy(cmdLineDontReencode, &(*argv)[14]);
@@ -1859,8 +1968,27 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
             fprintf(stderr, "\n");
             Usage(progName);
          }
+         if (cmdLineHasInputMethod) {
+            fprintf(stderr, "%s\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
+            Usage(progName);
+         }
          strcpy(cmdLineInputMethod, argv[i]);
          cmdLineHasInputMethod = TRUE;
+         cmdLineDoubleByteInputMethod = TRUE;
+      } else if (strcmp(argv[i], "-sbim") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-sbim");
+            fprintf(stderr, "\n");
+            Usage(progName);
+         }
+         if (cmdLineHasInputMethod) {
+            fprintf(stderr, "%s\n", TgLoadString(STID_ONLYONE_DBIM_OR_SBIM));
+            Usage(progName);
+         }
+         strcpy(cmdLineInputMethod, argv[i]);
+         cmdLineHasInputMethod = TRUE;
+         cmdLineDoubleByteInputMethod = FALSE;
       } else if (strcmp(argv[i], "-usexlib") == 0) {
          cmdLineUseXlib = TRUE;
       } else if (strcmp(argv[i], "-merge") == 0) {
@@ -1874,6 +2002,17 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
 #endif /* NOT_DEFINED */
       } else if (strcmp(argv[i], "-no-wb") == 0) {
          cmdLineNoWhiteBoard = TRUE;
+      } else if (strcmp(argv[i], "-no-chat") == 0) {
+         cmdLineNoChatWindow = TRUE;
+      } else if (strcmp(argv[i], "-chatnick") == 0) {
+         if (++i == argc) {
+            fprintf(stderr, TgLoadString(STID_MISSING_ARG_FOR_CMDLINE_OPT),
+                  "-chatnick");
+            fprintf(stderr, "\n");
+            Usage(progName);
+         }
+         cmdLineChatNickName = UtilStrDup(argv[i]);
+         if (cmdLineChatNickName == NULL) FailAllocMessage();
 #endif /* _TGIF_WB2 */
       } else if (strcmp(argv[i], "-fg") == 0) {
          if (++i == argc) {
@@ -1918,8 +2057,11 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
 #ifdef _DONT_REENCODE
          fprintf(stderr, "-D_DONT_REENCODE=\"%s\"\n", _DONT_REENCODE);
 #endif /* _DONT_REENCODE */
-      } else if (strcmp(argv[i], "-version") == 0) {
+      } else if (strcmp(argv[i], "-version") == 0 ||
+            strcmp(argv[i], "--version") == 0) {
          cmdLineVersion = TRUE;
+      } else if (strcmp(argv[i], "-nomode") == 0) {
+         cmdLineNoModeWindow = TRUE;
       } else if (strcmp(argv[i], "-pdfspd") == 0) {
          cmdLinePdfSetPageDevice = TRUE;
       } else if (strncmp(argv[i], "-pdfspd=", 8) == 0) {
@@ -1936,10 +2078,10 @@ int ProcessTgifOptions(argc, argv, file_name, name_sz)
                   "-pdfspd", tmp_str);
             fprintf(stderr, "\n");
          }
-      } else if (argv[i][i] == '=') {
+      } else if (argv[i][0] == '=') {
          strcpy(geometrySpec, argv[i]);
          geometrySpecified = TRUE;
-      } else if (argv[i][i] != '-') {
+      } else if (argv[i][0] != '-') {
          if (cmdLineMerge) {
             int j=0;
 
