@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.40 2002/03/24 21:18:32 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/navigate.c,v 4.45 2003/02/25 01:17:47 william Exp $
  */
 
 #define _INCLUDE_FROM_NAVIGATE_C_
@@ -103,7 +103,11 @@ static int maxURLCache=(-1), curURLCache=0;
 
 static int lineWidthIndexInSlideShow=4;
 static int makeUnsavableInSlideShow=FALSE;
+static int ignoreSlideShowOffsetsInFile=TRUE;
 
+static int savedSlideShowXOffset=0;
+static int savedSlideShowYOffset=0;
+static int savedSlideShowOffsets=FALSE;
 
 int ParseXYSpec(spec, pnx, pny)
    char *spec;
@@ -292,6 +296,10 @@ void CleanUpNavigate()
 
    CleanUpURLCache();
    CleanUpSlideShowInfo();
+
+   savedSlideShowXOffset = 0;
+   savedSlideShowYOffset = 0;
+   savedSlideShowOffsets = FALSE;
 }
 
 void InitNavigate()
@@ -326,6 +334,10 @@ void InitNavigate()
       UtilTrimBlanks(spec);
       if (ParseXYSpec(spec, &slideShowXOffset, &slideShowYOffset)) {
          slideShowInfoValid = TRUE;
+
+         savedSlideShowXOffset = slideShowXOffset;
+         savedSlideShowYOffset = slideShowYOffset;
+         savedSlideShowOffsets = TRUE;
       } else {
          fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
                TOOL_NAME, "SlideShowWindowOffsets", spec, "0,0");
@@ -355,6 +367,12 @@ void InitNavigate()
          NULL && UtilStrICmp(c_ptr, "true") == 0) {
       makeUnsavableInSlideShow = TRUE;
    }
+   ignoreSlideShowOffsetsInFile = TRUE;
+   if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
+         "IgnoreSlideShowOffsetsInFile")) != NULL &&
+         UtilStrICmp(c_ptr, "false") == 0) {
+      ignoreSlideShowOffsetsInFile = FALSE;
+   }
 }
 
 static
@@ -1308,6 +1326,8 @@ void StartSlideShow()
    ClearObjCachesInAllPages();
    HidePopupMenusForSlideShow();
 
+   RecordWBSlideShow(TRUE);
+
    if (exec_attr != NULL) {
       DoExecLoop(NULL, exec_attr);
    }
@@ -1318,6 +1338,8 @@ void EndSlideShow()
 {
    XSetWindowBackground(mainDisplay, mainWindow, myBgPixel);
 
+   RecordWBSlideShow(FALSE);
+
    ShowPopupMenusForSlideShow();
    ClearObjCachesInAllPages();
    if (gstSlideShow.saved_in_hyperspace != inHyperSpace) {
@@ -1341,6 +1363,7 @@ void EndSlideShow()
    MakeQuiescent();
 }
 
+static
 void ToggleSlideShow()
 {
    int need_to_restore_cur_choice=FALSE;
@@ -1397,6 +1420,16 @@ void ToggleSlideShow()
    if (need_to_restore_cur_choice) SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
+void EnterSlideShow()
+{
+   if (!inSlideShow) ToggleSlideShow();
+}
+
+void LeaveSlideShow()
+{
+   if (inSlideShow) ToggleSlideShow();
+}
+
 void SetSlideShowBorderColor()
 {
    char spec[MAXSTRING];
@@ -1447,6 +1480,10 @@ void SetSlideShowWindowOffsets()
       sprintf(gszMsgBox, TgLoadString(STID_SLIDE_WIN_XY_OFFSETS_SET_TO),
             slideShowXOffset, slideShowYOffset);
       Msg(gszMsgBox);
+
+      savedSlideShowXOffset = slideShowXOffset;
+      savedSlideShowYOffset = slideShowYOffset;
+      savedSlideShowOffsets = TRUE;
    }
    slideShowInfoValid = TRUE;
 }
@@ -1600,9 +1637,13 @@ int ReadSlideShowInfo(buf)
          slideShowBorderColor = UtilStrDup(color_str);
          if (slideShowBorderColor == NULL) FailAllocMessage();
       }
-      slideShowXOffset = x;
-      slideShowYOffset = y;
-
+      if (!ignoreSlideShowOffsetsInFile) {
+         slideShowXOffset = x;
+         slideShowYOffset = y;
+      } else if (savedSlideShowOffsets) {
+         slideShowXOffset = savedSlideShowXOffset;
+         slideShowYOffset = savedSlideShowYOffset;
+      }
       slideShowInfoValid = TRUE;
    }
    return TRUE;
