@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.55 2002/01/05 02:48:04 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/page.c,v 4.65 2003/02/25 01:17:47 william Exp $
  */
 
 #define _INCLUDE_FROM_PAGE_C_
@@ -46,6 +46,7 @@
 #include "cursor.e"
 #include "dialog.e"
 #include "drawing.e"
+#include "dup.e"
 #include "file.e"
 #include "font.e"
 #include "grid.e"
@@ -133,6 +134,33 @@ int DrawAHorizontalTab(page_num, page_ptr, x, y, on_top, skip)
    return w;
 }
 
+static
+int PageTabsVisible()
+{
+   int i=0, x=0;
+   struct PageRec *page_ptr=NULL;
+
+   x = 1 + (leftend_width<<2);
+   for (i=1, page_ptr=firstPage; page_ptr != NULL;
+         page_ptr=page_ptr->next, i++) {
+      if (i >= pageWindowFirstIndex) {
+         return TRUE;
+      }
+   }
+   return FALSE;
+}
+
+static
+void MakePageTabsVisible()
+{
+   while (pageWindowFirstIndex > 1) {
+      pageWindowFirstIndex--;
+      if (PageTabsVisible()) {
+         return;
+      }
+   }
+}
+
 void RedrawPageWindow()
 {
    int i, x, y=1;
@@ -144,6 +172,9 @@ void RedrawPageWindow()
          scrollBarW+(brdrW<<1)-1);
    if (pageLayoutMode == PAGE_TILE) return;
 
+   if (!PageTabsVisible()) {
+      MakePageTabsVisible();
+   }
    x = 1 + (leftend_width<<2);
    XSetFont(mainDisplay, defaultGC, rulerFontPtr->fid);
    for (i=1, page_ptr=firstPage; page_ptr != NULL;
@@ -267,7 +298,13 @@ void SetCurPage(page_number)
 #ifdef _TGIF_WB2_NO_DELAY_BUG
       sleep(1);
 #endif /* ~_TGIF_WB2_NO_DELAY_BUG */
-      SelAllObj(FALSE);
+      /* 
+       * In slideshow mode, SelAllObj() will *not* select anything!
+       * Since we really want the objects to be selected (because
+       *         we want to record them), we must set ignore_slideshow
+       *         to TRUE when we call SelAllObj().
+       */
+      SelAllObj(FALSE, TRUE);
       PrepareToRecord(CMD_NEW, NULL, NULL, 0);
       RecordCmd(CMD_NEW, NULL, topSel, botSel, numObjSelected);
       RemoveAllSel();
@@ -866,9 +903,15 @@ int OkToFlushUndoBuffer(msg_prefix)
 #define BEFORE_CUR_PAGE 0
 #define AFTER_CUR_PAGE 1
 
+typedef struct tagAddPageCopyAllInfo {
+   struct ObjRec *top_obj;
+   struct ObjRec *bot_obj;
+} AddPageCopyAllInfo;
+
 static
-void AddPage(AfterCurPage)
+void AddPage(AfterCurPage, papcai)
    int AfterCurPage;
+   AddPageCopyAllInfo *papcai;
 {
    int n;
    struct PageRec *page_ptr=NULL;
@@ -924,29 +967,48 @@ void AddPage(AfterCurPage)
    topObj = curPage->top;
    botObj = curPage->bot;
 
+   if (papcai != NULL) {
+      struct ObjRec *obj_ptr=NULL;
+      struct SelRec *top_sel=NULL, *bot_sel=NULL;
+
+      for (obj_ptr=papcai->bot_obj; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
+         obj_ptr->tmp_parent = NULL;
+         AddObjIntoSel(obj_ptr, NULL, topSel, &topSel, &botSel);
+      }
+      DupTheseObjects(topSel, botSel, &top_sel, &bot_sel);
+      RemoveAllSel();
+      curPage->top = topObj = top_sel->obj;
+      curPage->bot = botObj = bot_sel->obj;
+      topSel = top_sel;
+      botSel = bot_sel;
+      RemoveAllSel();
+   }
    ClearAndRedrawDrawWindow();
    RedrawTitleWindow();
    SetFileModified(TRUE);
    ShowPage();
-   if ((attr_ptr=FindAttrWithName(tgifObj, "template=", NULL)) != NULL) {
-      char file_name[MAXPATHLENGTH];
+   if (papcai == NULL && !cmdLineMerge) {
+      if ((attr_ptr=FindAttrWithName(tgifObj, "template=", NULL)) != NULL) {
+         char file_name[MAXPATHLENGTH];
 
-      if (*attr_ptr->attr_value.s == DIR_SEP) {
-         UtilStrCpyN(file_name, sizeof(file_name), attr_ptr->attr_value.s);
-      } else {
-         sprintf(file_name, "%s%c%s", bootDir, DIR_SEP, attr_ptr->attr_value.s);
-      }
-      importingFile = TRUE;
-      if (ImportGivenFile(file_name, TRUE, TRUE)) {
-         sprintf(gszMsgBox, TgLoadString(STID_NAMED_TEMPLATE_LOADED),
+         if (*attr_ptr->attr_value.s == DIR_SEP) {
+            UtilStrCpyN(file_name, sizeof(file_name), attr_ptr->attr_value.s);
+         } else {
+            sprintf(file_name, "%s%c%s", bootDir, DIR_SEP,
                attr_ptr->attr_value.s);
-         Msg(gszMsgBox);
-      } else {
-         sprintf(gszMsgBox, TgLoadString(STID_PROB_LOADING_NAMED_TEMPLATE),
-               file_name);
-         MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
+         importingFile = TRUE;
+         if (ImportGivenFile(file_name, TRUE, TRUE)) {
+            sprintf(gszMsgBox, TgLoadString(STID_NAMED_TEMPLATE_LOADED),
+                  attr_ptr->attr_value.s);
+            Msg(gszMsgBox);
+         } else {
+            sprintf(gszMsgBox, TgLoadString(STID_PROB_LOADING_NAMED_TEMPLATE),
+                  file_name);
+            MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+         }
+         importingFile = FALSE;
       }
-      importingFile = FALSE;
    }
    sprintf(gszMsgBox, TgLoadString(STID_GIVEN_PAGE_NUM_ADDED), curPageNum);
    Msg(gszMsgBox);
@@ -959,7 +1021,7 @@ void AddPageBefore()
       MsgBox(TgLoadString(STID_ADDPAGEBEFORE_IN_WB), TOOL_NAME, INFO_MB);
       return;
    }
-   AddPage(BEFORE_CUR_PAGE);
+   AddPage(BEFORE_CUR_PAGE, NULL);
 }
 
 void AddPageAfter()
@@ -968,7 +1030,37 @@ void AddPageAfter()
       MsgBox(TgLoadString(STID_ADDPAGEAFTER_IN_WB), TOOL_NAME, INFO_MB);
       return;
    }
-   AddPage(AFTER_CUR_PAGE);
+   AddPage(AFTER_CUR_PAGE, NULL);
+}
+
+void AddPageBeforeCopyAll()
+{
+   AddPageCopyAllInfo apcai;
+
+   if (gstWBInfo.do_whiteboard) {
+      MsgBox(TgLoadString(STID_ADDPAGEBEFORE_IN_WB), TOOL_NAME, INFO_MB);
+      return;
+   }
+   memset(&apcai, 0, sizeof(AddPageCopyAllInfo));
+   apcai.top_obj = topObj;
+   apcai.bot_obj = botObj;
+
+   AddPage(BEFORE_CUR_PAGE, &apcai);
+}
+
+void AddPageAfterCopyAll()
+{
+   AddPageCopyAllInfo apcai;
+
+   if (gstWBInfo.do_whiteboard) {
+      MsgBox(TgLoadString(STID_ADDPAGEAFTER_IN_WB), TOOL_NAME, INFO_MB);
+      return;
+   }
+   memset(&apcai, 0, sizeof(AddPageCopyAllInfo));
+   apcai.top_obj = topObj;
+   apcai.bot_obj = botObj;
+
+   AddPage(AFTER_CUR_PAGE, &apcai);
 }
 
 void DeleteCurPage()
@@ -982,7 +1074,7 @@ void DeleteCurPage()
       return;
    }
    if (lastPageNum == 1) {
-      MsgBox(TgLoadString(STID_CANT_DEL_ONLY_PAGE_IN_TILED), TOOL_NAME,
+      MsgBox(TgLoadString(STID_CANT_DEL_ONLY_PAGE_IN_STACKED), TOOL_NAME,
             INFO_MB);
       return;
    }
@@ -1110,6 +1202,7 @@ void PrintOnePage()
       case HTML_FILE:
       case PNG_FILE:
       case JPEG_FILE:
+      case PPM_FILE:
          sprintf(gszMsgBox, TgLoadString(STID_CANT_PRINTONEPAGE_IN_FORMAT),
                GetExportName(whereToPrint, EXPORT_THIS));
          MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -1462,6 +1555,39 @@ void DeletePages()
    SetCurChoice(curChoiceBeforeMakeQuiescent);
 }
 
+void SavePagesAs()
+{
+   char spec[MAXSTRING+1];
+   SpecifyPagesInfo spi;
+
+   if (pageLayoutMode == PAGE_TILE) {
+      MsgBox(TgLoadString(STID_CANT_DO_SAVEPAGESAS_IN_TILED), TOOL_NAME,
+            INFO_MB);
+      return;
+   }
+   if (lastPageNum == 1) {
+      SaveNewFile(FALSE, NULL);
+      return;
+   }
+   *spec = '\0';
+   Dialog(TgLoadString(STID_SPECIFY_PAGES_TO_SAVE_EX), NULL, spec);
+   UtilTrimBlanks(spec);
+   if (*spec == '\0') return;
+
+   memset(&spi, 0, sizeof(SpecifyPagesInfo));
+   if (!ParsePagesSpec(spec, &spi)) return;
+
+   MakeQuiescent();
+
+   if (spi.num_pages_specified == lastPageNum) {
+      SaveNewFile(FALSE, NULL);
+   } else if (spi.num_pages_specified > 0) {
+      SavePages(&spi);
+   }
+   FreePageSpec(&spi);
+   SetCurChoice(curChoiceBeforeMakeQuiescent);
+}
+
 void PrintPages()
 {
    char spec[MAXSTRING];
@@ -1482,6 +1608,7 @@ void PrintPages()
    case TIFFEPSI_FILE:
    case PNG_FILE:
    case JPEG_FILE:
+   case PPM_FILE:
       sprintf(gszMsgBox, TgLoadString(STID_CANT_PRINT_PAGES_IN_FORMAT),
             GetExportName(whereToPrint, EXPORT_THIS));
       MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
@@ -1996,7 +2123,6 @@ int FixPageNumInStrSeg(pStrSeg, attr_ptr, just_checking)
          return TRUE;
       }
       attr_owner_obj = GetTopOwner(attr_ptr->owner);
-      PrepareToReplaceAnObj(attr_owner_obj);
       while ((c_ptr=strstr(pStrSeg->dyn_str.s, "!(STACKED_PAGE_NUM)")) !=
             NULL) {
          c_ptr1 = &c_ptr[page_num_str_len];
@@ -2014,7 +2140,6 @@ int FixPageNumInStrSeg(pStrSeg, attr_ptr, just_checking)
          *c_ptr = '\0';
       }
       RecursivelyAdjObjBBox(attr_ptr->owner, attr_ptr->owner, attr_owner_obj);
-      RecordReplaceAnObj(attr_owner_obj);
 
       SetFileModified(TRUE);
       return TRUE;
@@ -2146,6 +2271,8 @@ int FixPageNumInObj(ObjPtr, just_checking)
    struct ObjRec *ObjPtr;
    int just_checking;
 {
+   int something_changed=FALSE;
+
    switch (ObjPtr->type) {
    case OBJ_POLY:
    case OBJ_BOX:
@@ -2157,8 +2284,12 @@ int FixPageNumInObj(ObjPtr, just_checking)
    case OBJ_XPM:
       if (!colorLayers ||
             ObjPtr->tmp_parent!=NULL || ObjInVisibleLayer(ObjPtr)) {
-         if (FixPageNumInAttr(ObjPtr->lattr, just_checking) && just_checking) {
-            return TRUE;
+         if (FixPageNumInAttr(ObjPtr->lattr, just_checking)) {
+            if (just_checking) {
+               return TRUE;
+            } else {
+               something_changed = TRUE;
+            }
          }
       }
       break;
@@ -2173,12 +2304,20 @@ int FixPageNumInObj(ObjPtr, just_checking)
 
          for ( ; obj_ptr != NULL; obj_ptr = obj_ptr->prev) {
             obj_ptr->tmp_parent = ObjPtr;
-            if (FixPageNumInObj(obj_ptr, just_checking) && just_checking) {
-               return TRUE;
+            if (FixPageNumInObj(obj_ptr, just_checking)) {
+               if (just_checking) {
+                  return TRUE;
+               } else {
+                  something_changed = TRUE;
+               }
             }
          }
-         if (FixPageNumInAttr(ObjPtr->lattr, just_checking) && just_checking) {
-            return TRUE;
+         if (FixPageNumInAttr(ObjPtr->lattr, just_checking)) {
+            if (just_checking) {
+               return TRUE;
+            } else {
+               something_changed = TRUE;
+            }
          }
       }
       break;
@@ -2188,16 +2327,26 @@ int FixPageNumInObj(ObjPtr, just_checking)
          struct ObjRec *obj_ptr=GetPinObj(ObjPtr);
 
          obj_ptr->tmp_parent = ObjPtr;
-         if (FixPageNumInObj(obj_ptr, just_checking) && just_checking) {
-            return TRUE;
+         if (FixPageNumInObj(obj_ptr, just_checking)) {
+            if (just_checking) {
+               return TRUE;
+            } else {
+               something_changed = TRUE;
+            }
          }
-         if (FixPageNumInAttr(ObjPtr->lattr, just_checking) && just_checking) {
-            return TRUE;
+         if (FixPageNumInAttr(ObjPtr->lattr, just_checking)) {
+            if (just_checking) {
+               return TRUE;
+            } else {
+               something_changed = TRUE;
+            }
          }
       }
       break;
    }
-   return FALSE;
+   if (just_checking) return FALSE;
+
+   return something_changed;
 }
 
 void FixPageNumbersForSlideShow()
@@ -2232,12 +2381,29 @@ void FixPageNumbersForSlideShow()
    StartCompositeCmd();
    for (i=0; i < num_pages_to_fix; i++) {
       page_num = pages_to_fix[i];
-      PrepareToRecord(CMD_GOTO_PAGE, NULL, NULL, curPageNum);
-      GotoPageNum(page_num);
-      RecordCmd(CMD_GOTO_PAGE, NULL, NULL, NULL, curPageNum);
-
+      if (page_num != curPageNum) {
+         PrepareToRecord(CMD_GOTO_PAGE, NULL, NULL, curPageNum);
+         GotoPageNum(page_num);
+         RecordCmd(CMD_GOTO_PAGE, NULL, NULL, NULL, curPageNum);
+      }
+      if (gstWBInfo.do_whiteboard && curPageNum != saved_cur_page_num) {
+         /*
+          * Since the whiteboard is active, others only see
+          *         the current page.  So, changes to other
+          *         pages should not be seen by other whiteboards.
+          */
+         gstWBInfo.dont_serialize = TRUE;
+      }
       for (obj_ptr=botObj; obj_ptr != NULL; obj_ptr=obj_ptr->prev) {
-         FixPageNumInObj(obj_ptr, FALSE);
+         PrepareToReplaceAnObj(obj_ptr);
+         if (FixPageNumInObj(obj_ptr, FALSE)) {
+            RecordReplaceAnObj(obj_ptr);
+         } else {
+            AbortPrepareCmd(CMD_REPLACE);
+         }
+      }
+      if (gstWBInfo.do_whiteboard && curPageNum != saved_cur_page_num) {
+         gstWBInfo.dont_serialize = FALSE;
       }
    }
    if (curPageNum != saved_cur_page_num) {
