@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,13 +28,14 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.18 2002/01/05 02:48:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/polygon.c,v 4.20 2003/02/25 01:17:47 william Exp $
  */
 
 #define _INCLUDE_FROM_POLYGON_C_
 
 #include "tgifdefs.h"
 
+#include "arc.e"
 #include "attr.e"
 #include "auxtext.e"
 #include "box.e"
@@ -541,6 +542,70 @@ void AddPtToCreatePolygon(AbsX, AbsY)
    pt_ptr->y = AbsY;
 }
 
+#define POLYGON_DRAW  (FALSE)
+#define POLYGON_ERASE (TRUE)
+#define POLYGON_CLICK (FALSE)
+#define POLYGON_DRAG  (TRUE)
+
+#define POLYGON_STARTSHOW 0
+#define POLYGON_DOSHOW    1
+#define POLYGON_ENDSHOW   2
+
+static
+void DoPolygonMeasureCursor(start, num_pts, x, y, dx, dy, erase, drag, closed)
+   int start, num_pts, x, y, dx, dy, erase, drag, closed;
+{
+   static IntPoint first_pt={0,0}, prev_prev_pt={0,0}, prev_pt={0,0};
+   char buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80], a_buf[80];
+   int angle2=0;
+
+   if (erase == POLYGON_DRAW && drag == POLYGON_CLICK) {
+      if (num_pts == 1) {
+         first_pt.x = prev_pt.x = prev_prev_pt.x = x;
+         first_pt.y = prev_pt.y = prev_prev_pt.y = y;
+      } else {
+         prev_prev_pt.x = prev_pt.x;
+         prev_prev_pt.y = prev_pt.y;
+         prev_pt.x = x;
+         prev_pt.y = y;
+      }
+   }
+   if (x == prev_pt.x && y == prev_pt.y) {
+      strcpy(a_buf, "0");
+   } else if (num_pts == 1) {
+      PointsToArc(prev_pt.x, prev_pt.y, prev_pt.x+100, prev_pt.y, x, y,
+            ARC_CCW, FALSE, NULL, NULL, NULL, NULL, NULL, &angle2);
+      if (angle2 > 180*64) angle2=(360*64)-angle2;
+      FormatAngle(angle2, a_buf);
+   } else if (closed) {
+      if (x == first_pt.x && y == first_pt.y) {
+         strcpy(a_buf, "0");
+      } else {
+         PointsToArc(x, y, first_pt.x, first_pt.y, prev_pt.x, prev_pt.y,
+               ARC_CCW, FALSE, NULL, NULL, NULL, NULL, NULL, &angle2);
+         if (angle2 > 180*64) angle2=(360*64)-angle2;
+         FormatAngle(angle2, a_buf);
+      }
+   } else {
+      PointsToArc(prev_pt.x, prev_pt.y, prev_prev_pt.x, prev_prev_pt.y, x, y,
+            ARC_CCW, FALSE, NULL, NULL, NULL, NULL, NULL, &angle2);
+      if (angle2 > 180*64) angle2=(360*64)-angle2;
+      FormatAngle(angle2, a_buf);
+   }
+   PixelToMeasurementUnit(w_buf, dx);
+   PixelToMeasurementUnit(h_buf, dy);
+   PixelToMeasurementUnit(x_buf, x);
+   PixelToMeasurementUnit(y_buf, y);
+   sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s\nangle=%s", w_buf, h_buf, x_buf, y_buf,
+         a_buf);
+
+   switch (start) {
+   case POLYGON_STARTSHOW: StartShowMeasureCursor(x, y, buf, TRUE); break;
+   case POLYGON_DOSHOW: ShowMeasureCursor(x, y, buf, TRUE); break;
+   case POLYGON_ENDSHOW: EndShowMeasureCursor(x, y, buf, TRUE); break;
+   }
+}
+
 static XComposeStatus c_stat;
 
 static
@@ -553,7 +618,6 @@ void ContinuePolygon(OrigX, OrigY)
    XButtonEvent *button_ev;
    XMotionEvent *motion_ev;
    KeySym key_sym;
-   char s[80], buf[80], w_buf[80], h_buf[80], x_buf[80], y_buf[80];
    int start_polygon_x=OrigX, start_polygon_y=OrigY;
    int end_x, end_y, grid_x, grid_y, done=FALSE;
    int saved_x, saved_y, closed=FALSE, abort=FALSE;
@@ -592,12 +656,8 @@ void ContinuePolygon(OrigX, OrigY)
    SetMouseStatus(TgLoadCachedString(CSTID_ADD_A_VERTEX),
       TgLoadCachedString(CSTID_CLOSE_POLYGON_SPLINE),
       TgLoadCachedString(CSTID_CLOSE_POLYGON_SPLINE));
-   PixelToMeasurementUnit(w_buf, 0);
-   PixelToMeasurementUnit(h_buf, 0);
-   PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-   PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-   sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
-   StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+   DoPolygonMeasureCursor(POLYGON_STARTSHOW, num_pts, ABS_X(grid_x),
+         ABS_Y(grid_y), 0, 0, POLYGON_DRAW, POLYGON_CLICK, closed);
    XGrabPointer(mainDisplay, drawWindow, FALSE,
          PointerMotionMask | ButtonPressMask,
          GrabModeAsync, GrabModeAsync, None, handCursor, CurrentTime);
@@ -610,25 +670,35 @@ void ContinuePolygon(OrigX, OrigY)
          SetXorDrawGC(xorColorPixels[colorIndex]);
       } else if ((!(shiftForDiagMouseMove && DiagEventCheck(&input))) &&
             (input.type == KeyPress || input.type == KeyRelease)) {
-         int has_ch;
+         char s[80];
+         int has_ch=FALSE;
 
          has_ch = XLookupString(&(input.xkey), s, sizeof(s), &key_sym, &c_stat);
          if (num_pts != 1 && !(curSpline != LT_STRAIGHT && splineRubberband) &&
                (key_sym == XK_Control_L || key_sym == XK_Control_R)) {
             XDrawLine(mainDisplay, drawWindow, drawGC, saved_x, saved_y,
                   grid_x, grid_y);
+            /* erase */
+            DoPolygonMeasureCursor(POLYGON_DOSHOW, num_pts, ABS_X(grid_x),
+                  ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+                  ABS_SIZE(abs(grid_y-OrigY)), POLYGON_ERASE, POLYGON_DRAG,
+                  closed);
             if (ControlKeyPressedEvent(&input)) {
                closed = TRUE;
             } else {
                closed = FALSE;
             }
+            /* draw */
+            DoPolygonMeasureCursor(POLYGON_DOSHOW, num_pts, ABS_X(grid_x),
+                  ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+                  ABS_SIZE(abs(grid_y-OrigY)), POLYGON_DRAW, POLYGON_DRAG,
+                  closed);
          } else if (CharIsESC(&(input.xkey), s, key_sym, &has_ch)) {
-            PixelToMeasurementUnit(w_buf, ABS_SIZE(abs(grid_x-OrigX)));
-            PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(grid_y-OrigY)));
-            PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-            PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-            sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
-            EndShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+            /* erase */
+            DoPolygonMeasureCursor(POLYGON_ENDSHOW, num_pts, ABS_X(grid_x),
+                  ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+                  ABS_SIZE(abs(grid_y-OrigY)), POLYGON_ERASE, POLYGON_CLICK,
+                  closed);
             abort = TRUE;
             done = TRUE;
          }
@@ -637,12 +707,11 @@ void ContinuePolygon(OrigX, OrigY)
             (input.type == KeyPress || input.type == KeyRelease))) {
          unsigned int event_state=0;
 
-         PixelToMeasurementUnit(w_buf, ABS_SIZE(abs(grid_x-OrigX)));
-         PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(grid_y-OrigY)));
-         PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-         PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-         sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
-         ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+         /* erase */
+         DoPolygonMeasureCursor(POLYGON_DOSHOW, num_pts, ABS_X(grid_x),
+               ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), POLYGON_ERASE, POLYGON_DRAG,
+               closed);
          if (curSpline != LT_STRAIGHT && splineRubberband) {
             XDrawLines(mainDisplay, drawWindow, drawGC, sv, sn,
                   CoordModeOrigin);
@@ -713,20 +782,17 @@ void ContinuePolygon(OrigX, OrigY)
             XDrawLine(mainDisplay, drawWindow, drawGC, OrigX, OrigY, grid_x,
                   grid_y);
          }
-         PixelToMeasurementUnit(w_buf, ABS_SIZE(abs(grid_x-OrigX)));
-         PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(grid_y-OrigY)));
-         PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-         PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-         sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
-         ShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+         /* draw */
+         DoPolygonMeasureCursor(POLYGON_DOSHOW, num_pts, ABS_X(grid_x),
+               ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), POLYGON_DRAW, POLYGON_DRAG, closed);
          while (XCheckMaskEvent(mainDisplay, PointerMotionMask, &ev)) ;
       } else if (input.type == ButtonPress) {
-         PixelToMeasurementUnit(w_buf, ABS_SIZE(abs(grid_x-OrigX)));
-         PixelToMeasurementUnit(h_buf, ABS_SIZE(abs(grid_y-OrigY)));
-         PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-         PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-         sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s", w_buf, h_buf, x_buf, y_buf);
-         EndShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+         /* erase */
+         DoPolygonMeasureCursor(POLYGON_ENDSHOW, num_pts, ABS_X(grid_x),
+               ABS_Y(grid_y), ABS_SIZE(abs(grid_x-OrigX)),
+               ABS_SIZE(abs(grid_y-OrigY)), POLYGON_ERASE, POLYGON_DRAG,
+               closed);
          button_ev = &(input.xbutton);
 
          end_x = button_ev->x;
@@ -806,13 +872,9 @@ void ContinuePolygon(OrigX, OrigY)
             switch(button_ev->button) {
             case Button1:
                OrigX = grid_x; OrigY = grid_y;
-               PixelToMeasurementUnit(w_buf, 0);
-               PixelToMeasurementUnit(h_buf, 0);
-               PixelToMeasurementUnit(x_buf, ABS_X(grid_x));
-               PixelToMeasurementUnit(y_buf, ABS_Y(grid_y));
-               sprintf(buf, "w=%s\nh=%s\nx=%s\ny=%s",
-                     w_buf, h_buf, x_buf, y_buf);
-               StartShowMeasureCursor(grid_x, grid_y, buf, TRUE);
+               /* draw */
+               DoPolygonMeasureCursor(POLYGON_STARTSHOW, num_pts, ABS_X(grid_x),
+                     ABS_Y(grid_y), 0, 0, POLYGON_DRAW, POLYGON_CLICK, closed);
                break;
             case Button2:
             case Button3:
