@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/ps.c,v 4.20 2002/01/05 02:48:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/ps.c,v 4.22 2003/02/25 01:17:47 william Exp $
  */
 
 #define _INCLUDE_FROM_PS_C_
@@ -638,7 +638,7 @@ static char *psColorImageMacro[] = {
   "         { pop 1 1 iw",
   "            { pop currentfile",
   "              tgifbwpicstr readhexstring pop 0 get tgifnewcolorspot",
-  "              x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
+  "              newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
   "              closepath fill",
   "              /x x 1 add def",
   "            } for",
@@ -668,7 +668,7 @@ static char *psColorImageMacro[] = {
   "       { pop 1 1 iw",
   "          { pop currentfile",
   "            tgifbwpicstr readhexstring pop 0 get tgiftranscolorspot",
-  "            { x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
+  "            { newpath x y moveto 1 0 rlineto 0 1 rlineto -1 0 rlineto",
   "              closepath fill",
   "            }",
   "            if",
