@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.17 2002/01/05 02:48:05 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/raster.c,v 4.21 2003/08/16 19:38:52 william Exp $
  */
 
 #define _INCLUDE_FROM_RASTER_C_
@@ -226,6 +226,7 @@
 #include "xbm/tiffepsi.xbm"
 #include "xbm/png.xbm"
 #include "xbm/jpeg.xbm"
+#include "xbm/pbm.xbm"
 #include "xbm/netlist.xbm"
 
 #include "xbm/file.xbm"
@@ -284,8 +285,6 @@
 #include "xbm/info.xbm"
 #include "xbm/dialog.xbm"
 
-Pixmap patPixmap[MAXPATTERNS+1];
-
 GC rasterGC=NULL;
 GC patGC=NULL;
 
@@ -299,6 +298,7 @@ GC revDefaultGC=NULL;
 GC revGrayGC=NULL;
 
 Pixmap choicePixmap[MAXCHOICES];
+Pixmap patPixmap[MAXPATTERNS+1];
 Pixmap *lineWidthPixmap=NULL;
 Pixmap lineTypePixmap[MAXLINETYPES];
 Pixmap lineDashPixmap[MAXDASHES];
@@ -947,6 +947,8 @@ void InitPattern()
          mainWindow, (char *)png_bits, png_width, png_height);
    whereToPrintPixmap[JPEG_FILE] = XCreateBitmapFromData(mainDisplay,
          mainWindow, (char *)jpeg_bits, jpeg_width, jpeg_height);
+   whereToPrintPixmap[PPM_FILE] = XCreateBitmapFromData(mainDisplay,
+         mainWindow, (char *)pbm_bits, pbm_width, pbm_height);
    whereToPrintPixmap[NETLIST_FILE] = XCreateBitmapFromData(mainDisplay,
          mainWindow, (char *)netlist_bits, netlist_width, netlist_height);
 
