@@ -40,10 +40,19 @@
 #include <pthread.h>
 
 #include "rmcache.h"
+#include "rmwinmask.h"
 
+#ifndef RMINTERNALS_H
+    #include "rminternals.h"
+#endif
 
+extern GLOBAL_OPTIONS rmcast_options;
+
+#ifdef SOLARIS
+pthread_mutex_t   cache_mutex;
+#else
 pthread_mutex_t   cache_mutex = PTHREAD_MUTEX_INITIALIZER;
-
+#endif
 
 /*---------------- Auxiliary functions to manipulate the cache ---------------------------------------*/
 
@@ -65,18 +74,37 @@ int cacheAllocHeaderNode(CACHE **cache, MEMBER_ID *member)
 {
 	if ((*cache = (CACHE*)malloc(sizeof(CACHE)))==NULL)
 	{
-		fprintf(stderr,"Error: Allocating cache header node\n");
+		fprintf(stderr,"cacheAllocHeaderNode ERROR: Allocating cache header node\n");
 		return(0);
 	}
 	bzero((*cache),sizeof(CACHE));
 	(*cache)->active = 1;
-	(*cache)->sm_info.member_id = *member;
-    (*cache)->sm_info.member_status.last_received_or_nak = -1;
+    
+    if (member!=NULL)
+    	(*cache)->sm_info.member_id = *member;
+    (*cache)->sm_info.member_status.last_identified = -1;
     
     (*cache)->sm_info.member_status.first_rcv = -1;
     (*cache)->sm_info.member_status.last_rcv = -1;  
     (*cache)->sm_info.member_status.last_seq_rcv = -1;
     
+#ifdef SINGLE_NACK
+
+    (*cache)->sm_info.member_status.window_size = MAX_WINDOW_SIZE;
+    (*cache)->sm_info.member_status.window_ini  = 0;
+    memset((*cache)->sm_info.member_status.window_mask, 0, MAX_WINDOW_SIZE);
+
+#ifdef DSINGLE_NACK
+    {
+        int h=0;
+        int l=0;
+        window_mask_array2numbers( (*cache)->sm_info.member_status.window_mask,&h,&l,(*cache)->sm_info.member_status.window_ini);
+        fprintf(stderr,"DSINGLE_NACK cacheAllocHeaderNode: window_mask h=%d l=%d\n",h,l);
+    }
+#endif
+    
+#endif    
+    
 	return (1);
 }
 
@@ -97,7 +125,7 @@ int cacheAllocMessageNode(CACHE_NODE **cache_node, DATA_PACKET *node_info)
 {
 	if ((*cache_node = (CACHE_NODE*)malloc(sizeof(CACHE_NODE)))==NULL)
 	{
-		fprintf(stderr,"Error allocating cache message node");
+		fprintf(stderr,"cacheAllocMessageNode ERROR: allocating cache message node");
 		return(0);
 	}
 	
@@ -150,8 +178,15 @@ int cacheFreeMessageNode(CACHE_NODE **node)
 
 void cacheInit(CACHE **cache)
 {
-	*cache = NULL;
-}
+
+#ifdef SOLARIS
+    pthread_mutex_init(&cache_mutex,NULL);
+#endif    
+
+
+	*cache = NULL; 
+} 
+
 
 /***************************************************************************************
  *
@@ -163,6 +198,27 @@ void cacheInit(CACHE **cache)
  *				member, the member who originaly sent the message;
  *				data_packet, message data.
  *
+ * Side Effects:    this routine updates, in the cache, the following fields regarding 
+ *                  the member who originaly sent the message:
+ *                      
+ *                      - number_of_nodes
+ *                      
+ *                      SM_INFO->MEMBER_STATUS
+ *
+ *                          - first_rcv
+ *                          - last_rcv     
+ *                          - last_seq_rcv 
+ *                          - last_identified
+ *                      #ifdef SINGLE_NACK
+ *                          - window_size;
+ *                          - window_mask[MAX_WINDOW_SIZE];
+ *                          - window_ini 
+ *                      #ifdef
+ *
+ *                      - CACHE_NODE *first
+ *                      - CACHE_NODE *last
+ *                      - CACHE_NODE *last_inserted; 
+ *
  * Return value:	0,	on error;
  *					1,	if the inserted message was in sequence,
  *						i.e. the message received was the one expected;
@@ -171,14 +227,27 @@ void cacheInit(CACHE **cache)
  ***************************************************************************************/
 
 
+#define CHECK_INTEGRITY_OF_CACHE                                                                 \
+                                                                                                 \
+		if (newMessageCacheNode -> next == NULL && memberHeaderNode->last != newMessageCacheNode)\
+	    {                                                                                        \
+			fprintf(stderr,"ATENTION!!!! memberHeaderNode->last != newMessageCacheNode           \
+				&& newMessageCacheNode->next == NULL");                                          \
+			exit(1);                                                                             \
+		}                                                                                        
+
+
+
+
 int cacheInsertMessage(CACHE **cache, MEMBER_ID *member, DATA_PACKET *data_packet)
 {
 	int count = 0;
 	int isfirst = 0;
-   	CACHE *memberHeaderNode;
-   	CACHE_NODE *newMessageCacheNode, *auxMessageNode;
+   	CACHE *memberHeaderNode = NULL;
+   	CACHE_NODE *newMessageCacheNode = NULL, *auxMessageNode = NULL;
     
-	
+    /* ====================== Allocating space for the new message ================== */
+
     memberHeaderNode = cacheLookForMember(cache, member);
     
     
@@ -191,55 +260,37 @@ int cacheInsertMessage(CACHE **cache, MEMBER_ID *member, DATA_PACKET *data_packe
 	}
 	
 	
-	if (memberHeaderNode->number_of_nodes == MEMBER_CACHE_SIZE)
+	if (memberHeaderNode->number_of_nodes == rmcast_options.max_member_cache_size)
 	{
-#ifdef	DEBUG2
-		fprintf(stderr,"yes\n");
-#endif	
-		
 		((memberHeaderNode->last)->previous)->next = NULL;
-		
 		auxMessageNode = (memberHeaderNode->last);
-		
 		(memberHeaderNode->last) = ((memberHeaderNode->last)->previous);
-		
 		cacheFreeMessageNode(&auxMessageNode);
-		
 		memberHeaderNode->number_of_nodes--;
 	}
-#ifdef	DEBUG2
-	else fprintf(stderr,"no\n");
-#endif	 
-		 
-	auxMessageNode = memberHeaderNode -> first;
-	
+ 
 	if (cacheAllocMessageNode(&newMessageCacheNode, data_packet)==(int)NULL)
     {
 		return (0);
     }
-	
-	memberHeaderNode -> last_inserted = newMessageCacheNode;
+    
+    /* ================ Inserting the new message =================================== */
 	
 	if (memberHeaderNode->number_of_nodes == 0)
 	{
-		// we will insert the first message node
+		/* we will insert the first message node*/
 		
 		memberHeaderNode -> last = memberHeaderNode -> first = newMessageCacheNode;
-        
-        memberHeaderNode -> sm_info.member_status.last_received_or_nak = -1;
-		
-		memberHeaderNode->sm_info.member_status.first_rcv = data_packet->sn;	  
-		
 		isfirst = 1;
 		
-#ifdef DEBUG2
-		fprintf(stderr,"we will insert the first message node\n");
+#ifdef DEBUG_CACHE
+		fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: we will insert the first message node\n");
 #endif		  
 	}
 	else
 	{
-#ifdef DEBUG2
-		fprintf(stderr,"a message node will be inserted\n");
+#ifdef DEBUG_CACHE
+		fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: a message node will be inserted\n");
 #endif		  
 		auxMessageNode = memberHeaderNode -> first;
 		
@@ -250,22 +301,25 @@ int cacheInsertMessage(CACHE **cache, MEMBER_ID *member, DATA_PACKET *data_packe
 			auxMessageNode = auxMessageNode -> next;
 			count ++;
 		}
+        
 		if (auxMessageNode)
 		{
 			if (auxMessageNode->data_pckt.sn == newMessageCacheNode->data_pckt.sn)
 			{
-				return (0);
+				return (0); /* repeated node */
 			}
 		}
+        
 		if (count == 0)
 		{
 			memberHeaderNode->first = newMessageCacheNode;
 		}
-#ifdef DEBUG2
-		fprintf(stderr,"adjusting pointers\n");
+
+#ifdef DEBUG_CACHE
+		fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: adjusting pointers\n");
 #endif		  
 		
-		if (auxMessageNode != NULL)
+		if (auxMessageNode)
 		{		 
 			newMessageCacheNode->next = auxMessageNode;
 			newMessageCacheNode->previous = auxMessageNode->previous;
@@ -277,86 +331,192 @@ int cacheInsertMessage(CACHE **cache, MEMBER_ID *member, DATA_PACKET *data_packe
 		}
 		else
 		{
-#ifdef DEBUG2
-			fprintf(stderr,"inserting in the end...");
+#ifdef DEBUG_CACHE
+			fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: inserting at the end...");
 			cacheShow(*cache);
 #endif
 			newMessageCacheNode -> previous = memberHeaderNode -> last;
 			memberHeaderNode -> last -> next = newMessageCacheNode;
 			memberHeaderNode -> last = newMessageCacheNode;
 		}
-#ifdef DEBUG			  
-		if (newMessageCacheNode -> next == NULL)		  // is it necessary? no?  
-		{	
-			if (memberHeaderNode->last != newMessageCacheNode)
-			{
-			fprintf(stderr,"ATENTION!!!! memberHeaderNode->last != newMessageCacheNode \
-				&& newMessageCacheNode->next == NULL");
-			exit(1);
-			}				
-		}
-#endif
-		
+#ifdef DEBUG_CACHE                                                                               
+        CHECK_INTEGRITY_OF_CACHE;
+#endif        
 	}
-#ifdef DEBUG0
-	fprintf(stderr,"node (%d) inserted in cache\n", data_packet->sn);    
+    
+#ifdef DEBUG_CACHE
+	fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: node (%d) inserted in cache\n", data_packet->sn);    
 #endif    
+
+    /* ====== Updating fields regarding the member  who originaly sent the msg ====== */
+    
+  	memberHeaderNode->last_inserted = newMessageCacheNode;      
     
 	memberHeaderNode->number_of_nodes++;	  	
 	
-	memberHeaderNode->sm_info.member_status.last_rcv = data_packet->sn;  
-	
+    
+    if (memberHeaderNode->sm_info.member_status.first_rcv == -1)
+        memberHeaderNode->sm_info.member_status.first_rcv = data_packet->sn;  
 
-    if (isfirst == 1)
+    memberHeaderNode->sm_info.member_status.last_rcv = data_packet->sn;          
+    
+    if ( memberHeaderNode->sm_info.member_status.last_identified < data_packet->sn ) 
     {
-        if (data_packet->sn !=0)
-        {
-           memberHeaderNode->sm_info.member_status.last_seq_rcv = -1;
+        memberHeaderNode->sm_info.member_status.last_identified  = data_packet->sn;   
+    }          
+    
+    
+    if ( ( memberHeaderNode->sm_info.member_status.last_seq_rcv + 1 ) == 
            
-           
-           return(2);
-        }
+           newMessageCacheNode->data_pckt.sn )
+    {
+          while ( ( newMessageCacheNode != NULL ) &&
+          
+                  ( memberHeaderNode->sm_info.member_status.last_seq_rcv + 1 == newMessageCacheNode->data_pckt.sn ) )
             
-        else
-        {
-           memberHeaderNode->sm_info.member_status.last_seq_rcv = 0;
-           
-           
-           return (1);
-        }   
+          {
+            memberHeaderNode->sm_info.member_status.last_seq_rcv++;
+
+#ifdef SINGLE_NACK            
+            {
+
+#ifdef DSINGLE_NACK
+                int l=0;
+                int h=0;
+                
+                window_mask_array2numbers( memberHeaderNode->sm_info.member_status.window_mask,
+                                           &h,
+                                           &l,
+                                           memberHeaderNode->sm_info.member_status.window_ini);
+
+               /* fprintf(stderr,"DSINGLE_NACK cacheInsertMessage: window_mask before l=%d h=%d\n",l,h); */
+#endif
+               
+                window_mask_set_bit2(memberHeaderNode->sm_info.member_status.window_mask,memberHeaderNode->sm_info.member_status.window_ini,0);
+                memberHeaderNode->sm_info.member_status.window_ini = (memberHeaderNode->sm_info.member_status.window_ini + 1) % MAX_WINDOW_SIZE;
+ 
+#ifdef DSINGLE_NACK
+                    window_mask_array2numbers( memberHeaderNode->sm_info.member_status.window_mask,
+                           &h,
+                           &l,
+                           memberHeaderNode->sm_info.member_status.window_ini);
+                    /* fprintf(stderr,"DSINGLE_NACK cacheInsertMessage: window_mask after l=%d h=%d\n",l,h); */
+#endif
+            }
+#endif            
+            
+            newMessageCacheNode = newMessageCacheNode -> previous;
+          }
+          
+#ifdef DEBUG_CACHE
+      	  fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: node (%d) inserted in cache, in sequence\n", data_packet->sn);    
+#endif    
+          
+          return (1);
+    }
+    else
+    {
+
+#ifdef SINGLE_NACK                
+
+        if ( ( memberHeaderNode->sm_info.member_status.last_seq_rcv <= data_packet->sn ) && ( data_packet->sn <= memberHeaderNode->sm_info.member_status.last_seq_rcv + MAX_WINDOW_SIZE ) )
+        
+            memberHeaderNode->sm_info.member_status.window_mask 
+            
+            [  ( memberHeaderNode->sm_info.member_status.window_ini + data_packet->sn - (memberHeaderNode->sm_info.member_status.last_seq_rcv + 1 ) ) % MAX_WINDOW_SIZE ] 
+            
+            = 0;
+#endif        
+        
+#ifdef DEBUG_CACHE
+   	    fprintf(stderr,"DEBUG_CACHE cacheInsertMessage: node (%d) inserted in cache, not in sequence\n", data_packet->sn);    
+#endif    
+    
+    	return (2);  /* the packet is not in sequence - it will be needed to send a nak */
         
     }
-	else if (data_packet->sn==memberHeaderNode->sm_info.member_status.last_seq_rcv+1 /* || isfirst == 1 */)
-	{
-	/* the message is in sequence - setting up the last sequence received field 
-		in the member header node */
-		
-		/* check this ! */
-		
-		auxMessageNode = newMessageCacheNode;
-		
-		while (auxMessageNode->previous!=NULL)
-		{
-			if (!((auxMessageNode->previous)->data_pckt.sn == auxMessageNode->data_pckt.sn + 1 ))
-				break;
-			auxMessageNode = auxMessageNode->previous;
-		}
-		
-		memberHeaderNode->sm_info.member_status.last_seq_rcv = auxMessageNode->data_pckt.sn;  
-#ifdef DEBUG2		
-		cacheShow(*cache);	  
-#endif        
+}
 
-		return (1);  /* the packet is in sequence */
-	}
-	else 
-	{
-#ifdef DEBUG2		    
-		cacheShow(*cache);	  
-#endif        		
+
+/***************************************************************************************
+ *
+ * void cachePack(CACHE **cache, char **buf, int *buffsize)
+ *
+ * Packs the cache information in a buffer.
+ *
+ * Arguments:	cache, the cache;
+ *				buf, points to the place where the packed info will be stored;
+ *				buffsize, returns the number of bytes in buf.
+ *
+ * Side effects: this routine allocates the buf variable, and stores its 
+ *               final size in buffsize.
+ *
+ ***************************************************************************************/
+
+void cachePack(CACHE **cache, char **buf, int *buffsize)
+{
+    CACHE *current_header_node;    
+    char *buf_next_write_pos;
+    int nelem = 0;
+
+    current_header_node = *cache;   *buf = NULL;
+    
+    while ( current_header_node != NULL )
+    {
+        nelem ++;
         
-		return (2); /* the packet is not in sequence - it will be needed to send a nak */
-	}
+        /* allocating space to store info about the new cache header node */
+        *buf = realloc( *buf, ( nelem * sizeof(CACHE)) );
+        
+        /* writing the information about the new cache header node */
+        buf_next_write_pos = *buf + ( (nelem - 1) * sizeof(CACHE) );        
+        memcpy(buf_next_write_pos, current_header_node, sizeof(CACHE) );
+        
+        current_header_node = current_header_node->next;
+        
+    }  
+    
+    *buffsize = nelem * sizeof(CACHE);
+}
+
+
+/***************************************************************************************
+ *
+ * void cacheUnpack(CACHE **cache, char *buf, int buffsize)
+ *
+ * Unpacks the cache information contained in a buffer.
+ *
+ * Arguments:	cache, the cache;
+ *				buf, points to the place from where the packed info will be read;
+ *				buffsize, the number of bytes in buf.
+ *
+ * Side effects: updates the cache.
+ *
+ ***************************************************************************************/
+
+void cacheUnpack(CACHE **cache, char *buf, int buffsize)
+{
+    char *buf_next_read_pos;
+    int nbread = 0; /* number of bytes read */
+    CACHE *cur_member;
+    CACHE cur_member_buf;
+    
+    buf_next_read_pos = buf;
+    
+    for (nbread = 0; nbread < buffsize; nbread += sizeof(CACHE))
+    {
+        memcpy(&cur_member_buf, buf_next_read_pos, sizeof(CACHE));
+        
+        cur_member = cacheInsertMember(cache, &(cur_member_buf.sm_info.member_id));
+        
+        cur_member->sm_info.member_status = cur_member_buf.sm_info.member_status;
+        
+        cur_member->sm_info.member_status.last_rcv        = cur_member->sm_info.member_status.last_seq_rcv;
+        cur_member->sm_info.member_status.last_identified = cur_member->sm_info.member_status.last_seq_rcv;        
+        
+        buf_next_read_pos += sizeof(CACHE);        
+        
+    }
 }
 
 /***************************************************************************************
@@ -383,14 +543,14 @@ CACHE *cacheInsertMember(CACHE **cache, MEMBER_ID *member)
 	
 	if (cacheIsEmpty(cache))
 	{
-		// cache is empty
+		/* cache is empty */
 		
 		*cache = newHeaderNode;
 	}
-	else  // inserts new node in the end of the list
+	else  /* inserts new node in the end of the list */
 	{
-#ifdef DEBUG2
-		fprintf(stderr,"searching where to insert new member node...\n");
+#ifdef DEBUG_CACHE
+		fprintf(stderr,"DEBUG_CACHE cacheInsertMember: searching where to insert new member node...\n");
 #endif		  
 		
 		while (auxHeaderNode->next!=NULL)
@@ -398,8 +558,8 @@ CACHE *cacheInsertMember(CACHE **cache, MEMBER_ID *member)
 			auxHeaderNode = auxHeaderNode->next;
 		}
 		
-#ifdef DEBUG2
-		fprintf(stderr,"member node will be inserted...\n");
+#ifdef DEBUG_CACHE
+		fprintf(stderr,"DEBUG_CACHE cacheInsertMember: member node will be inserted...\n");
 #endif		  
 		auxHeaderNode->next = newHeaderNode;
 	}
@@ -443,14 +603,14 @@ int cacheIsEmpty(CACHE **cache)
 int cacheOfAnyMemberIsFull(CACHE *cache)
 {
 	CACHE *auxCacheHeaderNode = cache;
-	while (auxCacheHeaderNode->number_of_nodes < MEMBER_CACHE_SIZE)
+	while (auxCacheHeaderNode->number_of_nodes < rmcast_options.max_member_cache_size)
 	{
 		auxCacheHeaderNode = auxCacheHeaderNode->next;
 		
 		if (auxCacheHeaderNode == NULL)
 			return (0);
 	}
-	return (!(auxCacheHeaderNode->number_of_nodes < MEMBER_CACHE_SIZE));
+	return (!(auxCacheHeaderNode->number_of_nodes < rmcast_options.max_member_cache_size));
 }
 
 int memberIsEqual(MEMBER_ID *member1, MEMBER_ID *member2)
@@ -481,7 +641,7 @@ CACHE* cacheLookForMember(CACHE **cache1, MEMBER_ID *member_id)
 	CACHE *auxHeaderNode=NULL;
     CACHE *cache=NULL;
     
-    if (!cacheIsEmpty(cache1))
+    if (!cacheIsEmpty(cache1) && (member_id!=NULL))
     {        
           
         cache = *cache1;
@@ -496,10 +656,10 @@ CACHE* cacheLookForMember(CACHE **cache1, MEMBER_ID *member_id)
 			}
 		}
 	}
-#ifdef DEBUG2    
+#ifdef DEBUG_CACHE   
     else
     {
-        fprintf(stderr,"Cache is empty. (cacheLookForMember2)\n");        
+        fprintf(stderr,"DEBUG_CACHE cacheLookForMember: Cache is empty.\n");        
     }
 #endif    
     
@@ -545,14 +705,19 @@ int cacheDesactivateMember(CACHE *cache, MEMBER_ID *member)
 
 void cacheShow(CACHE *cache)
 {
-#ifdef CACHE_SHOW
+#ifdef DEBUG_SHOW
 
 	CACHE* auxCacheHeaderNode = cache;
 	CACHE_NODE *auxMessageNode;
 	char aux_string[MAX_IP_STRING_SIZE];
 	int i;
+#ifdef DSINGLE_NACK
+    int h=0;
+    int l=0;
+    char aux_str[200];
+#endif
 	
-	fprintf(stderr,"\n-----  cache --------------------------------------------------\n");
+		fprintf(stderr,"\n-----cacheShow-------------------------------------------------------\n");    
 	
 	while (auxCacheHeaderNode!=NULL)
 	{
@@ -560,17 +725,33 @@ void cacheShow(CACHE *cache)
 			aux_string[i]=auxCacheHeaderNode->sm_info.member_id.ip[i];
 		
 		auxMessageNode = auxCacheHeaderNode->first; 	
-		fprintf(stderr,"[%x %x %sactive {first:%d last:%d last_seq:%d}] (%d nodes) %s %d:\n",
+
+
+        fprintf(stderr,"[%x %x %sactive {first:%d last:%d last_seq:%d last identified:%d}] (%d nodes) %s %d :\n",
 			(unsigned int)auxCacheHeaderNode->first,
 			(unsigned int)auxCacheHeaderNode->last,
 			(auxCacheHeaderNode->active==1?"   ":"des"),
 			auxCacheHeaderNode->sm_info.member_status.first_rcv,
 			auxCacheHeaderNode->sm_info.member_status.last_rcv,
 			auxCacheHeaderNode->sm_info.member_status.last_seq_rcv,
+            auxCacheHeaderNode->sm_info.member_status.last_identified,
 			auxCacheHeaderNode->number_of_nodes,
 			aux_string, 
 			auxCacheHeaderNode->sm_info.member_id.pid
 			);
+#ifdef DSINGLE_NACK
+        window_mask_array2numbers(auxCacheHeaderNode->sm_info.member_status.window_mask,&h,&l,auxCacheHeaderNode->sm_info.member_status.window_ini);
+        
+        sprintf(aux_str,"DSINGLE_NACK (DEBUG_SHOW) cacheShow: window_ini=%d h=%d l=%d\n",auxCacheHeaderNode->sm_info.member_status.window_ini,h,l);
+        for(i=0;i<MAX_WINDOW_SIZE;i++)
+        {
+            sprintf(aux_str,"%s%s%d",aux_str,(i==auxCacheHeaderNode->sm_info.member_status.window_ini)?"*":"",auxCacheHeaderNode->sm_info.member_status.window_mask[i]);
+            
+        }
+        fprintf(stderr,"%s\n",aux_str);
+#endif
+
+#ifdef DCACHE_CONTENTS
 		while (auxMessageNode!=NULL)
 		{
 			fprintf(stderr," \t([%08x %08x %08x] sn: %d pck size: %d %s)\n",
@@ -583,10 +764,11 @@ void cacheShow(CACHE *cache)
 				); 
 			auxMessageNode = auxMessageNode->next;
 		} 
-		
+#endif		
 		auxCacheHeaderNode = auxCacheHeaderNode->next;
 	}
-	fprintf(stderr,"\n-------------------------------------------------------------\n"); 
+
+	fprintf(stderr,"\n--------------------------------------------------------------------\n\n"); 
 
 #endif
 }
@@ -620,7 +802,7 @@ void cacheGetSMInfo(CACHE *cache, SM_INFO **sm_table, int *nusers)
 		
 		if ((*sm_table= (SM_INFO*)realloc(*sm_table,(i+1)*sizeof(SM_INFO)))==NULL)
 		{
-			fprintf(stderr,"Error allocating sm_info");
+			fprintf(stderr,"cacheGetSMInfo ERROR: allocating sm_info");
 			exit(1);
 		}
 		
@@ -653,6 +835,10 @@ DATA_PACKET *cacheLookForMessage(CACHE **cache1, MEMBER_ID *member_id,int messag
 
     cache = *cache1;
 	
+#ifdef DSINGLE_NACK
+      fprintf(stderr,"DSINGLE_NACK cacheLookForMessage: entering, looking for %s:%d:%d\n",member_id->ip,member_id->pid,message_sn);
+#endif
+
 	if ((auxMember=cacheLookForMember(&cache,member_id))!=NULL)
 	{
 		auxMsg=auxMember->first;
@@ -700,7 +886,7 @@ int cacheAllocNAKNode(NAK_LIST **nak_node, int sn)
 	bzero((*nak_node),sizeof(NAK_LIST));
 	(*nak_node)->sn = sn;
     (*nak_node)->hasReceived = 0;
-	(*nak_node)->counter = 1; //at least one nak was send
+	(*nak_node)->counter = 1; /* at least one nak was send */
 	
 	return (1);
 }
@@ -729,8 +915,6 @@ int cacheUpdateNakList(CACHE **cache1, MEMBER_ID *member, int sn)
     
     cache = *cache1;
 	
-	//FIXME    fprintf(stderr,"antes do cacheLookForMember:\nip: %s\npid: %d\n",member->ip,member->pid);	
-	
 	if((memberHeaderNode = cacheLookForMember(&cache, member))==NULL)
 	{
 		fprintf(stderr,"Error:cacheLookForMember failed at cacheUpdateNakList\n");
@@ -760,10 +944,8 @@ int cacheUpdateNakList(CACHE **cache1, MEMBER_ID *member, int sn)
 			}  
 			else 
 			{  
-				if( auxNakNode->counter >= MAX_NAK )
+				if( auxNakNode->counter >= rmcast_options.max_nak )
                 {
-					fprintf(stderr,"Error:Number of sent NAKs reached MAX_NAK at cacheUpdateNakList\n");
-
 					return(0);
 				}
 #ifdef DONT_SEND_NAK_AGAIN_IMMEDIATLY
@@ -835,3 +1017,4 @@ void cacheUpdateNakListNakWasReceived(CACHE **cache1, MEMBER_ID *member, int sn)
 #endif
 
 
+
