@@ -29,6 +29,8 @@
 
 #ifndef RMMSGPCKT_C
 
+#define RMMSGPCKT_C
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -38,8 +40,6 @@
 #include "rmstruct.h"
 #include "rmmsgpckt.h"
 
-#define RMMSGPCKT_C
-
 #define MIN(a,b)  ((a<b)?a:b) 
 
 #define get(dest, org, bytes) { bzero(dest,bytes); memcpy(dest,org, bytes); org+=bytes; }
@@ -108,12 +108,15 @@ void msgPcktShowMessage(BYTE *message)
 	MEMBER_ID member_id;  char string[1001];
 	
 	int sn, pckt_size, data_size, k, port;
-   
+
+#ifdef DEBUG_MSG_CONTENTS   
     int i;
+#endif
 	
 	get(&type,message,sizeof(char));
 
-	fprintf(stderr,"\ntype: %d\n",type);
+	fprintf(stderr,"DEBUG_SHOW msgPcktShowMessage BEGIN -------------------\n");
+    fprintf(stderr,"\ntype: %d\n",type);
 	fprintf(stderr,"member info: \n");
 
 	get_member_id(member_id, message);
@@ -163,21 +166,12 @@ void msgPcktShowMessage(BYTE *message)
 
 		fprintf(stderr,"\tdata: ");
 
-
+#ifdef DEBUG_MSG_CONTENTS
 		for(i=0; i<data_size; i++)
 		{
-			if (isalnum((int)string[i]))
-			{
-				fprintf(stderr,"%c ",string[i]);
-			}
-			else
-			{
-				fprintf(stderr,"\\%d ",string[i]);
-			}
+     		fprintf(stderr,"%c",string[i]);
 		}
-        
-		fprintf(stderr,"\n\n");
-        
+#endif        
 		break;
 		
 	case RETRANSM_PACKET_TYPE: 
@@ -208,17 +202,12 @@ void msgPcktShowMessage(BYTE *message)
  		fprintf(stderr,"\tdata: %s\n",string);
 
 			
+#ifdef DEBUG_MSG_CONTENTS
 		for(i=0; i<data_size; i++)
 		{
-			if (isalnum((int)string[i]))
-			{
-				fprintf(stderr,"%c ",string[i]);
-			}
-			else
-			{
-				fprintf(stderr,"\\%d ",string[i]);
-			}
+     		fprintf(stderr,"%c",string[i]);
 		}
+#endif
        
 		fprintf(stderr,"\n\n");
 
@@ -237,20 +226,39 @@ void msgPcktShowMessage(BYTE *message)
         
 		
 		get(&sn,message,sizeof(int)); 
-        
+                
         fprintf(stderr,"nak packet: \n");
+        
+        fprintf(stderr,"\tsn: %d\n",sn);
 
 #ifdef SOLARIS
         EXCHANGE_BYTE_ORDER(sn);
 #endif  
+
+#ifdef  SINGLE_NACK       
     
+        {
+            int base_sn, window_size, hmask, lmask;
+            
+    		get(&(base_sn), message, sizeof(int));	
+	    	get(&(window_size),message, sizeof(int));	        
+		    get(&(hmask), message, sizeof(int));	        
+    		get(&(lmask), message, sizeof(int));	        
+
+#ifdef SOLARIS
+            EXCHANGE_BYTE_ORDER(base_sn);
+            EXCHANGE_BYTE_ORDER(window_size);
+            EXCHANGE_BYTE_ORDER(hmask);
+            EXCHANGE_BYTE_ORDER(lmask);
+#endif  
+            
+            fprintf(stderr,"\tbase sn: %d window size: %d \n\thigher part of the mask: %d lower part of the mask: %d\n", 
+                base_sn, window_size, hmask, lmask);
+        }
         
-        fprintf(stderr,"\tsn: %d\n",sn);
-/*        
-        get(&sn,message,sizeof(int)); 
-        
-        fprintf(stderr,"\tlast sn: %d\n",sn);
-*/		
+#endif        // single nack
+
+
     break;
 		
 		
@@ -297,6 +305,7 @@ void msgPcktShowMessage(BYTE *message)
         
 	}
 
+	fprintf(stderr,"DEBUG_SHOW msgPcktShowMessage END -------------------\n");
 #endif
 
 	return;
@@ -341,6 +350,16 @@ void msgPcktExchangeByteOrder(PACKET_INFO *pckt_info)
     
         EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.requested_member_id.pid);
         EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.sn);
+        
+#ifdef  SINGLE_NACK                
+
+        EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.base_sn);
+        EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.window_size);
+        EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.hmask);
+        EXCHANGE_BYTE_ORDER(pckt_info->packet_data.nak_packet.lmask);
+        
+#endif
+        
         break;                
     
     case REFRESH_PACKET_TYPE:    
@@ -378,7 +397,7 @@ void msgPcktExchangeByteOrder(PACKET_INFO *pckt_info)
 void msgPcktUnmountMessage(PACKET_INFO *pckt_info, BYTE *message)
 {
 #ifdef DEBUG_PCKT
-    fprintf(stderr,"msgPcktUnmountMessage() [char: %d MEMBER_ID: %d int: %d]\n",
+    fprintf(stderr,"DEBUG_PCKT msgPcktUnmountMessage: [char: %d MEMBER_ID: %d int: %d]\n",
         sizeof(char),
         sizeof(MEMBER_ID),
         sizeof(int));
@@ -411,7 +430,16 @@ void msgPcktUnmountMessage(PACKET_INFO *pckt_info, BYTE *message)
 		
 		get(&(pckt_info->packet_data.nak_packet.requested_member_id),message,sizeof(MEMBER_ID));	
         get(&(pckt_info->packet_data.nak_packet.sn),message,sizeof(int));	
-/*FIXME get(&(pckt_info->packet_data.nak_packet.last_sn),message,sizeof(int));	*/
+        
+#ifdef  SINGLE_NACK        
+
+		get(&(pckt_info->packet_data.nak_packet.base_sn), message, sizeof(int));	
+		get(&(pckt_info->packet_data.nak_packet.window_size),message, sizeof(int));	        
+		get(&(pckt_info->packet_data.nak_packet.hmask), message, sizeof(int));	        
+		get(&(pckt_info->packet_data.nak_packet.lmask), message, sizeof(int));	        
+        
+#endif        
+        
 		break;
 		
 #ifdef REFRESH
@@ -447,17 +475,17 @@ void msgPcktUnmountMessage(PACKET_INFO *pckt_info, BYTE *message)
 	}
 #ifdef SOLARIS    
 
-    fprintf(stderr,"That's SOLARIS so, excanching by order:\n[sender pid] ");
+    fprintf(stderr,"SOLARIS msgPcktUnmountMessage: That's SOLARIS so, excanching by order:\n[sender pid] ");
     fprintf(stderr,"%d %x\n",pckt_info->sender_id.pid,pckt_info->sender_id.pid);
 
     msgPcktExchangeByteOrder(pckt_info);
 
-    fprintf(stderr,"[sender pid]: %d %x\n",pckt_info->sender_id.pid,pckt_info->sender_id.pid);
+    fprintf(stderr,"SOLARIS msgPcktUnmountMessage: [sender pid]: %d %x\n",pckt_info->sender_id.pid,pckt_info->sender_id.pid);
     
 #endif    
 
 #ifdef DEBUG_PCKT
-    fprintf(stderr,"returning from msgPcktUnmountMessage()\n");
+    fprintf(stderr,"DEBUG_PCKT msgPcktUnmountMessage: returning\n");
 #endif    
 
 	return; 
@@ -488,7 +516,7 @@ void msgPcktMountMessage(PACKET_INFO *pckt_info, BYTE **message, int *msgsize)
 	int packet_size = 0;
 
 #ifdef DEBUG_PCKT
-	fprintf(stderr,"mounting packet\n");
+	fprintf(stderr,"DEBUG_PCKT msgPcktMountMessage: mounting packet\n");
 #endif
 
 #ifdef SOLARIS
@@ -500,7 +528,7 @@ void msgPcktMountMessage(PACKET_INFO *pckt_info, BYTE **message, int *msgsize)
 #endif
 
 #ifdef DEBUG_PCKT
-    fprintf(stderr,"[sender pid] ");
+    fprintf(stderr,"DEBUG_PCKT msgPcktMountMessage: [sender pid] ");
     fprintf(stderr,"%d %x\n",pckt_info->sender_id.pid,pckt_info->sender_id.pid);
 #endif    
 
@@ -512,7 +540,7 @@ void msgPcktMountMessage(PACKET_INFO *pckt_info, BYTE **message, int *msgsize)
 	aux_message_pt = common_header;
 
 #ifdef DEBUG_PCKT
-	fprintf(stderr,"common_header allocated at: %p\n", common_header);
+	fprintf(stderr,"DEBUG_PCKT msgPcktMountMessage:common_header allocated at: %p\n", common_header);
 #endif	  
 	
 	getPCKInfo(aux_message_pt, &(pckt_info->type), sizeof(char));
@@ -571,8 +599,16 @@ void msgPcktMountMessage(PACKET_INFO *pckt_info, BYTE **message, int *msgsize)
 		getPCKInfo(aux_message_pt,common_header,common_header_size);
 		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.requested_member_id), sizeof(MEMBER_ID));        
 		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.sn), sizeof(int));
-/*		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.last_sn), sizeof(int));	
-FIXME relativo a possibilidade de envio de nak pra um grupo de pacotes*/	
+        
+#ifdef  SINGLE_NACK        
+
+		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.base_sn), sizeof(int));	
+		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.window_size), sizeof(int));	        
+		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.hmask), sizeof(int));	        
+		getPCKInfo(aux_message_pt, &(pckt_info->packet_data.nak_packet.lmask), sizeof(int));	        
+        
+#endif        
+
 		break;
 		
 
@@ -616,7 +652,7 @@ FIXME relativo a possibilidade de envio de nak pra um grupo de pacotes*/
         
     default:
     
-        fprintf(stderr,"Unknown packet type: %d\n",pckt_info->type);
+        fprintf(stderr,"DEBUG_PCKT msgPcktMountMessage: Unknown packet type: %d\n",pckt_info->type);
 	}
 	
 	packet_size++;
@@ -630,12 +666,13 @@ FIXME relativo a possibilidade de envio de nak pra um grupo de pacotes*/
 #ifdef DEBUG_PCKT
     {
         int cont;
-        fprintf(stderr,"Mounted message: \n");
+        fprintf(stderr,"DEBUG_PCKT msgPcktMountMessage: Mounted message: \n");
         for (cont = 0; cont < packet_size; cont++)
         {
-            fprintf(stderr,"%d [%c] ", ((signed char*)(*message))[cont], (*message)[cont]);
+            fprintf(stderr,"%d", ((signed char*)(*message))[cont]);
         }	
     }
+    fprintf(stderr,"\n");
 #endif 
     
 
