@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.72 2002/03/21 07:44:55 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/setup.c,v 4.88 2003/08/16 19:38:53 william Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -39,7 +39,7 @@
 
 #include "attr.e"
 #include "auxtext.e"
-/* #include "chat.e" */
+#include "chat.e"
 #include "choice.e"
 #include "choose.e"
 #include "cmd.e"
@@ -88,7 +88,9 @@
 #include "stretch.e"
 #include "strtbl.e"
 #include "tangram2.e"
+#include "tdgtlist.e"
 #include "text.e"
+#include "tidget.e"
 #include "util.e"
 #include "version.e"
 #include "wb.e"
@@ -125,7 +127,8 @@
 #define STATUS_WINDOW_H 20
 #define MODE_WINDOW_W (CHOICE_IMAGE_W)
 #define MODE_WINDOW_H (DRAW_WINDOW_H+RULER_W+(BRDR_W<<1))
-#define CHAT_WINDOW_H (CHOICE_WINDOW_H)
+#define CHAT_WINDOW_W (TITLE_WINDOW_W)
+#define CHAT_WINDOW_H 40
 
 int	debugNoPointerGrab=FALSE;
 
@@ -163,7 +166,7 @@ int	statusSubWindowW[MAX_STATUS_BTNS];
 int	statusSubWindowH[MAX_STATUS_BTNS];
 int	modeWindowW = MODE_WINDOW_W;
 int	modeWindowH = MODE_WINDOW_H;
-int	chatWindowW = STATUS_WINDOW_W;
+int	chatWindowW = CHAT_WINDOW_W;
 int	chatWindowH = CHAT_WINDOW_H;
 
 int	initialMenubarWindowH=MENUBAR_WINDOW_H;
@@ -383,6 +386,7 @@ void InitWinSizes()
          drawWinW+rulerW+brdrW+scrollBarW+(brdrW<<1);
    menubarWindowW = titleWindowW;
    statusWindowW = titleWindowW-userRedrawWindowW-(brdrW<<1);
+   chatWindowW = titleWindowW;
    CalcMenubarWindowHeight();
    msgWindowW = titleWindowW-choiceWindowW-(brdrW<<1);
    mainWinW = titleWindowW+(brdrW<<1);
@@ -393,6 +397,7 @@ void InitWinSizes()
          titleWindowH+vSBarH+scrollBarW+6*brdrW;
    if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcStatusSubWinGeom();
+   SetChatWindowGeom();
 }
 
 void InverseInitWinSizes()
@@ -400,6 +405,7 @@ void InverseInitWinSizes()
 {
    titleWindowW = mainWinW-(brdrW<<1);
    menubarWindowW = titleWindowW;
+   chatWindowW = titleWindowW;
    statusWindowW = titleWindowW-userRedrawWindowW-(brdrW<<1);
    CalcMenubarWindowHeight();
    msgWindowW = titleWindowW-choiceWindowW-(brdrW<<1);
@@ -417,6 +423,7 @@ void InverseInitWinSizes()
    if (colorLayers) vSBarH = DEF_VSBAR_H;
    CalcMenubarWindowHeight();
    CalcStatusSubWinGeom();
+   SetChatWindowGeom();
 }
 
 void ComputeMainWinXY(MainWinX, MainWinY)
@@ -505,10 +512,6 @@ void DoReconfigure()
    XMoveWindow(mainDisplay, dummyWindow2,
          x_offset+drawWinW+rulerW+(brdrW<<2), cur_y);
    cur_y += scrollBarW + (brdrW<<1);
-   if (!noChatWindow) {
-      XMoveWindow(mainDisplay, chatWindow, 0, cur_y);
-      cur_y += chatWindowH + (brdrW<<1);
-   }
    if (!noStatusWindow) {
       int i;
 
@@ -521,6 +524,11 @@ void DoReconfigure()
                statusSubWindowX[i], statusSubWindowY[i],
                statusSubWindowW[i], statusSubWindowH[i]);
       }
+      cur_y += statusWindowH + (brdrW<<1);
+   }
+   if (!noChatWindow) {
+      MoveResizeChatWindow(0, cur_y, chatWindowW, chatWindowH);
+      cur_y += chatWindowH + (brdrW<<1);
    }
    if (inSlideShow) {
       int dpy_w=DisplayWidth(mainDisplay,mainScreen);
@@ -617,6 +625,8 @@ void Reconfigure(Forced)
             (brdrW*6);
    }
    menubarWindowW = titleWindowW;
+   chatWindowW = titleWindowW;
+   if (!noChatWindow) SetChatWindowGeom();
    statusWindowW = titleWindowW-userRedrawWindowW-(brdrW<<1);
    CalcMenubarWindowHeight();
    msgWindowW = titleWindowW - choiceWindowW - (brdrW<<1);
@@ -671,7 +681,7 @@ int mainWinEventHandler(input)
       TranslateKeys(s, &key_sym);
 
       if (CharIsESC(key_ev, s, key_sym, &has_ch)) {
-         ToggleSlideShow();
+         LeaveSlideShow();
          return INVALID;
       }
       switch (key_sym) {
@@ -865,7 +875,34 @@ void InitTmpDir()
 {
    int tmp_dir_in_x_def=FALSE;
    char *c_ptr=NULL;
+   struct stat stat_buf;
 
+   if ((c_ptr=getenv("HOME")) == NULL) {
+      strcpy(homeDir, DIR_SEP_STR);
+   } else if (strlen(c_ptr) >= MAXPATHLENGTH-1) {
+      strcpy(homeDir, DIR_SEP_STR);
+   } else {
+      strcpy(homeDir, c_ptr);
+   }
+   sprintf(tgifDir, "%s%c.%s", homeDir, DIR_SEP, TOOL_NAME);
+   memset(&stat_buf, 0, sizeof(struct stat));
+   if (stat(tgifDir, &stat_buf) != 0) {
+      /* directory does not exist */
+      if (mkdir(tgifDir, 0755) != 0) {
+         fprintf(stderr, TgLoadString(STID_CANT_CREATE_NAMED_DIRECTORY),
+               tgifDir);
+         fprintf(stderr, "\n");
+         if (!PRTGIF) {
+            fprintf(stderr, "\t");
+            fprintf(stderr, TgLoadString(STID_OPR_USING_NAMED_DIR_WILL_FAIL),
+                  tgifDir);
+            fprintf(stderr, "\n");
+         }
+      }
+   }
+   if (*bootDir == '\0') {
+      GetWorkingDirectory(bootDir, sizeof(bootDir));
+   }
    if (!PRTGIF || cmdLineOpenDisplay) {
       /* do not translate -- program constants */
       if (mainDisplay != NULL &&
@@ -920,7 +957,6 @@ void Setup()
    int bg_pixel=(-1);
    char *c_ptr=NULL;
    int def_x_neg=0, def_y_neg=0, paper_size_set;
-   struct stat stat_buf;
    XWMHints wmhints;
    XSizeHints sizehints;
    XClassHint classhints;
@@ -993,6 +1029,9 @@ void Setup()
          whereToPrint = PNG_FILE;
       } else if (UtilStrICmp(c_ptr, "JPEG") == 0) {
          whereToPrint = JPEG_FILE;
+      } else if (UtilStrICmp(c_ptr, "PPM") == 0 ||
+            UtilStrICmp(c_ptr, "PBM") == 0) {
+         whereToPrint = PPM_FILE;
       } else if (UtilStrICmp(c_ptr, "NetList") == 0) {
          whereToPrint = NETLIST_FILE;
       } else {
@@ -1239,13 +1278,28 @@ void Setup()
          noChoiceWindow = TRUE;
       }
    }
-   noModeWindow = FALSE;
-   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoModeWindow")) != NULL) {
-      if (UtilStrICmp("true", c_ptr) == 0) {
-         noModeWindow = TRUE;
+   if (cmdLineNoModeWindow) {
+      noModeWindow = TRUE;
+   } else {
+      noModeWindow = FALSE;
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoModeWindow")) != NULL) {
+         if (UtilStrICmp("true", c_ptr) == 0) {
+            noModeWindow = TRUE;
+         }
       }
    }
-   noChatWindow = TRUE;
+#ifdef _TGIF_WB2
+   if (cmdLineNoChatWindow) {
+      noChatWindow = TRUE;
+   } else {
+      noChatWindow = FALSE;
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"NoChatWindow")) != NULL) {
+         if (UtilStrICmp("true", c_ptr) == 0) {
+            noChatWindow = TRUE;
+         }
+      }
+   }
+#endif /* _TGIF_WB2 */
 
    scrollBarW = baseScrollBarW = SCROLLBAR_W;
    if ((c_ptr=XGetDefault(mainDisplay, TOOL_NAME, "ScrollBarWidth")) != NULL) {
@@ -1267,6 +1321,11 @@ void Setup()
       pageStyle = LANDSCAPE;
       paper_size_set = TRUE;
    }
+   /* do not translate -- program constants */
+   protocolAtom = XInternAtom(mainDisplay, "WM_DELETE_WINDOW", False);
+   executeCmdByIDAtom = XInternAtom(mainDisplay, "TGIF_EXECUTE_CMD_BY_ID",
+         False);
+
    InitStrTable();
    InitPaperSize();
 
@@ -1503,32 +1562,6 @@ void Setup()
    i = strlen(drawPath);
    if (drawPath[i-1] == DIR_SEP) drawPath[i-1] = '\0';
 
-   if ((c_ptr=getenv("HOME")) == NULL) {
-      strcpy(homeDir, DIR_SEP_STR);
-   } else if (strlen(c_ptr) >= MAXPATHLENGTH-1) {
-      strcpy(homeDir, DIR_SEP_STR);
-   } else {
-      strcpy(homeDir, c_ptr);
-   }
-   sprintf(tgifDir, "%s%c.%s", homeDir, DIR_SEP, TOOL_NAME);
-   memset(&stat_buf, 0, sizeof(struct stat));
-   if (stat(tgifDir, &stat_buf) != 0) {
-      /* directory does not exist */
-      if (mkdir(tgifDir, 0755) != 0) {
-         fprintf(stderr, TgLoadString(STID_CANT_CREATE_NAMED_DIRECTORY),
-               tgifDir);
-         fprintf(stderr, "\n");
-         if (!PRTGIF) {
-            fprintf(stderr, "\t");
-            fprintf(stderr, TgLoadString(STID_OPR_USING_NAMED_DIR_WILL_FAIL),
-                  tgifDir);
-            fprintf(stderr, "\n");
-         }
-      }
-   }
-   if (*bootDir == '\0') {
-      GetWorkingDirectory(bootDir, sizeof(bootDir));
-   }
    InitTmpDir();
 
    enableMouseWheel = TRUE;
@@ -1554,11 +1587,12 @@ void Setup()
       statusWindowH += windowPadding;
    }
    userRedrawWindowH = statusWindowH;
-/* chatWindowH = GetChatWindowHeight();
+
+   InitChat();
+   chatWindowH = GetInitialChatWindowHeight();
    if (threeDLook) {
       chatWindowH += windowPadding;
    }
- */
    noMinWinSize = TRUE;
    if (threeDLook && (c_ptr=XGetDefault(mainDisplay, TOOL_NAME,
          "NoMinWinSize")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
@@ -1690,7 +1724,6 @@ void Setup()
    if (newColormapUsed) {
       XSetWindowColormap(mainDisplay, mainWindow, mainColormap);
    }
-   InitWhiteBoard();
    InitMenu();
    InitPattern();
 
@@ -1811,6 +1844,13 @@ void Setup()
          brdrW, myBorderPixel, bg_pixel)) == 0) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
    }
+   cur_y += statusWindowH + (brdrW<<1);
+
+   if ((chatWindow=XCreateSimpleWindow(mainDisplay, mainWindow,
+         0, cur_y, chatWindowW, chatWindowH, brdrW, myBorderPixel,
+         bg_pixel)) == 0) {
+      FailToCreateWindowMessage("Setup()", NULL, TRUE);
+   }
    InitScroll();
    CalcStatusSubWinGeom();
    for (i=0; i < MAX_STATUS_BTNS; i++) {
@@ -1821,7 +1861,11 @@ void Setup()
          FailToCreateWindowMessage("Setup()", NULL, TRUE);
       }
    }
+   InitTidget();
    if (!noChatWindow) {
+      if (!CreateChatSubWindows()) {
+         FailToCreateWindowMessage("Setup()", NULL, TRUE);
+      }
       cur_y += chatWindowH + (brdrW<<1);
    }
    if (!noStatusWindow) {
@@ -1844,7 +1888,6 @@ void Setup()
          FailToCreateWindowMessage("Setup()", NULL, TRUE);
       }
    }
-/* CreateChatWindows(); */
 
 #ifdef NOTR4MODE
    XSetNormalHints(mainDisplay, mainWindow, &sizehints);
@@ -1905,10 +1948,6 @@ void Setup()
 #endif
    XDefineCursor(mainDisplay, mainWindow, defaultCursor);
 
-   /* do not translate -- program constants */
-   protocolAtom = XInternAtom(mainDisplay, "WM_DELETE_WINDOW", False);
-   executeCmdByIDAtom = XInternAtom(mainDisplay, "TGIF_EXECUTE_CMD_BY_ID",
-         False);
    RegisterWM_DELETE_WINDOW(mainWindow);
 
 #ifdef MAPBEFORESELECT
@@ -1920,7 +1959,8 @@ void Setup()
             PropertyChangeMask);
    } else {
       XMapWindow(mainDisplay, titleWindow);
-      XSelectInput(mainDisplay, titleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput(mainDisplay, titleWindow,
+            ExposureMask | EnterWindowMask | ButtonPressMask);
 
       if (menubarWindow != None) XMapWindow(mainDisplay, menubarWindow);
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -1984,7 +2024,11 @@ void Setup()
 
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-/*       MapChatSubWindows(); */
+         XSelectInput(mainDisplay, chatWindow,
+               KeyPressMask | ButtonPressMask | ExposureMask |
+               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
+
+         MapChatSubWindows();
       }
       if (!noStatusWindow) {
          XMapWindow(mainDisplay, userRedrawWindow);
@@ -2014,7 +2058,8 @@ void Setup()
             PropertyChangeMask);
       XMapWindow(mainDisplay, drawWindow); 
    } else {
-      XSelectInput(mainDisplay, titleWindow, ExposureMask | EnterWindowMask);
+      XSelectInput(mainDisplay, titleWindow,
+            ExposureMask | EnterWindowMask | ButtonPressMask);
       XMapWindow(mainDisplay, titleWindow);
 
       XSelectInput(mainDisplay, menubarWindow, ExposureMask |
@@ -2081,7 +2126,10 @@ void Setup()
 
       if (!noChatWindow) {
          XMapWindow(mainDisplay, chatWindow);
-/*       MapChatSubWindows(); */
+         MapChatSubWindows();
+         XSelectInput(mainDisplay, chatWindow,
+               KeyPressMask | ButtonPressMask | ExposureMask |
+               StructureNotifyMask | EnterWindowMask | LeaveWindowMask);
       }
       if (!noStatusWindow) {
          XSelectInput(mainDisplay, userRedrawWindow, ButtonPressMask |
