@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/shape.c,v 4.18 2002/03/06 07:23:39 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/shape.c,v 4.20 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_SHAPE_C_
@@ -831,7 +831,7 @@ void GenerateShadow()
    struct ObjRec *obj_ptr;
    struct SelRec *tmp_top_sel=NULL, *tmp_bot_sel=NULL;
 
-   SelAllObj(FALSE);
+   SelAllObj(FALSE, FALSE);
 
    JustDupSelObj(&tmp_top_sel, &tmp_bot_sel);
    /*
@@ -970,7 +970,7 @@ struct AttrRec *GenerateShape()
     * Use the next two functions to set selLtX, etc. to be used
     *    in CreateGroupObj().
     */
-   SelAllObj(FALSE);
+   SelAllObj(FALSE, FALSE);
    RemoveAllSel();
 
    tmp_top_obj = topObj;
