@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.58 2002/01/05 02:48:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/special.c,v 4.62 2003/08/15 16:03:07 william Exp $
  */
 
 #define _INCLUDE_FROM_SPECIAL_C_
@@ -262,8 +262,8 @@ struct ObjRec *GetObjRepresentation(PathName, SymName, FileType)
    return obj_ptr;
 }
 
-unsigned int PlaceTopObj(ObjPtr)
-   struct ObjRec *ObjPtr;
+unsigned int PlaceTopObj(ObjPtr, savedTopObj, savedBotObj)
+   struct ObjRec *ObjPtr, *savedTopObj, *savedBotObj;
 {
    int x, y, grid_x, grid_y, dx, dy, placing=TRUE;
    int cursor_x, cursor_y, orig_x, orig_y, obj_ltx, obj_lty, obj_rbx, obj_rby;
@@ -324,8 +324,21 @@ unsigned int PlaceTopObj(ObjPtr)
       }
 
       if (input.type == Expose || input.type == VisibilityNotify) {
-         SelBox(drawWindow, revDefaultGC, obj_ltx+grid_dx, obj_lty+grid_dy,
-               obj_rbx+grid_dx, obj_rby+grid_dy);
+/*       SelBox(drawWindow, revDefaultGC, obj_ltx+grid_dx, obj_lty+grid_dy,
+               obj_rbx+grid_dx, obj_rby+grid_dy); */
+         if (savedTopObj != NULL && savedBotObj != NULL) {
+            struct ObjRec *saved_top_obj=topObj, *saved_bot_obj=botObj;
+
+            curPage->top = topObj = NULL;
+            curPage->bot = botObj = NULL;
+            curPage->top = topObj = savedTopObj;
+            curPage->bot = botObj = savedBotObj;
+            placingTopObj = FALSE;
+            RedrawDrawWindow(botObj);
+            placingTopObj = TRUE;
+            curPage->top = topObj = saved_top_obj;
+            curPage->bot = botObj = saved_bot_obj;
+         }
          ExposeEventHandler(&input, TRUE);
          SelBox(drawWindow, revDefaultGC, obj_ltx+grid_dx, obj_lty+grid_dy,
                obj_rbx+grid_dx, obj_rby+grid_dy);
@@ -437,8 +450,6 @@ void DeleteUnMakeIconicOnInstantiateText()
    DelAllSelObj();
 }
 
-void UnMakeIconic();
-
 void Instantiate()
 {
    char file_name[MAXPATHLENGTH], *rest=NULL, msg[MAXSTRING];
@@ -531,7 +542,7 @@ void Instantiate()
       AssignNewObjIds(obj_ptr);
       AddObj(NULL, topObj, obj_ptr);
       AdjObjBBox(obj_ptr);
-      PlaceTopObj(obj_ptr);
+      PlaceTopObj(obj_ptr, NULL, NULL);
 
       SelectTopObj();
       if (UnMakeIconicOnInstantiate(obj_ptr)) {
@@ -2588,7 +2599,7 @@ void MergeWithTable()
                SetStringStatus(gszMsgBox);
                XSync(mainDisplay, False);
             }
-            if (PlaceTopObj(obj_ptr) == Button3) {
+            if (PlaceTopObj(obj_ptr, NULL, NULL) == Button3) {
                Msg(TgLoadString(STID_OPERATION_CANCEL_BY_USER));
                ok = FALSE;
             }
