@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.10 2002/01/05 02:48:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.12 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_SPLINE_C_
@@ -864,6 +864,7 @@ void DoubleClosedSetupMatrix(NumPts, Vs)
 
 static int gaussIteration=0;
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG
 static
 void DebugClosedMatrix(NumPts)
@@ -909,6 +910,7 @@ void DebugClosedMatrix(NumPts)
    printf("|  (%+8.2f,%+8.2f)\n", mtxInfo.x[i], mtxInfo.y[i]);
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 void Gaussian(NumPts)
