@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 2001-2001, William Chia-Wei Cheng.
+ * Copyright (C) 2001-2003, William Chia-Wei Cheng.
  *
  * This file may be distributed under the terms of the Q Public License
  * as defined by Trolltech AS of Norway and appearing in the file
@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.10 2002/01/05 02:48:06 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/spline.c,v 4.12 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_SPLINE_C_
@@ -851,6 +851,7 @@ void DoubleClosedSetupMatrix(NumPts, Vs)
 
 static int gaussIteration=0;
 
+#ifdef NOT_DEFINED
 #ifdef _TGIF_DBG
 static
 void DebugClosedMatrix(NumPts)
@@ -896,6 +897,7 @@ void DebugClosedMatrix(NumPts)
    printf("|  (%+8.2f,%+8.2f)\n", mtxInfo.x[i], mtxInfo.y[i]);
 }
 #endif /* _TGIF_DBG */
+#endif /* NOT_DEFINED */
 
 static
 void Gaussian(NumPts)
