@@ -2,7 +2,7 @@
  * Author:      Daniel Ratton Figueiredo <ratton@land.ufrj.br>
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -29,7 +29,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.28 2002/01/05 02:48:07 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tangram2.c,v 4.30 2003/02/25 01:17:48 william Exp $
  */
 
 #define _INCLUDE_FROM_TANGRAM2_C_
@@ -351,6 +351,8 @@ void ExecStartSimulator(obj_ptr, orig_cmd)
         fprintf( stderr, "TGIF: Can't listen on %s port\n", service );
         return;
     }
+    fprintf(stdout,"TGIF: ready to receive data from the simulator.\n");
+    fflush(stdout);
     
     FD_ZERO( &soc_pool );
     FD_SET( s, &soc_pool );
