@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.15 2002/01/05 02:48:08 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/tgif_dbg.c,v 4.22 2003/02/25 01:17:49 william Exp $
  */
 #define _INCLUDE_FROM_TGIF_DBG_C_
 
@@ -121,6 +121,16 @@ void tgif_track(nBegin)
    gnCreateStringCount = gnFreeStringCount = 0;
 }
 
+void tgif_dbg_dump_count(msg)
+   char *msg;
+{
+   if (gpTraceFile != NULL) {
+      fprintf(gpTraceFile,
+            "%% %s Next gnAllocCount is %1d and next gnFreeCount is %1d\n",
+            msg, gnAllocCount+1, gnFreeCount+1);
+   }
+}
+
 /* --------------- malloc, realloc, free --------------- */
 
 int gnAllocTrigger=0, gnFreeTrigger=0;
@@ -175,7 +185,9 @@ void tgif_free(pVoid)
             gnFreeCount, (unsigned int)pVoid);
    }
    if (pVoid == NULL) {
+      /* debug, do not translate */
       fprintf(stderr, "ERROR: free(NULL) called!\n");
+      return;
    }
    free(pVoid);
 }
@@ -184,6 +196,7 @@ size_t tgif_strlen(psz)
    char *psz;
 {
    if (psz == NULL) {
+      /* debug, do not translate */
       fprintf(stderr, "ERROR: strlen(NULL) called!\n");
    }
    return strlen(psz);
