@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.46 2002/01/05 02:48:08 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/types.h,v 4.66 2003/08/13 20:15:16 william Exp $
  */
 
 #ifndef _TGIF_TYPES_H_
@@ -352,18 +352,47 @@ typedef struct VSelRec {
    struct VSelRec	* next, * prev;
 } * VSelRecPtr;
 
-typedef struct SubCmdRec {
-   union {
-      struct MoveSubCmdRec	* mv;
-   } detail;
-} * SubCmdRecPtr;
-
 typedef struct MoveSubCmdRec {
    int	dx, dy;
 } * MoveSubCmdRecPtr;
 
+typedef struct tagTextFormatInfo {
+   int color_index, font_style;
+   char color_str[40];
+} TextFormatInfo;
+
+#define CHAT_STATE_NORMAL     0
+#define CHAT_STATE_HELLO      1
+#define CHAT_STATE_DISCONNECT 2
+
+typedef struct ChatSubCmdRec {
+   int type;
+   TextFormatInfo tfi;
+   char nick_name[MAXSTRING];
+   char encoding[MAXSTRING];
+   char *buf;
+} * ChatSubCmdRecPtr;
+
+typedef struct SlideShowSubCmdRec {
+   int into_slideshow; /* TRUE if going into slideshow mode */
+} * SlideShowSubCmdRecPtr;
+
+typedef struct ClearAllSubCmdRec {
+   int page_style;
+   float print_mag;
+} * ClearAllSubCmdRecPtr;
+
+typedef struct SubCmdRec {
+   union {
+      struct MoveSubCmdRec	move;
+      struct ChatSubCmdRec	chat;
+      struct SlideShowSubCmdRec	slideshow;
+      struct ClearAllSubCmdRec	clearall;
+   } detail;
+} * SubCmdRecPtr;
+
 typedef struct CmdRec {
-   int			type, dx, dy, undone;
+   int			type, undone;
    int			include_tgif_obj, new_colormap, logical_clock;
    char			* sender_process_id;
    int			* pos_before, count_before, pos_before_has_ids;
@@ -380,6 +409,7 @@ typedef struct CmdRec {
    int			serialized;
    int			first_redo_after_deserialize;
    int			skipped;
+   struct SubCmdRec	* subcmd;
 } * CmdRecPtr;
 
 typedef struct PageRec {
@@ -477,9 +507,9 @@ typedef struct WinInfoRec {
    Window	window;
    int		mapped;
    int		raise;
-   int		(*ev_handler)();
-   void		(*expose_handler)();
-   void		(*cleanup)();
+   int		(*ev_handler)ARGS_DECL((XEvent*, struct WinInfoRec *));
+   void		(*expose_handler)ARGS_DECL((XEvent*, struct WinInfoRec *));
+   void		(*cleanup)ARGS_DECL((struct WinInfoRec *));
    void		*userdata;
 } * WinInfoPtr;
 
@@ -501,6 +531,13 @@ typedef struct MouseStatusStrRec {
    char	* l, * m, * r;
 } MouseStatusStrPtr;
 
+typedef struct tagMouseOverStatusInfo {
+   char btn_str[MAX_STATUS_BTNS][MAXSTRING+1];
+   char one_line_str[MAXSTRING+1];
+   int one_line_status;
+   struct tagMouseOverStatusInfo *next;
+} MouseOverStatusInfo;
+
 typedef struct VRec {
    int vtype;
    union {
@@ -538,6 +575,75 @@ typedef struct ObjListRec {
    int count;
 } ObjListInfo;
 
+typedef struct tagSimpleWinInfo {
+   int x, y, w, h;
+} SimpleWinInfo;
+
+typedef struct tagDirtyBBoxInfo {
+   int valid;
+   int force_redraw_all;
+   struct BBRec bbox;
+} DirtyBBoxInfo;
+
+typedef struct tagTextHighlightInfo {
+   /* all these values are screen offsets */
+   int start_x, start_min_y, start_max_y, start_baseline_y;
+   int end_x, end_min_y, end_max_y, end_baseline_y;
+   StrBlockInfo *start_str_block_ptr, *end_str_block_ptr;
+   int start_index, end_index;
+   int highlighting; /* used in PaintMiniLines() */
+   int mode; /* one of PAINT_* above */
+} TextHighlightInfo;
+
+typedef struct tagDrawTextInfo {
+   /* the following is from miniline.e */
+
+   TextHighlightInfo text_highlight_info;
+
+   int text_orig_x, text_orig_y, text_orig_baseline_y;
+   int text_abs_x, text_abs_y, text_abs_baseline_y;
+
+   int text_cur_x, text_cur_y, text_cur_baseline_y;
+   int text_cur_index;
+   int text_end_x, text_end_y, text_end_baseline_y;
+   int text_end_index;
+
+   int text_abs_min_lbearing, text_abs_max_rextra;
+   int text_w, text_h;
+
+   int text_highlight;
+
+   struct ObjRec *cur_text_obj;
+
+   struct BBRec cur_text_obbox, cur_text_bbox;
+
+   StrBlockInfo *cur_str_block;
+   StrBlockInfo *end_str_block;
+   MiniLineInfo *first_miniLine, *last_miniLine;
+
+   int esc_pressed;
+
+   int cur_text_outline_w, cur_text_outline_half_w;
+
+   /* the following is from text.e */
+
+   int text_drawn;
+   int cur_text_modified;
+   int text_vspace;
+
+   int text_just;
+   int text_cursor_shown;
+   int text_cursor_h;
+   int editing_text;
+   int cur_text_is_new;
+
+   int edit_text_size;
+
+   /* the following is from struct PushedFontRec */
+
+   struct PushedFontRec font_info;
+} DrawTextInfo;
+
 /* ------------------ imageproc stuff ------------------ */
 
 typedef void (ImageMapColorFunc)ARGS_DECL((int nColorIndex, XColor*));
@@ -753,9 +859,19 @@ typedef struct TgMenuInfoRec {
 
 /* ------------------ shortcut stuff ------------------ */
 
+#ifdef __DARWIN__
+#ifdef _DARWIN_MOD1_AS_META
+#define METAMASK (Mod1Mask)
+#else /* ~_DARWIN_MOD1_AS_META */
+#define METAMASK (Mod2Mask)
+#endif /* _DARWIN_MOD1_AS_META */
+#else /* ~__DARWIN__ */
+#define METAMASK (Mod1Mask)
+#endif /* __DARWIN__ */
+
 #define CNTRL_ONLY (ControlMask)
-#define META_ONLY (Mod1Mask)
-#define CNTRL_META (ControlMask|Mod1Mask)
+#define META_ONLY (METAMASK)
+#define CNTRL_META (ControlMask|METAMASK)
 
 typedef struct ShortCutRec {
    char         code;
