@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/util.c,v 4.13 2002/01/05 02:48:08 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/util.c,v 4.16 2003/02/25 01:17:49 william Exp $
  */
 
 #define _INCLUDE_FROM_UTIL_C_
@@ -418,7 +418,8 @@ int UtilCopyFile(pszFromFile, pszToFile)
          return TG_REMOTE_STATUS_WRITE;
       }
    } else {
-      if ((fd2=open(pszToFile, O_WRONLY|O_CREAT|O_TRUNC)) == (-1)) {
+      if ((fd2=open(pszToFile, O_WRONLY|O_CREAT|O_TRUNC, tmpFileMode)) ==
+            (-1)) {
          close(fd1);
          return TG_REMOTE_STATUS_WRITE;
       }
@@ -449,6 +450,24 @@ int UtilPathExists(pszPath)
    return (stat(pszPath, &stat_buf) == 0);
 }
 
+int UtilPathIsLink(char *pszPath)
+    /*
+     *      * if pszPath is not valid, return FALSE
+     *           * if pszPath is valid but not a link, return FALSE
+     *                * if pszPath is valid and is a link, return TRUE
+     *                     */
+{
+#ifndef _NO_LSTAT
+    struct stat stat_buf;
+
+    if (lstat(pszPath, &stat_buf) == 0 &&
+            ((stat_buf.st_mode & S_IFLNK) == S_IFLNK)) {
+        return TRUE;
+    }
+#endif /* _NO_LSTAT */
+    return FALSE;
+}
+
 int UtilIsDirectory(pszPath)
    char *pszPath;
 {
@@ -458,3 +477,116 @@ int UtilIsDirectory(pszPath)
       (stat_buf.st_mode & S_IFDIR) == S_IFDIR);
 }
 
+int UtilCreateDirHier(char *pszDir)
+{
+    char *psz=NULL;
+    int ok=TRUE;
+
+    if (*pszDir == '\0') return FALSE;
+
+    if (UtilPathExists(pszDir)) {
+        if (UtilPathIsLink(pszDir)) {
+            return TRUE;
+        }
+        if (UtilIsDirectory(pszDir)) {
+            return TRUE;
+        }
+        return FALSE;
+    }
+    if (*pszDir == DIR_SEP && pszDir[1] == '\0') {
+        return FALSE;
+    }
+    if ((psz=strrchr(pszDir, DIR_SEP)) == NULL) {
+        return FALSE;
+    }
+    *psz = '\0';
+    if (!UtilCreateDirHier(pszDir)) {
+        ok = FALSE;
+    }
+    *psz = DIR_SEP;
+
+    if (ok) {
+        if (mkdir(pszDir, 0755) != 0) {
+            return FALSE;
+        }
+    }
+    return ok;
+}
+
+char *UtilMkTempFile(char *buf, int buf_sz, char *pszDir, char *pszPrefix)
+{
+    int fd=(-2), fd_used=FALSE;
+
+#ifdef _TGIF_DBG /* debug, do not translate */
+    if (buf_sz <= sizeof(char*)) {
+       fprintf(stderr, "buf_sz <= sizeof(char*) in UtilMkTempFile()\n");
+    }
+#endif /* _TGIF_DBG */
+
+#ifndef _DONT_USE_MKTEMP
+    sprintf(buf, "%s%sXXXXXX", pszDir, pszPrefix);
+    mktemp(buf);
+#ifdef O_EXCL
+    fd_used = TRUE;
+    fd = open(buf, O_CREAT|O_EXCL|O_WRONLY);
+#endif /* O_EXCL */
+#else /* _DONT_USE_MKTEMP */
+    sprintf(buf, "%s%sXXXXXX", pszDir, pszPrefix);
+    fd_used = TRUE;
+    fd = mkstemp(buf);
+#endif /* ~_DONT_USE_MKTEMP */
+
+    if (fd_used) {
+        if (fd == (-1)) {
+            /* barf loudly that someone is trying to hack the box */
+            char gszMsgBox[MAXSTRING<<1];
+    
+            sprintf(gszMsgBox, TgLoadString(STID_RACE_IN_CREATE_TMP_FILE), buf);
+            if (PRTGIF) {
+               fprintf(stderr, "%s\n", gszMsgBox);
+            } else {
+               MsgBox(gszMsgBox, TOOL_NAME, INFO_MB);
+            }
+        } else {
+            close(fd);
+        }
+    }
+    unlink(buf);
+    
+    return buf;
+}
+
+#ifndef _NO_GETTIMEOFDAY
+static
+long ms_time(tv)
+   struct timeval *tv;
+{
+   return ((long)(tv->tv_usec / 1000.0)) + ((long)(tv->tv_sec * 1000));
+}
+#endif /* ~_NO_GETTIMEOFDAY */
+
+long UtilGetMilliSecTime(long *pl_sec, long *pl_msec)
+{
+#ifdef _NO_GETTIMEOFDAY
+   struct timeb start;
+#else /* ~_NO_GETTIMEOFDAY */
+   struct timeval start;
+   struct timezone zone;
+#endif /* _NO_GETTIMEOFDAY */
+   long ms_start_time=0L;
+
+#ifdef _NO_GETTIMEOFDAY
+   ftime(&start);
+   ms_start_time = ((long)start.time * 1000) + ((long)start.millitm);
+   if (pl_sec != NULL) *pl_sec = ((long)(start.time));
+   if (pl_msec != NULL) *pl_msec = ((long)(start.millitm));
+#else /* ~_NO_GETTIMEOFDAY */
+   gettimeofday(&start, &zone);
+   ms_start_time = ms_time(&start);
+   if (pl_sec != NULL) *pl_sec = ((long)(start.tv_sec));
+   if (pl_msec != NULL) *pl_msec = ((long)(start.tv_usec/1000));
+#endif /* _NO_GETTIMEOFDAY */
+
+   return ms_start_time;
+}  
+
