@@ -1,7 +1,7 @@
 /*
  * Author:      William Chia-Wei Cheng (bill.cheng@acm.org)
  *
- * Copyright (C) 1990-2002, William Chia-Wei Cheng.
+ * Copyright (C) 1990-2003, William Chia-Wei Cheng.
  *
  * Permission limited to the use, copy, display, distribute without
  * charging for a fee, and produce derivative works of "tgif" and
@@ -28,7 +28,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm/src/tgif/v4/RCS/version.c,v 4.15 2002/02/26 01:05:53 william Exp $
+ * @(#)$Header: /mm/src/tgif/v4/RCS/version.c,v 4.18 2003/08/15 03:56:20 william Exp $
  */
 
 #define _INCLUDE_FROM_VERSION_C_
@@ -41,15 +41,16 @@
 char *versionString="4.1";
 char *authorEmailString="bill.cheng@acm.org";
 #ifdef QPL_LICENSE
-char *copyrightString="Copyright (C) 2001-2002, William Chia-Wei Cheng";
+char *copyrightString="Copyright (C) 2001-2003, William Chia-Wei Cheng";
 #else /* ~QPL_LICENSE */
-char *copyrightString="Copyright (C) 1990-2002, William Chia-Wei Cheng";
+char *copyrightString="Copyright (C) 1990-2003, William Chia-Wei Cheng";
 #endif /* QPL_LICENSE */
 char *homePageURL="http://bourbon.usc.edu:8001/tgif/";
 char *oldMailingList="kragen-tgif@kragen.dnaco.net";
 char *mailingList="tgif4-announce@yahoogroups.com";
 char *joinMailingList="tgif4-announce-subscribe@yahoogroups.com";
-char *currentReleaseURL="http://bourbon.usc.edu:8001/tgif/current.html";
+char *mailingURL="http://bourbon.usc.edu:8001/tgif/faq/mailings.html";
+char *currentReleaseURL="http://bourbon.usc.edu:8001/tgif/download.html";
 char *hyperGraphicsURL="http://bourbon.usc.edu:8001/tgif/index.obj";
 
 char fullVersionString[MAXSTRING];
